package com.pancik.ciernypetrzlen.engine;

import com.badlogic.gdx.graphics.g2d.TextureRegion;
import com.badlogic.gdx.math.Vector2;
import com.pancik.ciernypetrzlen.DrawableData;

public class Animation {
    public static int DOWN = 0;
    public static int UP = 1;
    public static int LEFT = 2;
    public static int RIGHT = 3;
    public static float ERROR_FILTERING = 0.001f;
    
    public static TextureRegion[][] cutAnimation4way(String texture, int startX, int sizeX, int sizeY, int count) {
        TextureRegion[][] out = new TextureRegion[4][count];
        for(int i = 0; i < 4; i++) {
            out[i] = new TextureRegion(DrawableData.textureAtlas.findRegion(texture), startX, sizeY * i, sizeX * count, sizeY).split(sizeX, sizeY)[0];
        }
        for(int j = 0; j < out.length; j++) {
            for(int i = 0; i < out[j].length; i++) {
                out[j][i].setRegion(out[j][i].getU() + ERROR_FILTERING, out[j][i].getV() + ERROR_FILTERING, out[j][i].getU2() - ERROR_FILTERING, out[j][i].getV2() - ERROR_FILTERING);
            }
        }
        
        return out;
    }
    
    public static TextureRegion[][] cutAnimation3way(String texture, int startX, int sizeX, int sizeY, int count) {
        TextureRegion[][] out = new TextureRegion[4][count];
        for(int i = 0; i < 3; i++) {
            out[i] = new TextureRegion(DrawableData.textureAtlas.findRegion(texture), startX, sizeY * i, sizeX * count, sizeY).split(sizeX, sizeY)[0];
        }
        
        for(int j = 0; j < out.length - 1; j++) {
            for(int i = 0; i < out[j].length; i++) {
                out[j][i].setRegion(out[j][i].getU() + ERROR_FILTERING, out[j][i].getV() + ERROR_FILTERING, out[j][i].getU2() - ERROR_FILTERING, out[j][i].getV2() - ERROR_FILTERING);
            }
        }
        
        out[RIGHT] = new TextureRegion[out[LEFT].length];
        for(int i = 0; i < out[RIGHT].length; i++) {
            out[RIGHT][i] = new TextureRegion(out[LEFT][i]);
            out[RIGHT][i].flip(true, false);
        }

        return out;
    }
    
    public static int getDirectionIndex(Vector2 direction) {
        if(Math.abs(direction.x) > Math.abs(direction.y)) {
            if(direction.x > 0) {
                return RIGHT;
            } else {
                return LEFT;
            }
        } else {
            if(direction.y > 0) {
                return DOWN;
            } else {
                return UP;
            }
        }
    }
    
    public static TextureRegion getTexture(TextureRegion[][] textures, Vector2 direction, int step) {
        int index = getDirectionIndex(direction);
        return textures[index][step % textures[index].length];
    }
}
