package com.pancik.ciernypetrzlen.engine;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.audio.Music;
import com.badlogic.gdx.audio.Sound;
import com.badlogic.gdx.math.MathUtils;
import java.util.HashMap;
import java.util.Map;

public class SoundData {
    public static boolean SOUND_IS_ON = true;
    private static boolean isLoaded = false;
    
    public static Map<String, Sound> sounds;
    public static Map<String, Music> musics;
    
    public static void loadData() {
        if(!isLoaded) {
            sounds = new HashMap<String, Sound>();
            musics = new HashMap<String, Music>();

            musics.put("menu-theme", Gdx.audio.newMusic(Gdx.files.internal("data/sound/intro-loop.ogg")));
            musics.get("menu-theme").setVolume(0.3f);
            musics.put("game-theme", Gdx.audio.newMusic(Gdx.files.internal("data/sound/main-loop.ogg")));
            musics.get("game-theme").setVolume(0.3f);
            
            sounds.put("eat", Gdx.audio.newSound(Gdx.files.internal("data/sound/eat.mp3")));
            sounds.put("bones", Gdx.audio.newSound(Gdx.files.internal("data/sound/bones.mp3")));
            sounds.put("bow", Gdx.audio.newSound(Gdx.files.internal("data/sound/bow.mp3")));
            sounds.put("debris-smash", Gdx.audio.newSound(Gdx.files.internal("data/sound/debris-smash.mp3")));
            sounds.put("item-inventory", Gdx.audio.newSound(Gdx.files.internal("data/sound/item-inventory.mp3")));
            sounds.put("potion", Gdx.audio.newSound(Gdx.files.internal("data/sound/potion.mp3")));
            sounds.put("spell", Gdx.audio.newSound(Gdx.files.internal("data/sound/spell.mp3")));
            sounds.put("sword", Gdx.audio.newSound(Gdx.files.internal("data/sound/sword.mp3")));
            sounds.put("door", Gdx.audio.newSound(Gdx.files.internal("data/sound/door.mp3")));
            sounds.put("explosion", Gdx.audio.newSound(Gdx.files.internal("data/sound/explosion.mp3")));
            sounds.put("roar", Gdx.audio.newSound(Gdx.files.internal("data/sound/roar.mp3")));
            sounds.put("stomp", Gdx.audio.newSound(Gdx.files.internal("data/sound/stomp.mp3")));
            sounds.put("throw-axe", Gdx.audio.newSound(Gdx.files.internal("data/sound/throw-axe.mp3")));
            sounds.put("level-up", Gdx.audio.newSound(Gdx.files.internal("data/sound/level-up.mp3")));
            sounds.put("transmute", Gdx.audio.newSound(Gdx.files.internal("data/sound/transmute.mp3")));
            
            isLoaded = true;
        }
    }
    
    public static void dispose() {
        if(isLoaded) {
            for(Map.Entry<String, Sound> sound : sounds.entrySet()) {
                sound.getValue().dispose();
            }

            for(Map.Entry<String, Music> music : musics.entrySet()) {
                music.getValue().dispose();
            }
            
            isLoaded = false;
        }
    }
    
    public static void playSound(String name, float volume) {
        if(SOUND_IS_ON) {
            SoundData.sounds.get(name).play(volume);
        }
    }
    
    public static void playMusicAsSound(String name) {
        if(SOUND_IS_ON) {
            if(!SoundData.musics.get(name).isPlaying()) {
                SoundData.musics.get(name).play();
            }
        }
    }
    
    public static void playRandomSound(float volume, String ... names) {
        playSound(names[MathUtils.random(names.length-1)], volume);
    }
    
    public static void playMusic(String name) {
        SoundData.musics.get(name).play();
    }
    
    public static void playMenuMusic() {
        stopEngineMusic();
        if(!SoundData.musics.get("menu-theme").isLooping()) {
            SoundData.musics.get("menu-theme").setLooping(true);
        }
        if(!SoundData.musics.get("menu-theme").isPlaying()) {
            SoundData.playMusic("menu-theme");
        }
    }
    
    public static void stopMenuMusic() {
        SoundData.musics.get("menu-theme").stop();
    }
    
    public static void playEngineMusic() {
        stopMenuMusic();
        if(!SoundData.musics.get("game-theme").isLooping()) {
            SoundData.musics.get("game-theme").setLooping(true);
        }
        if(!SoundData.musics.get("game-theme").isPlaying()) {
            SoundData.playMusic("game-theme");
        }
    }
    
    public static void stopEngineMusic() {
        SoundData.musics.get("game-theme").stop();
    }
}
