package com.pancik.ciernypetrzlen.engine.component;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.graphics.GL10;
import com.badlogic.gdx.graphics.Mesh;
import com.badlogic.gdx.graphics.glutils.ShaderProgram;
import com.badlogic.gdx.math.Matrix4;
import com.badlogic.gdx.math.Vector3;
import com.badlogic.gdx.utils.Disposable;
import com.pancik.ciernypetrzlen.engine.Engine;
import com.pancik.ciernypetrzlen.engine.component.level.Level;
import com.pancik.ciernypetrzlen.engine.component.level.LevelMeshes;

public class LevelManager implements Disposable {
    private final Engine.Controls engineControls;
    private final ShaderProgram shader;
    
    private LevelMeshes levelMeshes;
    
    public LevelManager(Level level, Engine.Controls engineControls) {
        this.engineControls = engineControls;
        
        this.shader = new ShaderProgram(Gdx.files.internal("data/shaders/pointlight-tex-vs.glsl"), Gdx.files.internal("data/shaders/light-tex-fs.glsl"));
        if (!this.shader.isCompiled()) {
            throw new RuntimeException(this.shader.getLog());
        }
        
        this.levelMeshes = new LevelMeshes(level);
    }
    
    public void tick() {
    }
    
    public void render() {
        beginShader(null);
        Mesh[][] meshes = levelMeshes.getMeshes();
        for (int z = 0; z < meshes[0].length; z++) {
            for (int x = 0; x < meshes.length; x++) {
                meshes[x][z].render(this.shader, GL10.GL_TRIANGLES);
            }
        }
        endShader();
    }

    float testAngle = 0.0f;
    
    public void beginShader(Matrix4 transformation) {
        levelMeshes.getTexture().bind();
        Vector3 lookAt = engineControls.getCameraLookAt();
        shader.begin();
        if(transformation == null) {
            shader.setUniformMatrix("u_projTrans", engineControls.getCamera().combined);
        } else {
            shader.setUniformMatrix("u_projTrans", engineControls.getCamera().combined.cpy().mul(transformation));
        }
        shader.setUniformf("u_lightIntensity", engineControls.getLightIntensity());
        shader.setUniformf("u_lightPos", lookAt.add(0.0f, 4.0f, 0.0f));
        shader.setUniformf("u_lightColor", engineControls.getLightColor());
    }
    
    public void endShader() {
        shader.end();
    }
    
    public ShaderProgram getShader() {
        return shader;
    }
    
    public LevelMeshes getLevelMeshes() {
        return levelMeshes;
    }
    
    @Override
    public void dispose() {
        levelMeshes.dispose();
    }
}
