package com.pancik.ciernypetrzlen.engine.component.entity;

import com.badlogic.gdx.graphics.g3d.decals.Decal;
import com.badlogic.gdx.math.Intersector;
import com.badlogic.gdx.math.Vector2;
import com.badlogic.gdx.math.Vector3;
import com.badlogic.gdx.math.collision.Ray;
import com.pancik.ciernypetrzlen.engine.Engine;
import static com.pancik.ciernypetrzlen.engine.component.entity.Unit.tmp3;
import com.pancik.ciernypetrzlen.engine.component.entity.loot.Loot;
import com.pancik.ciernypetrzlen.engine.component.spell.Buff;
import com.pancik.ciernypetrzlen.engine.player.inventory.Item;
import java.util.HashMap;
import java.util.Map;

public abstract class Attackable extends Entity implements Interactable {
    protected static Vector2 tmp2 = new Vector2();
    protected static Vector3[] tmp3 = {new Vector3(), new Vector3(), new Vector3(), new Vector3()};
    public static final int DEBRIS_TEAM = 0;
    
    protected Vector2 position;
    protected Vector2 size;
    protected int team;
    protected int health;
    protected int maxHealth;
    protected HashMap<Class<? extends Buff>, Buff> buffs;
    
    protected Decal decal;
    protected boolean highlighted = false;
    protected Loot loot;

    public Attackable(Vector2 pos, Vector2 size, int team, int health, Loot loot, Engine.Controls engineControls) {
        super(engineControls);
        this.position = pos;
        this.size = size;
        this.team = team;
        this.maxHealth = health;
        this.health = maxHealth;
        this.loot = loot;
        this.buffs = new HashMap<Class<? extends Buff>, Buff>();
    }

    @Override
    public void tick() {
        for(Map.Entry<Class<? extends Buff>, Buff> entry : buffs.entrySet()) {
            if(entry.getValue() != null) {
                entry.getValue().tick();
                if(entry.getValue().remove()) {
                    entry.setValue(null);
                }
            }
        }
    }
    
    @Override
    public Vector2 getPosition() {
        return position;
    }

    public Vector2 getSize() {
        return size;
    }
    
    public int getTeam() {
        return team;
    }
    
    public int getHit(int damage, Attackable from) {
        boolean alive = !isDead();
        health -= damage;
        for(Buff b : buffs.values()) {
            if(b != null) {
                b.damageTaken(damage);
            }
        }
        if(alive && isDead() && from instanceof Hero) {
            ((Hero)from).rewardExperience(getRewardExperience());
        }
        return damage;
    }

    @Override
    public void onRemove() {
        loot.onDeath(engineControls, position);
    }

    public boolean isDead() {
        return (health <= 0);
    }
    
    @Override
    public void highlight() {
        highlighted = true;
    }
    
    @Override
    public void dehighlight() {
        highlighted = false;
    }

    @Override
    public boolean isHighlighted() {
        return highlighted;
    }

    @Override
    public boolean getClosestIntersection(Ray ray, Vector3 out) {
        decal.setPosition(position.x, decal.getY(), position.y);
        boolean found;
        float[] res = decal.getVertices();
        
        for(int i = 0; i < 3; i++) {
            tmp3[i].set(res[(i * 6)], res[(i * 6) + 1], res[(i * 6) + 2]);
        }
        found = Intersector.intersectRayTriangle(ray, tmp3[0], tmp3[1], tmp3[2], out);
        
        for(int i = 1; i < 4; i++) {
            tmp3[i - 1].set(res[(i * 6)], res[(i * 6) + 1], res[(i * 6) + 2]);
        }
        if(!found) {
            found = Intersector.intersectRayTriangle(ray, tmp3[0], tmp3[1], tmp3[2], out);
        } else {
            if(Intersector.intersectRayTriangle(ray, tmp3[0], tmp3[1], tmp3[2], tmp3[3])) {
                if(tmp3[3].dst(ray.origin) < out.dst(ray.origin)) {
                    out.set(tmp3[3]);
                }
            }
        }
        return found;
    }

    @Override
    public float getPercentualHealth() {
        return (float)health/(float)maxHealth;
    }
    
    public int getRewardExperience() {
        return 0;
    }
    
    public void addBuff(Buff buff) {
        if(buffs.containsKey(buff.getClass())) {
            if(buffs.get(buff.getClass()) == null) {
                buffs.put(buff.getClass(), buff);
            } else {
                buffs.get(buff.getClass()).refresh();
            }
        } else {
            buffs.put(buff.getClass(), buff);
        }
    }
    
    public HashMap<Class<? extends Buff>, Buff> getBuffs() {
        return buffs;
    }
    
    public float getPercentualModifier(Buff.Stat stat) {
        float out = 1.0f;
        for(Buff b : buffs.values()) {
            if(b != null) {
                out *= b.getPercentualModifier(stat);
            }
        }
        return out;
    }
    
    public float getScalarModifier(Buff.Stat stat) {
        float out = 0.0f;
        for(Buff b : buffs.values()) {
            if(b != null) {
                out += b.getScalarModifier(stat);
            }
        }
        return out;
    }
    
    public void changeHealth(int delta) {
        if(!isDead()) {
            health += delta;
            if(health > maxHealth) {
                health = maxHealth;
            }
            if(health < 0) {
                health = 0;
            }
        }
    }
    
    public int getHealth() {
        return health;
    }
    
    public void addItemToLoot(Item item) {
        loot.addItem(item);
    }
}
