package com.pancik.ciernypetrzlen.engine.component.entity;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.graphics.Color;
import com.badlogic.gdx.graphics.GL10;
import com.badlogic.gdx.graphics.Mesh;
import com.badlogic.gdx.graphics.VertexAttribute;
import com.badlogic.gdx.graphics.g2d.TextureRegion;
import com.badlogic.gdx.graphics.g3d.decals.Decal;
import com.badlogic.gdx.graphics.glutils.ShapeRenderer;
import com.badlogic.gdx.math.Intersector;
import com.badlogic.gdx.math.Vector2;
import com.badlogic.gdx.math.Vector3;
import com.badlogic.gdx.math.collision.BoundingBox;
import com.badlogic.gdx.math.collision.Ray;
import com.badlogic.gdx.utils.Disposable;
import com.badlogic.gdx.utils.FloatArray;
import com.pancik.ciernypetrzlen.DrawableData;
import com.pancik.ciernypetrzlen.engine.Engine;
import com.pancik.ciernypetrzlen.engine.SoundData;
import com.pancik.ciernypetrzlen.engine.component.LevelManager;
import com.pancik.ciernypetrzlen.engine.player.YellText;
import com.pancik.ciernypetrzlen.engine.player.inventory.Item;

public class Door extends Entity implements Interactable, Disposable {
    protected static final float WIDTH = 0.25f;
    protected static final float HEIGHT = 1.0f;
    protected Vector2 position;
    protected Mesh[] meshes = new Mesh[2];
    protected BoundingBox[] boundingBoxes = new BoundingBox[2];
    protected boolean horizontal = true;
    protected Vector2 leftTop;
    protected Vector2 leftBottom;
    protected Vector2 rightTop;
    protected Vector2 rightBottom;
    protected boolean highlighted = false;
    protected boolean closed = true;
    protected String openYellText;
    protected boolean locked;
    protected boolean picklockable;
    protected boolean picklockWarningShowed = false;
    protected String keyItemClassName;
    protected YellText tmpText;

    public Door(Vector2 position, Engine.Controls engineControls, String openYellText, boolean locked, boolean picklockable, String keyItemClassName) {
        super(engineControls);
        this.position = position;
        this.openYellText = openYellText;
        this.locked = locked;
        this.picklockable = picklockable;
        this.keyItemClassName = keyItemClassName;
    }

    @Override
    public void create() {
        float gridX = (int)position.x;
        float gridY = (int)position.y;
        float offsetX = 0.5f;
        float offsetY = 0.0f;
        if(!engineControls.getCollisionMap().isWalkable(gridX, gridY + 1.0f) || !engineControls.getCollisionMap().isWalkable(gridX, gridY - 1.0f)) {
            horizontal = false;
            offsetX = 0.0f;
            offsetY = 0.5f;
        }
        
        TextureRegion[] t = null;
        if (!locked) {
            t = engineControls.getLevelManager().getLevelMeshes().getNormalDoorTiles();
        } else {
            if (picklockable) {
                t = engineControls.getLevelManager().getLevelMeshes().getPicklockDoorTiles();
            } else {
                t = engineControls.getLevelManager().getLevelMeshes().getKeyDoorTiles();
            }
        }
        float[] normalsHorizontal = {0.0f, -1.0f, 0.0f, 1.0f, 1.0f, 0.0f, -1.0f, 0.0f,};
        float[] normalsVertical = {-1.0f, 0.0f, 1.0f, 0.0f, 0.0f, -1.0f, 0.0f, 1.0f,};

        for(int i = 0; i < 2; i++) {
            FloatArray v = new FloatArray(8 * 8);
            
            float[] normals = (i == 0) ? normalsHorizontal : normalsVertical;
            if(i == 0) {
                leftTop = new Vector2(position.x - 0.5f, position.y - WIDTH/2);
                leftBottom = new Vector2(position.x - 0.5f, position.y + WIDTH/2);
                rightTop = new Vector2(position.x + 0.5f, position.y - WIDTH/2);
                rightBottom = new Vector2(position.x + 0.5f, position.y + WIDTH/2);
            } else {
                rightTop = new Vector2(position.x - WIDTH/2, position.y - 0.5f);
                leftTop = new Vector2(position.x - WIDTH/2, position.y + 0.5f);
                rightBottom = new Vector2(position.x + WIDTH/2, position.y - 0.5f);
                leftBottom = new Vector2(position.x + WIDTH/2, position.y + 0.5f);
            }
            if((i == 1 && horizontal) || (i == 0 && !horizontal)) {
                leftTop.add(offsetX, offsetY);
                leftBottom.add(offsetX, offsetY);
                rightTop.add(offsetX, offsetY);
                rightBottom.add(offsetX, offsetY);
                if(horizontal) {
                    leftTop.add(0.0f, -0.01f);
                    leftBottom.add(0.0f, -0.01f);
                    rightTop.add(0.0f, 0.01f);
                    rightBottom.add(0.0f, 0.01f);
                } else {
                    leftTop.add(0.01f, 0.0f);
                    leftBottom.add(0.01f, 0.0f);
                    rightTop.add(-0.01f, 0.0f);
                    rightBottom.add(-0.01f, 0.0f);
                }
            }

            float height = 0.99f;
            int index = 0;
            //LT+RT
            v.add(leftTop.x); v.add(0.0f); v.add(leftTop.y); v.add(normals[0]); v.add(0.0f); v.add(normals[1]); v.add(t[index].getU()); v.add(t[index].getV());
            v.add(rightTop.x); v.add(0.0f); v.add(rightTop.y); v.add(normals[0]); v.add(0.0f); v.add(normals[1]); v.add(t[index].getU2()); v.add(t[index].getV());
            v.add(rightTop.x); v.add(height); v.add(rightTop.y); v.add(normals[0]); v.add(0.0f); v.add(normals[1]); v.add(t[index].getU2()); v.add(t[index].getV2());

            v.add(rightTop.x); v.add(height); v.add(rightTop.y); v.add(normals[0]); v.add(0.0f); v.add(normals[1]); v.add(t[index].getU2()); v.add(t[index].getV2());
            v.add(leftTop.x); v.add(height); v.add(leftTop.y); v.add(normals[0]); v.add(0.0f); v.add(normals[1]); v.add(t[index].getU()); v.add(t[index].getV2());
            v.add(leftTop.x); v.add(0.0f); v.add(leftTop.y); v.add(normals[0]); v.add(0.0f); v.add(normals[1]); v.add(t[index].getU()); v.add(t[index].getV());

            //LB+RB
            v.add(leftBottom.x); v.add(0.0f); v.add(leftBottom.y); v.add(normals[2]); v.add(0.0f); v.add(normals[3]); v.add(t[index].getU()); v.add(t[index].getV());
            v.add(rightBottom.x); v.add(0.0f); v.add(rightBottom.y); v.add(normals[2]); v.add(0.0f); v.add(normals[3]); v.add(t[index].getU2()); v.add(t[index].getV());
            v.add(rightBottom.x); v.add(height); v.add(rightBottom.y); v.add(normals[2]); v.add(0.0f); v.add(normals[3]); v.add(t[index].getU2()); v.add(t[index].getV2());

            v.add(rightBottom.x); v.add(height); v.add(rightBottom.y); v.add(normals[2]); v.add(0.0f); v.add(normals[3]); v.add(t[index].getU2()); v.add(t[index].getV2());
            v.add(leftBottom.x); v.add(height); v.add(leftBottom.y); v.add(normals[2]); v.add(0.0f); v.add(normals[3]); v.add(t[index].getU()); v.add(t[index].getV2());
            v.add(leftBottom.x); v.add(0.0f); v.add(leftBottom.y); v.add(normals[2]); v.add(0.0f); v.add(normals[3]); v.add(t[index].getU()); v.add(t[index].getV());

            index = 1;
            //RT+RB
            v.add(rightTop.x); v.add(0.0f); v.add(rightTop.y); v.add(normals[4]); v.add(0.0f); v.add(normals[5]); v.add(t[index].getU()); v.add(t[index].getV());
            v.add(rightBottom.x); v.add(0.0f); v.add(rightBottom.y); v.add(normals[4]); v.add(0.0f); v.add(normals[5]); v.add(t[index].getU2()); v.add(t[index].getV());
            v.add(rightBottom.x); v.add(height); v.add(rightBottom.y); v.add(normals[4]); v.add(0.0f); v.add(normals[5]); v.add(t[index].getU2()); v.add(t[index].getV2());

            v.add(rightBottom.x); v.add(height); v.add(rightBottom.y); v.add(normals[4]); v.add(0.0f); v.add(normals[5]); v.add(t[index].getU2()); v.add(t[index].getV2());
            v.add(rightTop.x); v.add(height); v.add(rightTop.y); v.add(normals[4]); v.add(0.0f); v.add(normals[5]); v.add(t[index].getU()); v.add(t[index].getV2());
            v.add(rightTop.x); v.add(0.0f); v.add(rightTop.y); v.add(normals[4]); v.add(0.0f); v.add(normals[5]); v.add(t[index].getU()); v.add(t[index].getV());

            //LT+LB
            v.add(leftTop.x); v.add(0.0f); v.add(leftTop.y); v.add(normals[6]); v.add(0.0f); v.add(normals[7]); v.add(t[index].getU()); v.add(t[index].getV());
            v.add(leftBottom.x); v.add(0.0f); v.add(leftBottom.y); v.add(normals[6]); v.add(0.0f); v.add(normals[7]); v.add(t[index].getU2()); v.add(t[index].getV());
            v.add(leftBottom.x); v.add(height); v.add(leftBottom.y); v.add(normals[6]); v.add(0.0f); v.add(normals[7]); v.add(t[index].getU2()); v.add(t[index].getV2());

            v.add(leftBottom.x); v.add(height); v.add(leftBottom.y); v.add(normals[6]); v.add(0.0f); v.add(normals[7]); v.add(t[index].getU2()); v.add(t[index].getV2());
            v.add(leftTop.x); v.add(height); v.add(leftTop.y); v.add(normals[6]); v.add(0.0f); v.add(normals[7]); v.add(t[index].getU()); v.add(t[index].getV2());
            v.add(leftTop.x); v.add(0.0f); v.add(leftTop.y); v.add(normals[6]); v.add(0.0f); v.add(normals[7]); v.add(t[index].getU()); v.add(t[index].getV());

            //TOP
            v.add(leftBottom.x); v.add(height); v.add(leftBottom.y); v.add(0.0f); v.add(1.0f); v.add(0.0f); v.add(t[index].getU2()); v.add(t[index].getV2());
            v.add(rightBottom.x); v.add(height); v.add(rightBottom.y); v.add(0.0f); v.add(1.0f); v.add(0.0f); v.add(t[index].getU2()); v.add(t[index].getV());
            v.add(leftTop.x); v.add(height); v.add(leftTop.y); v.add(0.0f); v.add(1.0f); v.add(0.0f); v.add(t[index].getU()); v.add(t[index].getV2());

            v.add(leftTop.x); v.add(height); v.add(leftTop.y); v.add(0.0f); v.add(1.0f); v.add(0.0f); v.add(t[index].getU()); v.add(t[index].getV2());
            v.add(rightTop.x); v.add(height); v.add(rightTop.y); v.add(0.0f); v.add(1.0f); v.add(0.0f); v.add(t[index].getU()); v.add(t[index].getV());
            v.add(rightBottom.x); v.add(height); v.add(rightBottom.y); v.add(0.0f); v.add(1.0f); v.add(0.0f); v.add(t[index].getU2()); v.add(t[index].getV());

            v.shrink();
            meshes[i] = new Mesh(true, v.size / 5, 0, new VertexAttribute[]{VertexAttribute.Position(), VertexAttribute.Normal(), VertexAttribute.TexCoords(0)});
            meshes[i].setVertices(v.items);
            boundingBoxes[i] = meshes[i].calculateBoundingBox();
        }
        
        engineControls.getCollisionMap().setWalkable(gridX, gridY, false);
    }
    
    @Override
    public void tick() {
    }
    
    @Override
    public void render() {
        LevelManager manager = engineControls.getLevelManager();
        manager.beginShader(null);
        meshes[getCurrentMesh()].render(manager.getShader(), GL10.GL_TRIANGLES);
        manager.endShader();
        
        if(highlighted) {
            Gdx.gl.glDisable(GL10.GL_DEPTH_TEST);
            DrawableData.shapeRenderer.begin(ShapeRenderer.ShapeType.Line);
            DrawableData.shapeRenderer.setColor(0.0f, 0.0f, 1.0f, 1.0f);
            boolean renderHorizontal = getCurrentMesh() == 0 ? true : false;
            if(renderHorizontal) {
                DrawableData.shapeRenderer.box(position.x - 0.5f, position.y - WIDTH/2, 0.0f, 1.0f, WIDTH, 1.0f);
            } else {
                DrawableData.shapeRenderer.box(position.x - WIDTH/2, position.y - 0.5f, 0.0f, WIDTH, 1.0f, 1.0f);
            }
            DrawableData.shapeRenderer.end();
            Gdx.gl.glEnable(GL10.GL_DEPTH_TEST);
        }
    }

    @Override
    public void debugRender() {
    }

    @Override
    public Decal getDecal() {
        return null;
    }

    @Override
    public boolean remove() {
        return false;
    }

    @Override
    public void onRemove() {
    }

    @Override
    public Vector2 getPosition() {
        return position;
    }

    @Override
    public void highlight() {
        highlighted = true;
    }

    @Override
    public void dehighlight() {
        highlighted = false;
    }

    @Override
    public boolean isHighlighted() {
        return highlighted;
    }

    @Override
    public boolean getClosestIntersection(Ray ray, Vector3 out) {
        if(closed) {
            return Intersector.intersectRayBounds(ray, boundingBoxes[getCurrentMesh()], out);
        } else {
            return false;
        }
    }

    @Override
    public void interact() {
        boolean doOpen = false;
        
        if(!locked) {
            doOpen = true;
        } else {
            if(picklockable) {
                if(picklockWarningShowed) {
                    Item picklock = engineControls.getPlayer().getInventory().getItem("Picklock");
                    if(picklock == null) {
                        picklockWarningShowed = false;
                    } else {
                        doOpen = true;
                        engineControls.getPlayer().getInventory().removeItem(picklock);
                        if(tmpText != null) {
                            tmpText.destroy();
                            tmpText = null;
                        }
                    }
                } else {
                    Vector3 yellPosition = new Vector3();
                    yellPosition.set(position.x, 0.0f, position.y - 1.0f);
                    if(tmpText != null) {
                        tmpText.destroy();
                    }
                    tmpText = new YellText(engineControls,
                            yellPosition,
                            "You will consume one picklock in inventory to open this door. Click again to do so.",
                            Color.WHITE);
                    engineControls.getPlayer().getTextEffectManager().addTextEffect(tmpText);
                    picklockWarningShowed = true;
                }
            } else {
                Item key = engineControls.getPlayer().getInventory().getItem(keyItemClassName);
                if(key == null) {
                    try {
                        Class clazz = Class.forName("com.pancik.ciernypetrzlen.engine.player.inventory." + keyItemClassName);
                        Vector3 yellPosition = new Vector3();
                        yellPosition.set(position.x, 0.0f, position.y - 1.0f);
                        if(tmpText != null) {
                            tmpText.destroy();
                        }
                        tmpText = new YellText(engineControls,
                                yellPosition,
                                "You need to have " + ((Item)clazz.newInstance()).getName() + " in order to open this door.",
                                Color.WHITE);
                        engineControls.getPlayer().getTextEffectManager().addTextEffect(tmpText);
                    } catch (Exception e) {
                        doOpen = true;
                        e.printStackTrace(System.err);
                    }
                } else {
                    engineControls.getPlayer().getInventory().removeItem(key);
                    doOpen = true;
                    if(tmpText != null) {
                        tmpText.destroy();
                        tmpText = null;
                    }
                }
            }
        }
        
        if(doOpen) {
            closed = false;
            engineControls.getCollisionMap().setWalkable((int) position.x, (int) position.y, true);
        
            if(openYellText != null) {
                Vector3 yellPosition = new Vector3();
                yellPosition.set(position.x, 0.0f, position.y - 1.0f);
                engineControls.getPlayer().getTextEffectManager().addTextEffect(new YellText(engineControls,
                        yellPosition,
                        openYellText,
                        Color.WHITE));
            }
            
            SoundData.playSound("door", 1.0f);
        }
    }

    @Override
    public void dispose() {
        for(Mesh m : meshes) {
            m.dispose();
        }
    }
    
    protected int getCurrentMesh() {
        if(horizontal) {
            if(closed) {
                return 0;
            }
        } else {
            if(!closed) {
                return 0;
            }
        }
        return 1;
    }

    @Override
    public String getName() {
        return "Door";
    }

    @Override
    public float getPercentualHealth() {
        return 0.0f;
    }
}
