package com.pancik.ciernypetrzlen.engine.component.entity;

import com.badlogic.gdx.graphics.g2d.TextureRegion;
import com.badlogic.gdx.graphics.g3d.decals.Decal;
import com.badlogic.gdx.math.Vector2;
import com.pancik.ciernypetrzlen.DrawableData;
import com.pancik.ciernypetrzlen.engine.Engine;
import com.pancik.ciernypetrzlen.engine.component.spell.BurningBuff;

public class Fire extends Entity {
    public static final int LIFE_LENGTH = 600;
    public static final int DAMAGE = 10;
    
    protected Vector2 position;
    protected Attackable target;
    protected float diameter = 1.0f;
    protected boolean blue;
    protected TextureRegion[] texture;
    protected Decal decal;
    protected int animationStep = 0;
    protected int life = 0;
    protected int lifeLength;
    protected int damage;

    public Fire(Vector2 position, Attackable target, int damage, boolean blue, Engine.Controls engineControls) {
        this(position, target, damage, blue, LIFE_LENGTH, engineControls);
    }
    
    public Fire(Vector2 position, Attackable target, int damage, boolean blue, int lifeLength, Engine.Controls engineControls) {
        super(engineControls);
        this.position = position;
        this.target = target;
        this.damage = damage;
        this.blue = blue;
        this.lifeLength = lifeLength;
        this.texture = DrawableData.textureAtlas.findRegion(blue ? "entity-fire-blue" : "entity-fire-red").split(16, 16)[0];
    }
    
    @Override
    public void create() {
        decal = engineControls.createDecal(position, diameter, diameter, texture[0]);
    }

    @Override
    public void tick() {
        animationStep++;
        life++;
        
        if(position.dst(target.getPosition()) < diameter) {
            target.addBuff(new BurningBuff(target, damage, blue));
        }
    }

    @Override
    public Decal getDecal() {
        decal.setTextureRegion(texture[(animationStep / 5) % 3]);
        decal.setPosition(position.x, decal.getY(), position.y);
        return decal;
    }

    @Override
    public boolean remove() {
        return (life >= lifeLength);
    }

    @Override
    public void onRemove() {
    }
}
