package com.pancik.ciernypetrzlen.engine.component.entity;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.graphics.GL10;
import com.badlogic.gdx.graphics.g2d.TextureRegion;
import com.badlogic.gdx.graphics.g3d.decals.Decal;
import com.badlogic.gdx.graphics.glutils.ShapeRenderer;
import com.badlogic.gdx.math.Vector2;
import com.pancik.ciernypetrzlen.DrawableData;
import com.pancik.ciernypetrzlen.engine.Animation;
import com.pancik.ciernypetrzlen.engine.Engine;
import com.pancik.ciernypetrzlen.engine.SoundData;
import com.pancik.ciernypetrzlen.engine.component.entity.behaviour.HeroBehaviour;
import com.pancik.ciernypetrzlen.engine.component.entity.loot.EmptyLoot;
import com.pancik.ciernypetrzlen.engine.component.spell.Buff;
import com.pancik.ciernypetrzlen.engine.player.StatsPack;
import com.pancik.ciernypetrzlen.util.RenderUtils;

public class Hero extends Unit {
    public static final Vector2 SIZE = new Vector2(1.0f, 1.0f);
    public static final float SPEED = 2.0f / 60.0f;
    public static final int HEALTH = 200;
    public static final float ATTACK_RANGE = 1.0f;
    public static final float INTERACT_RANGE = 1.0f;
    public static final int ATTACK_STEP = 5;
    public static final int ATTACK_COOLDOWN = 40;
    public static final int REGEN_STEPS = 60;
    
    protected static final String textureName = "unit-hero";
    protected static TextureRegion[][] stand = Animation.cutAnimation4way(textureName, 0, 16, 16, 2);
    protected static TextureRegion[][] walk = Animation.cutAnimation4way(textureName, 32, 16, 16, 4);
    protected static TextureRegion[][] attack = Animation.cutAnimation4way(textureName, 96, 16, 16, 4);
    
    protected StatsPack statsPack;
    protected Vector2 targetPosition;
    protected int attackCooldown= 0;
    
    protected int mana;
    protected int regenTick = 0;
    
    public Hero(Vector2 position, int team, StatsPack statsPack, Engine.Controls engineControls) {
        super(position, SIZE, team, 0, SPEED, new EmptyLoot(), new HeroBehaviour(), engineControls);
        this.statsPack = statsPack;
        this.maxHealth = statsPack.getHitpoints();
        this.health = maxHealth;
        this.mana = statsPack.getMana();
    }
    
    @Override
    public void create() {
        decal = engineControls.createDecal(position, size.x, size.y, stand[Animation.DOWN][0]);
        //TextureRegion hero = new TextureRegion(DrawableData.textureAtlas.findRegion("unit-hero"), 0, 0, 16, 16);
        //decalTest = engineControls.createDecal(pos, 0.15f, SIZE.x, SIZE.y, hero);
    }

    @Override
    public void tick() {
        regenTick++;
        if(regenTick > REGEN_STEPS) {
            regenTick = 0;
            health += statsPack.getHitpointRegeneration();
            mana += statsPack.getManaRegeneration();
        }
        
        this.speed = SPEED * (statsPack.getMovementSpeedCoefficient() + getScalarModifier(Buff.Stat.MOVEMENT_SPEED)) * getPercentualModifier(Buff.Stat.MOVEMENT_SPEED);
        if(mana > statsPack.getMana()) {
            mana = (int)(((float)statsPack.getMana() + getScalarModifier(Buff.Stat.MANA)) * getPercentualModifier(Buff.Stat.MANA));
        }
        
        this.maxHealth = (int)(((float)statsPack.getHitpoints() + getScalarModifier(Buff.Stat.HEALTH)) * getPercentualModifier(Buff.Stat.HEALTH));
        if(health > maxHealth) {
            health = maxHealth;
        }
        
        super.tick();
        
        if(attackCooldown > 0) {
            attackCooldown--;
        }
    }
    
    @Override
    public Decal getDecal() {
        TextureRegion[][] textures;
        int step;
        if(currentAction == Action.WALK || currentAction == Action.STAND) {
            textures = currentAction == Action.WALK ? walk : stand;
            step = animationStep / 20;
        } else {
            textures = attack;
            step = animationStep / ATTACK_STEP;
        }
        decal.setTextureRegion(Animation.getTexture(textures, direction, step));
        decal.setPosition(position.x, decal.getY(), position.y);
        return decal;
    }

    @Override
    public void render() {
        Gdx.gl.glDisable(GL10.GL_DEPTH_TEST);
        DrawableData.shapeRenderer.begin(ShapeRenderer.ShapeType.Line);
        DrawableData.shapeRenderer.setColor(1.0f, 1.0f, 1.0f, 1.0f);
        if(getTargetPosition() != null) {
            DrawableData.shapeRenderer.setColor(1.0f, 1.0f, 1.0f, 1.0f);
            Vector2 pos = getTargetPosition();
            DrawableData.shapeRenderer.circle(pos.x, pos.y, 0.25f, 25);
        }
        DrawableData.shapeRenderer.end();
        Gdx.gl.glEnable(GL10.GL_DEPTH_TEST);
    }
    
    @Override
    public void debugRender() {
        DrawableData.shapeRenderer.begin(ShapeRenderer.ShapeType.Line);
        DrawableData.shapeRenderer.setColor(1.0f, 1.0f, 1.0f, 1.0f);
        DrawableData.shapeRenderer.rect(position.x - size.x/2, position.y - size.y/2, size.x, size.y);
        DrawableData.shapeRenderer.line(position.x, position.y, position.x + direction.x, position.y + direction.y);
        if(path != null) {
            DrawableData.shapeRenderer.setColor(0.0f, 1.0f, 1.0f, 1.0f);
            for(Vector2 v : path) {
                DrawableData.shapeRenderer.circle(v.x, v.y, 0.25f, 10);
            }
        }
        
        DrawableData.shapeRenderer.end();
    }
    
    public void renderCharacter2D(float x, float y, float size) {
        RenderUtils.blit(stand[Animation.DOWN][0], x, y, size, size, 0.0f, false);
    }

    @Override
    public boolean remove() {
        return isDead();
    }

    @Override
    public void onRemove() {
    }
    
    @Override
    public boolean isInRangeForAttack(Attackable a) {
        return (position.dst(a.getPosition()) < ATTACK_RANGE);
    }

    @Override
    public void attack(Attackable a) {
        direction.set(a.getPosition()).sub(position).nor();
        if(attackCooldown == 0) {
            setAttack(attack[0].length * ATTACK_STEP, 3 * ATTACK_STEP, a,
                    (int)(((float)statsPack.getAttackDamage() + getScalarModifier(Buff.Stat.ATTACK)) * getPercentualModifier(Buff.Stat.ATTACK)));
            attackCooldown = (int)((float)ATTACK_COOLDOWN * (1.0f/((statsPack.getAttackSpeedCoefficient() + getScalarModifier(Buff.Stat.ATTACK_SPEED)) * getPercentualModifier(Buff.Stat.ATTACK_SPEED))));
        }
    }
    
    public void onInteract() {
        setAttack(attack[0].length * ATTACK_STEP, 3 * ATTACK_STEP, null, 10);
    }

    public Vector2 getTargetPosition() {
        return targetPosition;
    }

    public void setTargetPosition(Vector2 target) {
        if(target != null) {
            if(engineControls.getCollisionMap().isWalkable(target)) {
                targetPosition = target;
            }
        } else {
            targetPosition = null;
        }
    }

    @Override
    public void interact() {
    }
    
    public float getInteractRange() {
        return INTERACT_RANGE;
    }

    @Override
    protected void doAttack(int damage, Attackable a) {
        if(!a.isDead()) {
            a.getHit(damage, this);
        }
        
        if(actionListener != null) {
            actionListener.damageDealtAttack(damage, a);
        }
        
        SoundData.playSound("sword", 1.0f);
    }
    
    public void rewardExperience(int e) {
        if (statsPack.rewardExperience(e)) {
            this.health = statsPack.getHitpoints();
            this.mana = statsPack.getMana();
            engineControls.startLevelUpEffect();
        }
    }
    
    @Override
    public String getName() {
        return "Hero";
    }

    @Override
    public int getHit(int damage, Attackable from) {
        int reducedDamage = Math.round((float)damage * (1.0f - ((statsPack.getPercentualDefense() + getScalarModifier(Buff.Stat.DEFENSE)) * getPercentualModifier(Buff.Stat.DEFENSE))));
        return super.getHit(reducedDamage, from);
    }
    
    public int getMana() {
        return mana;
    }
    
    public void changeMana(int delta) {
        if(!isDead()) {
            mana += delta;
            if(mana > statsPack.getMana()) {
                mana = statsPack.getMana();
            }
            if(mana < 0) {
                mana = 0;
            }
        }
    }

    public void setMana(int mana) {
        this.mana = mana;
    }

    public void setHealth(int health) {
        this.health = health;
    }
    
    public float getPercentualMana() {
        return (float)mana/(float)statsPack.getMana();
    }
    
    public float getPercentualExperience() {
        return (float)statsPack.getCurrentExperience()/(float)statsPack.getRequiredExperience();
    }
}
