package com.pancik.ciernypetrzlen.engine.component.entity;

import com.badlogic.gdx.graphics.g3d.decals.Decal;
import com.badlogic.gdx.graphics.glutils.ShapeRenderer;
import com.badlogic.gdx.math.Intersector;
import com.badlogic.gdx.math.MathUtils;
import com.badlogic.gdx.math.Vector2;
import com.badlogic.gdx.math.Vector3;
import com.badlogic.gdx.math.collision.Ray;
import com.pancik.ciernypetrzlen.DrawableData;
import com.pancik.ciernypetrzlen.engine.Engine;
import static com.pancik.ciernypetrzlen.engine.component.entity.Attackable.tmp3;
import com.pancik.ciernypetrzlen.engine.player.inventory.Item;

public class PickableItem extends Entity implements Interactable {
    protected static final float size = 0.5f;
    
    protected Vector2 position;
    protected float positionDeltaX = 0.0f;
    protected boolean adding = true;
    protected Item item;
    protected Decal decal;
    protected boolean picked = false;
    protected boolean highlighted = false;
    protected float maxDeltaY;
    
    public PickableItem(Vector2 position, Item item, Engine.Controls engineControls) {
        super(engineControls);
        this.position = position;
        this.item = item;
        this.maxDeltaY = MathUtils.random(0.040f);
    }
    
    @Override
    public void create() {
        decal = engineControls.createDecal(position, size, size, item.getTexture());
    }

    @Override
    public void tick() {
        if(positionDeltaX < -maxDeltaY) {
            adding = true;
            positionDeltaX = -maxDeltaY;
        } else if(positionDeltaX > maxDeltaY) {
            adding = false;
            positionDeltaX = maxDeltaY;
        }
        if(adding) {
            positionDeltaX += maxDeltaY/20.0f;
        } else {
            positionDeltaX -= maxDeltaY/20.0f;
        }
    }

    @Override
    public Decal getDecal() {
        decal.setPosition(position.x + positionDeltaX, decal.getY(), position.y);
        return decal;
    }

    @Override
    public void debugRender() {
        DrawableData.shapeRenderer.begin(ShapeRenderer.ShapeType.Line);
        DrawableData.shapeRenderer.setColor(0.5f, 1.0f, 0.0f, 1.0f);
        DrawableData.shapeRenderer.rect(position.x - size/2, position.y - size/2, size, size);
        DrawableData.shapeRenderer.end();
    }

    @Override
    public boolean remove() {
        return picked;
    }

    @Override
    public void onRemove() {
    }

    @Override
    public Vector2 getPosition() {
        return position;
    }

    @Override
    public void interact() {
        picked = engineControls.getPlayer().addInventoryItem(item);
    }

    @Override
    public void highlight() {
        highlighted = true;
    }
    
    @Override
    public void dehighlight() {
        highlighted = false;
    }

    @Override
    public boolean isHighlighted() {
        return highlighted;
    }

    @Override
    public boolean getClosestIntersection(Ray ray, Vector3 out) {
        decal.setPosition(position.x, decal.getY(), position.y);
        boolean found;
        float[] res = decal.getVertices();
        
        for(int i = 0; i < 3; i++) {
            tmp3[i].set(res[(i * 6)], res[(i * 6) + 1], res[(i * 6) + 2]);
        }
        found = Intersector.intersectRayTriangle(ray, tmp3[0], tmp3[1], tmp3[2], out);
        
        for(int i = 1; i < 4; i++) {
            tmp3[i - 1].set(res[(i * 6)], res[(i * 6) + 1], res[(i * 6) + 2]);
        }
        if(!found) {
            found = Intersector.intersectRayTriangle(ray, tmp3[0], tmp3[1], tmp3[2], out);
        } else {
            if(Intersector.intersectRayTriangle(ray, tmp3[0], tmp3[1], tmp3[2], tmp3[3])) {
                if(tmp3[3].dst(ray.origin) < out.dst(ray.origin)) {
                    out.set(tmp3[3]);
                }
            }
        }
        return found;
    }

    @Override
    public String getName() {
        return item.getName();
    }

    @Override
    public float getPercentualHealth() {
        return 0.0f;
    }
}
