package com.pancik.ciernypetrzlen.engine.component.entity;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.graphics.Color;
import com.badlogic.gdx.graphics.GL10;
import com.badlogic.gdx.graphics.Mesh;
import com.badlogic.gdx.graphics.VertexAttribute;
import com.badlogic.gdx.graphics.g2d.TextureRegion;
import com.badlogic.gdx.graphics.g3d.decals.Decal;
import com.badlogic.gdx.graphics.glutils.ShapeRenderer;
import com.badlogic.gdx.math.Intersector;
import com.badlogic.gdx.math.MathUtils;
import com.badlogic.gdx.math.Vector2;
import com.badlogic.gdx.math.Vector3;
import com.badlogic.gdx.math.collision.Ray;
import com.badlogic.gdx.utils.Disposable;
import com.badlogic.gdx.utils.FloatArray;
import com.pancik.ciernypetrzlen.DrawableData;
import com.pancik.ciernypetrzlen.engine.Engine;
import com.pancik.ciernypetrzlen.engine.component.LevelManager;
import com.pancik.ciernypetrzlen.engine.player.YellText;
import com.pancik.ciernypetrzlen.screens.WonGameScreen;

public class Stairs extends Entity implements Interactable, Disposable {
    protected static final int STEPS = 4;
    protected Vector2 position;
    protected Vector2 interactPosition;
    protected String changeLevel;
    protected boolean up;
    protected Mesh mesh;
    protected boolean highlighted = false;
    protected float[] highlightVertices;
    protected float[] collisionVertices;
    protected YellText tmpText;

    public Stairs(Vector2 interactPosition, String changeLevel, boolean up, Engine.Controls engineControls) {
        super(engineControls);
        this.interactPosition = interactPosition;
        this.position = new Vector2((int)interactPosition.x, (int)interactPosition.y);
        this.changeLevel = changeLevel;
        this.up = up;
    }

    @Override
    public void create() {
        TextureRegion[] t = engineControls.getLevelManager().getLevelMeshes().getStairsTiles();
        FloatArray v = new FloatArray(17);
        if(up) {
            newStairUp(v, position.x, 0.0f, position.y, STEPS, t);
        } else {
            newStairDown(v, position.x, 0.0f, position.y, STEPS, t);
        }
        v.shrink();
        mesh = new Mesh(true, v.size / 5, 0, new VertexAttribute[]{VertexAttribute.Position(), VertexAttribute.Normal(), VertexAttribute.TexCoords(0)});
        mesh.setVertices(v.items);
        
        engineControls.getCollisionMap().setWalkable(position.x, position.y, false);
        
        if (up) {
            highlightVertices = new float[]{
                position.x, position.y, 0.0f,
                position.x + 1.0f, position.y, 0.0f,
                position.x, position.y, 0.0f,
                position.x + 1.0f, position.y, -1.0f,
                position.x + 1.0f, position.y, 0.0f,
                position.x + 1.0f, position.y, -1.0f,
                position.x, position.y + 1.0f, 0.0f,
                position.x + 1.0f, position.y + 1.0f, 0.0f,
                position.x, position.y + 1.0f, 0.0f,
                position.x + 1.0f, position.y + 1.0f, -1.0f,
                position.x + 1.0f, position.y + 1.0f, 0.0f,
                position.x + 1.0f, position.y + 1.0f, -1.0f,
                position.x, position.y, 0.0f,
                position.x, position.y + 1.0f, 0.0f,
                position.x + 1.0f, position.y, 0.0f,
                position.x + 1.0f, position.y + 1.0f, 0.0f,
                position.x + 1.0f, position.y, -1.0f,
                position.x + 1.0f, position.y + 1.0f, -1.0f,};
        } else {
            highlightVertices = new float[]{
                position.x, position.y, 0.0f,
                position.x, position.y, 1.0f,
                position.x, position.y, 0.0f,
                position.x + 1.0f, position.y, 1.0f,
                position.x, position.y, 1.0f,
                position.x + 1.0f, position.y, 1.0f,
                position.x, position.y + 1.0f, 0.0f,
                position.x, position.y + 1.0f, 1.0f,
                position.x, position.y + 1.0f, 0.0f,
                position.x + 1.0f, position.y + 1.0f, 1.0f,
                position.x, position.y + 1.0f, 1.0f,
                position.x + 1.0f, position.y + 1.0f, 1.0f,
                position.x, position.y, 0.0f,
                position.x, position.y + 1.0f, 0.0f,
                position.x, position.y, 1.0f,
                position.x, position.y + 1.0f, 1.0f,
                position.x + 1.0f, position.y, 1.0f,
                position.x + 1.0f, position.y + 1.0f, 1.0f,};
        }
    
        if (up) {
            collisionVertices = new float[]{
                position.x, 0.0f, position.y,
                position.x + 1.0f, 0.0f, position.y,
                position.x + 1.0f, 1.0f, position.y,
                position.x, 0.0f, position.y + 1.0f,
                position.x + 1.0f, 0.0f, position.y + 1.0f,
                position.x + 1.0f, 1.0f, position.y + 1.0f,
                position.x + 1.0f, 0.0f, position.y,
                position.x + 1.0f, 0.0f, position.y + 1.0f,
                position.x + 1.0f, 1.0f, position.y + 1.0f,
                position.x + 1.0f, 0.0f, position.y,
                position.x + 1.0f, 1.0f, position.y,
                position.x + 1.0f, 1.0f, position.y + 1.0f,
                position.x, 0.0f, position.y,
                position.x + 1.0f, 1.0f, position.y,
                position.x + 1.0f, 1.0f, position.y + 1.0f,
                position.x, 0.0f, position.y,
                position.x, 0.0f, position.y + 1.0f,
                position.x + 1.0f, 1.0f, position.y + 1.0f,};
        } else {
            collisionVertices = new float[]{
                position.x, 0.0f, position.y,
                position.x, -1.0f, position.y,
                position.x + 1.0f, -1.0f, position.y,
                position.x, 0.0f, position.y + 1.0f,
                position.x, -1.0f, position.y + 1.0f,
                position.x + 1.0f, -1.0f, position.y + 1.0f,
                position.x, 0.0f, position.y,
                position.x + 1.0f, -1.0f, position.y,
                position.x + 1.0f, -1.0f, position.y + 1.0f,
                position.x, 0.0f, position.y,
                position.x, 0.0f, position.y + 1.0f,
                position.x + 1.0f, -1.0f, position.y + 1.0f,
            };
        }
    }
    
    @Override
    public void tick() {
    }

    @Override
    public void render() {
        LevelManager manager = engineControls.getLevelManager();
        manager.beginShader(null);
        mesh.render(manager.getShader(), GL10.GL_TRIANGLES);
        manager.endShader();
        
        if(highlighted) {
            Gdx.gl.glDisable(GL10.GL_DEPTH_TEST);
            DrawableData.shapeRenderer.begin(ShapeRenderer.ShapeType.Line);
            DrawableData.shapeRenderer.setColor(0.0f, 0.0f, 1.0f, 1.0f);
            

            for(int i = 0; i < highlightVertices.length; i += 6) {
                DrawableData.shapeRenderer.line(highlightVertices[i],
                        highlightVertices[i + 1],
                        highlightVertices[i + 2],
                        highlightVertices[i + 3],
                        highlightVertices[i + 4],
                        highlightVertices[i + 5]);
            }
            
            DrawableData.shapeRenderer.end();
            Gdx.gl.glEnable(GL10.GL_DEPTH_TEST);
        }
    }

    @Override
    public void debugRender() {
    }

    @Override
    public Decal getDecal() {
        return null;
    }

    @Override
    public boolean remove() {
        return false;
    }

    @Override
    public void onRemove() {
    }

    @Override
    public Vector2 getPosition() {
        return interactPosition;
    }

    @Override
    public void highlight() {
        highlighted = true;
    }

    @Override
    public void dehighlight() {
        highlighted = false;
    }

    @Override
    public boolean isHighlighted() {
        return highlighted;
    }

    @Override
    public boolean getClosestIntersection(Ray ray, Vector3 out) {
        return Intersector.intersectRayTriangles(ray, collisionVertices, out);
    }

    
    
    @Override
    public void interact() {
        if(changeLevel == null) {
            Vector3 yellPosition = new Vector3();
            yellPosition.set(position.x, 0.0f, position.y - 1.0f);
            if (tmpText != null) {
                tmpText.destroy();
            }
            String[] texts = {"I don't want to go back there...", "I came from there just now...", "No need for me to go back!", "Developer is too lazy to implement this."};
            tmpText = new YellText(engineControls,
                    yellPosition,
                    texts[MathUtils.random(3)],
                    Color.WHITE);
            engineControls.getPlayer().getTextEffectManager().addTextEffect(tmpText);
        } else {
            if(changeLevel.equals("WIN_GAME_SCREEN")) {
                engineControls.getMainControls().setScreen(new WonGameScreen(engineControls.getMainControls()), true);
            } else {
                engineControls.getPlayer().setLevelAndSaveExit(changeLevel);
            }
        }
    }

    @Override
    public void dispose() {
        mesh.dispose();
    }
        
    private void newStairUp(FloatArray v, float x, float y, float z, int steps, TextureRegion[] r) {
        float delta = 1.0f / steps;
        int c = 0;
        for(int i = 0; i < steps; i++) {
            float startX = x + delta * i;
            float endX = startX + delta;
            float startY = y + delta * i;
            float endY = startY + delta;
            
            //TOP
            {
                v.add(startX); v.add(endY); v.add(z); v.add(0.0f); v.add(1.0f); v.add(0.0f); v.add(r[c].getU()); v.add(r[c].getV());
                v.add(endX); v.add(endY); v.add(z); v.add(0.0f); v.add(1.0f); v.add(0.0f); v.add(r[c].getU2()); v.add(r[c].getV());
                v.add(endX); v.add(endY); v.add(z + 1.0f); v.add(0.0f); v.add(1.0f); v.add(0.0f); v.add(r[c].getU2()); v.add(r[c].getV2());
                
                v.add(endX); v.add(endY); v.add(z + 1.0f); v.add(0.0f); v.add(1.0f); v.add(0.0f); v.add(r[c].getU2()); v.add(r[c].getV2());
                v.add(startX); v.add(endY); v.add(z + 1.0f); v.add(0.0f); v.add(1.0f); v.add(0.0f); v.add(r[c].getU()); v.add(r[c].getV2());
                v.add(startX); v.add(endY); v.add(z); v.add(0.0f); v.add(1.0f); v.add(0.0f); v.add(r[c].getU()); v.add(r[c].getV());
            }
            
            //SIDES VERTICAL
            {
                v.add(startX); v.add(0.0f); v.add(z); v.add(-1.0f); v.add(0.0f); v.add(0.0f); v.add(r[c].getU()); v.add(r[c].getV());
                v.add(startX); v.add(0.0f); v.add(z + 1.0f); v.add(-1.0f); v.add(0.0f); v.add(0.0f); v.add(r[c].getU2()); v.add(r[c].getV());
                v.add(startX); v.add(endY); v.add(z + 1.0f); v.add(-1.0f); v.add(0.0f); v.add(0.0f); v.add(r[c].getU2()); v.add(r[c].getV2());
                
                v.add(startX); v.add(endY); v.add(z + 1.0f); v.add(-1.0f); v.add(0.0f); v.add(0.0f); v.add(r[c].getU2()); v.add(r[c].getV2());
                v.add(startX); v.add(endY); v.add(z); v.add(-1.0f); v.add(0.0f); v.add(0.0f); v.add(r[c].getU()); v.add(r[c].getV2());
                v.add(startX); v.add(0.0f); v.add(z); v.add(-1.0f); v.add(0.0f); v.add(0.0f); v.add(r[c].getU()); v.add(r[c].getV());

                if(i + 1 == steps) {
                    v.add(endX); v.add(0.0f); v.add(z); v.add(1.0f); v.add(0.0f); v.add(0.0f); v.add(r[c].getU()); v.add(r[c].getV());
                    v.add(endX); v.add(0.0f); v.add(z + 1.0f); v.add(1.0f); v.add(0.0f); v.add(0.0f); v.add(r[c].getU2()); v.add(r[c].getV());
                    v.add(endX); v.add(endY); v.add(z + 1.0f); v.add(1.0f); v.add(0.0f); v.add(0.0f); v.add(r[c].getU2()); v.add(r[c].getV2());

                    v.add(endX); v.add(endY); v.add(z + 1.0f); v.add(1.0f); v.add(0.0f); v.add(0.0f); v.add(r[c].getU2()); v.add(r[c].getV2());
                    v.add(endX); v.add(endY); v.add(z); v.add(1.0f); v.add(0.0f); v.add(0.0f); v.add(r[c].getU()); v.add(r[c].getV2());
                    v.add(endX); v.add(0.0f); v.add(z); v.add(1.0f); v.add(0.0f); v.add(0.0f); v.add(r[c].getU()); v.add(r[c].getV());
                }
            }
            
            //SIDES HORIZONTAL
            {
                //UP
                v.add(startX); v.add(0.0f); v.add(z); v.add(0.0f); v.add(0.0f); v.add(-1.0f); v.add(r[c].getU()); v.add(r[c].getV());
                v.add(endX); v.add(0.0f); v.add(z); v.add(0.0f); v.add(0.0f); v.add(-1.0f); v.add(r[c].getU2()); v.add(r[c].getV());
                v.add(endX); v.add(endY); v.add(z); v.add(0.0f); v.add(0.0f); v.add(-1.0f); v.add(r[c].getU2()); v.add(r[c].getV2());

                v.add(endX); v.add(endY); v.add(z); v.add(0.0f); v.add(0.0f); v.add(-1.0f); v.add(r[c].getU2()); v.add(r[c].getV2());
                v.add(startX); v.add(endY); v.add(z); v.add(0.0f); v.add(0.0f); v.add(-1.0f); v.add(r[c].getU()); v.add(r[c].getV2());
                v.add(startX); v.add(0.0f); v.add(z); v.add(0.0f); v.add(0.0f); v.add(-1.0f); v.add(r[c].getU()); v.add(r[c].getV());
                
                //DOWN
                v.add(startX); v.add(0.0f); v.add(z + 1.0f); v.add(0.0f); v.add(0.0f); v.add(1.0f); v.add(r[c].getU()); v.add(r[c].getV());
                v.add(endX); v.add(0.0f); v.add(z + 1.0f); v.add(0.0f); v.add(0.0f); v.add(1.0f); v.add(r[c].getU2()); v.add(r[c].getV());
                v.add(endX); v.add(endY); v.add(z + 1.0f); v.add(0.0f); v.add(0.0f); v.add(1.0f); v.add(r[c].getU2()); v.add(r[c].getV2());

                v.add(endX); v.add(endY); v.add(z + 1.0f); v.add(0.0f); v.add(0.0f); v.add(1.0f); v.add(r[c].getU2()); v.add(r[c].getV2());
                v.add(startX); v.add(endY); v.add(z + 1.0f); v.add(0.0f); v.add(0.0f); v.add(1.0f); v.add(r[c].getU()); v.add(r[c].getV2());
                v.add(startX); v.add(0.0f); v.add(z + 1.0f); v.add(0.0f); v.add(0.0f); v.add(1.0f); v.add(r[c].getU()); v.add(r[c].getV());
            }
        }
    }
    
    private void newStairDown(FloatArray v, float x, float y, float z, int steps, TextureRegion[] r) {
        float delta = 1.0f / steps;
        int c = 0;
        for(int i = 0; i < steps; i++) {
            float startX = x + delta * i;
            float endX = startX + delta;
            float startY = y - delta * i;
            float endY = startY - delta;
            
            
            //TOP
            {
                v.add(startX); v.add(endY); v.add(z); v.add(0.0f); v.add(1.0f); v.add(0.0f); v.add(r[c].getU()); v.add(r[c].getV());
                v.add(endX); v.add(endY); v.add(z); v.add(0.0f); v.add(1.0f); v.add(0.0f); v.add(r[c].getU2()); v.add(r[c].getV());
                v.add(endX); v.add(endY); v.add(z + 1.0f); v.add(0.0f); v.add(1.0f); v.add(0.0f); v.add(r[c].getU2()); v.add(r[c].getV2());
                
                v.add(endX); v.add(endY); v.add(z + 1.0f); v.add(0.0f); v.add(1.0f); v.add(0.0f); v.add(r[c].getU2()); v.add(r[c].getV2());
                v.add(startX); v.add(endY); v.add(z + 1.0f); v.add(0.0f); v.add(1.0f); v.add(0.0f); v.add(r[c].getU()); v.add(r[c].getV2());
                v.add(startX); v.add(endY); v.add(z); v.add(0.0f); v.add(1.0f); v.add(0.0f); v.add(r[c].getU()); v.add(r[c].getV());
            }
            
            //SIDES VERTICAL
            {
                v.add(startX); v.add(startY); v.add(z); v.add(1.0f); v.add(0.0f); v.add(0.0f); v.add(r[c].getU()); v.add(r[c].getV());
                v.add(startX); v.add(startY); v.add(z + 1.0f); v.add(1.0f); v.add(0.0f); v.add(0.0f); v.add(r[c].getU2()); v.add(r[c].getV());
                v.add(startX); v.add(endY); v.add(z + 1.0f); v.add(1.0f); v.add(0.0f); v.add(0.0f); v.add(r[c].getU2()); v.add(r[c].getV2());

                v.add(startX); v.add(endY); v.add(z + 1.0f); v.add(1.0f); v.add(0.0f); v.add(0.0f); v.add(r[c].getU2()); v.add(r[c].getV2());
                v.add(startX); v.add(endY); v.add(z); v.add(1.0f); v.add(0.0f); v.add(0.0f); v.add(r[c].getU()); v.add(r[c].getV2());
                v.add(startX); v.add(startY); v.add(z); v.add(1.0f); v.add(0.0f); v.add(0.0f); v.add(r[c].getU()); v.add(r[c].getV());
            }
        }
        
        //UP
        v.add(x); v.add(0.0f); v.add(z); v.add(0.0f); v.add(0.0f); v.add(1.0f); v.add(r[c].getU()); v.add(r[c].getV());
        v.add(x + 1.0f); v.add(0.0f); v.add(z); v.add(0.0f); v.add(0.0f); v.add(1.0f); v.add(r[c].getU2()); v.add(r[c].getV());
        v.add(x + 1.0f); v.add(-1.0f); v.add(z); v.add(0.0f); v.add(0.0f); v.add(1.0f); v.add(r[c].getU2()); v.add(r[c].getV2());

        v.add(x + 1.0f); v.add(-1.0f); v.add(z); v.add(0.0f); v.add(0.0f); v.add(1.0f); v.add(r[c].getU2()); v.add(r[c].getV2());
        v.add(x); v.add(-1.0f); v.add(z); v.add(0.0f); v.add(0.0f); v.add(1.0f); v.add(r[c].getU()); v.add(r[c].getV2());
        v.add(x); v.add(0.0f); v.add(z); v.add(0.0f); v.add(0.0f); v.add(1.0f); v.add(r[c].getU()); v.add(r[c].getV());

        //DOWN
        v.add(x); v.add(0.0f); v.add(z + 1.0f); v.add(0.0f); v.add(0.0f); v.add(-1.0f); v.add(r[c].getU()); v.add(r[c].getV());
        v.add(x + 1.0f); v.add(0.0f); v.add(z + 1.0f); v.add(0.0f); v.add(0.0f); v.add(-1.0f); v.add(r[c].getU2()); v.add(r[c].getV());
        v.add(x + 1.0f); v.add(-1.0f); v.add(z + 1.0f); v.add(0.0f); v.add(0.0f); v.add(-1.0f); v.add(r[c].getU2()); v.add(r[c].getV2());

        v.add(x + 1.0f); v.add(-1.0f); v.add(z + 1.0f); v.add(0.0f); v.add(0.0f); v.add(-1.0f); v.add(r[c].getU2()); v.add(r[c].getV2());
        v.add(x); v.add(-1.0f); v.add(z + 1.0f); v.add(0.0f); v.add(0.0f); v.add(-1.0f); v.add(r[c].getU()); v.add(r[c].getV2());
        v.add(x); v.add(0.0f); v.add(z + 1.0f); v.add(0.0f); v.add(0.0f); v.add(-1.0f); v.add(r[c].getU()); v.add(r[c].getV());
    }

    @Override
    public String getName() {
        return "Stairs";
    }

    @Override
    public float getPercentualHealth() {
        return 0.0f;
    }
}
