package com.pancik.ciernypetrzlen.engine.component.entity.behaviour;

import com.badlogic.gdx.math.Vector2;
import com.badlogic.gdx.utils.Array;
import com.pancik.ciernypetrzlen.engine.Engine;
import com.pancik.ciernypetrzlen.engine.component.entity.Hero;
import com.pancik.ciernypetrzlen.engine.component.entity.Unit;
import com.pancik.ciernypetrzlen.engine.pathfinding.AStarPathFinder;
import com.pancik.ciernypetrzlen.engine.pathfinding.Cell;
import com.pancik.ciernypetrzlen.engine.pathfinding.CollisionMap;
import com.pancik.ciernypetrzlen.engine.pathfinding.DistanceEstimator;

public class HeroBehaviour implements Behaviour {
    private final Vector2 tmp = new Vector2();
    
    @Override
    public void tick(Engine.Controls engineControls, Unit unit) {
        if(unit instanceof Hero) {
            Hero h = (Hero) unit;
            
            if(h.getTargetPosition() != null) {
                getPath(engineControls, h);
                followPath(h.getPath(), h);
                h.setState(Unit.State.WALK);
            } else {
                h.setState(Unit.State.IDLE);
            }
        }
    }
    
    private Array<Vector2> getPath(Engine.Controls engineControls, Hero h) {
        Array path = h.getPath();
        Vector2 target = h.getTargetPosition();

        boolean recalculate = (path == null) || (((Vector2) path.get(path.size - 1)).dst2(target) > 0.5f);
        if (recalculate) {
            CollisionMap map = engineControls.getCollisionMap();
            Cell startCell = map.getCell(h.getPosition().x, h.getPosition().y);
            Cell endCell = map.getCell(target.x, target.y);
            path = AStarPathFinder.findPath(startCell, endCell, DistanceEstimator.EUCLIDIAN, DistanceEstimator.MANHATTEN);
            if(path != null) {
                if(path.size > 1) {
                    if(((Vector2) path.get(path.size - 2)).dst(target) < ((Vector2) path.get(path.size - 2)).dst((Vector2) path.get(path.size - 1))) {
                        path.set(path.size - 1, target.cpy());
                    } else {
                        path.add(target.cpy());
                    }
                } else {
                    path.add(target.cpy());
                }
            }
            if(startCell.getPosition().equals(endCell.getPosition())) {
                path = new Array();
                path.add(target.cpy());
            }
            h.setPath(path);
            //System.out.println("repathing");
        }
        return path;
    }

    private void followPath(Array<Vector2> path, Hero h) {
        if(path == null) return;
        
        Vector2 desiredVelocity = this.tmp.set((Vector2) path.get(0)).sub(h.getPosition());

        if (desiredVelocity.len() < 0.1f) {
            path.removeIndex(0);
            //System.out.println(path.size + " way points left");
            if (path.size == 0) {
                h.setPath(null);
                h.setTargetPosition(null);
                return;
            }
            desiredVelocity.set((Vector2) path.get(0)).sub(h.getPosition());
        }

        desiredVelocity.nor().scl(h.getSpeed()).sub(h.getVelocity());
        h.getAcceleration().add(desiredVelocity);
    }

    @Override
    public void dispose() {
    }
}
