package com.pancik.ciernypetrzlen.engine.component.entity.loot;

import com.badlogic.gdx.math.MathUtils;
import com.badlogic.gdx.math.Vector2;
import com.pancik.ciernypetrzlen.engine.Engine;
import com.pancik.ciernypetrzlen.engine.component.entity.PickableItem;
import com.pancik.ciernypetrzlen.engine.player.inventory.BigHealthPotion;
import com.pancik.ciernypetrzlen.engine.player.inventory.BigManaPotion;
import com.pancik.ciernypetrzlen.engine.player.inventory.Item;
import com.pancik.ciernypetrzlen.engine.player.inventory.Picklock;
import com.pancik.ciernypetrzlen.engine.player.inventory.SmallHealthPotion;
import com.pancik.ciernypetrzlen.engine.player.inventory.SmallManaPotion;
import com.pancik.ciernypetrzlen.engine.player.inventory.Broadsword;
import com.pancik.ciernypetrzlen.engine.player.inventory.Dagger;
import com.pancik.ciernypetrzlen.engine.player.inventory.FoodChicken;
import com.pancik.ciernypetrzlen.engine.player.inventory.FoodFish;
import com.pancik.ciernypetrzlen.engine.player.inventory.IronChest;
import com.pancik.ciernypetrzlen.engine.player.inventory.IronHelmet;
import com.pancik.ciernypetrzlen.engine.player.inventory.IronShield;
import com.pancik.ciernypetrzlen.engine.player.inventory.LeatherChest;
import com.pancik.ciernypetrzlen.engine.player.inventory.Longsword;
import com.pancik.ciernypetrzlen.engine.player.inventory.MagicalBroadsword;
import com.pancik.ciernypetrzlen.engine.player.inventory.MagicalChest;
import com.pancik.ciernypetrzlen.engine.player.inventory.MagicalHelmet;
import com.pancik.ciernypetrzlen.engine.player.inventory.MagicalShield;
import com.pancik.ciernypetrzlen.engine.player.inventory.Stick;
import com.pancik.ciernypetrzlen.engine.player.inventory.WoodenHelmet;
import com.pancik.ciernypetrzlen.engine.player.inventory.WoodenShield;
import java.util.ArrayList;

public abstract class Loot {
    private Vector2 deathPos = new Vector2();
    protected ArrayList<Item> items = new ArrayList<Item>();
    
    public void onDeath(Engine.Controls engineControls, Vector2 pos) {
        deathPos.set((int)pos.x, (int)pos.y);
        
        for(Item i : items) {
            engineControls.addEntity(new PickableItem(deathPos.cpy().add(0.2f + MathUtils.random(0.6f), 0.2f + MathUtils.random(0.6f)),
                    i, engineControls));
        }
    }
    
    public void addItem(Item item) {
        items.add(item);
    }
    
    protected void addPicklock(float chance) {
        if(MathUtils.random(100.0f) < chance) {
            items.add(new Picklock());
        }
    }
    
    protected void addRandomFood(float chance, int level) {
        if(MathUtils.random(100.0f) < chance) {
            if(level < 5) {
                items.add(new FoodChicken());
            } else {
                items.add(new FoodFish());
            }
        }
    }
    
    protected void addRandomPotion(float chance) {
        if(MathUtils.random(100.0f) < chance) {
            int which = MathUtils.random(100);
            if(which < 30) {
                items.add(new SmallHealthPotion());
            } else if (which < 60) {
                items.add(new SmallManaPotion());
            } else if (which < 80) {
                items.add(new BigHealthPotion());
            } else {
                items.add(new BigManaPotion());
            }
        }
    }
    
    protected void addRandomEquipment(float chance) {
        if(MathUtils.random(100.0f) < chance) {
            int which = MathUtils.random(100);
            if(which < 15) {
                which = MathUtils.random(100);
                if(which < 60) {
                    items.add(new WoodenShield());
                } else {
                    items.add(new IronShield());
                }
            } else if(which < 30) {
                which = MathUtils.random(100);
                if(which < 60) {
                    items.add(new WoodenHelmet());
                } else {
                    items.add(new IronHelmet());
                }
            } else if(which < 45) {
                which = MathUtils.random(100);
                if(which < 60) {
                    items.add(new LeatherChest());
                } else {
                    items.add(new IronChest());
                }
            } else {
                which = MathUtils.random(100);
                if(which < 50) {
                    items.add(new Stick());
                } else if(which < 82) {
                    items.add(new Dagger());
                } else if(which < 92) {
                    items.add(new Longsword());
                } else {
                    items.add(new Broadsword());
                }
            }
        }
    }
    
    protected void addRandomMagicalEquipment(float chance) {
        if(MathUtils.random(100.0f) < chance) {
            int which = MathUtils.random(100);
            if(which < 15) {
                items.add(new MagicalShield());
            } else if(which < 30) {
                items.add(new MagicalHelmet());
            } else if(which < 45) {
                items.add(new MagicalChest());
            } else {
                items.add(new MagicalBroadsword());
            }
        }
    }
}
