package com.pancik.ciernypetrzlen.engine.component.entity.projectile;

import com.badlogic.gdx.graphics.g2d.TextureRegion;
import com.badlogic.gdx.graphics.g3d.decals.Decal;
import com.badlogic.gdx.graphics.glutils.ShapeRenderer;
import com.badlogic.gdx.math.Vector2;
import com.pancik.ciernypetrzlen.DrawableData;
import com.pancik.ciernypetrzlen.engine.Animation;
import com.pancik.ciernypetrzlen.engine.Engine;
import com.pancik.ciernypetrzlen.engine.component.particle.ParticleArrowDestroy;

public class Arrow extends Projectile {
    public static final float SPEED = 10.0f / 60.0f;
    public static final float DIAMETER = 0.50f;
    
    protected TextureRegion[] texture = DrawableData.textureAtlas.findRegion("projectile-arrow").split(8, 8)[0];
    protected Decal decal;
    
    public Arrow(Vector2 targetPos, Vector2 pos, Engine.Controls engineControls, Callback callback) {
        super(targetPos, pos, DIAMETER, SPEED, engineControls, callback);
    }

    @Override
    protected void onProjectileArrived() {
    }

    @Override
    public void create() {
        decal = engineControls.createDecal(position, diameter, diameter, texture[0]);
    }

    @Override
    public Decal getDecal() {
        tmp.set(targetPosition).sub(position);
        int index = Animation.getDirectionIndex(tmp);
        decal.setTextureRegion(texture[index]);
        decal.setPosition(position.x, decal.getY(), position.y);
        return decal;
    }

    @Override
    public void debugRender() {
        DrawableData.shapeRenderer.begin(ShapeRenderer.ShapeType.Line);
        DrawableData.shapeRenderer.setColor(1.0f, 0.0f, 0.0f, 1.0f);
        DrawableData.shapeRenderer.rect(position.x - diameter/2, position.y - diameter/2, diameter, diameter);
        DrawableData.shapeRenderer.end();
    }

    @Override
    public void onRemove() {
        engineControls.addEntity(new ParticleArrowDestroy(position, diameter, engineControls));
    }
}
