package com.pancik.ciernypetrzlen.engine.component.entity.projectile;

import com.badlogic.gdx.math.MathUtils;
import com.badlogic.gdx.math.Vector2;
import com.pancik.ciernypetrzlen.engine.Engine;
import com.pancik.ciernypetrzlen.engine.component.entity.Entity;

public abstract class Projectile extends Entity {
    public static interface Callback {
        public void onProjectileArrived(Vector2 pos);
        public void onProjectileCrashed(Vector2 pos);
    }

    protected static final Vector2 tmp = new Vector2();
    protected Vector2 targetPosition;
    protected Vector2 position;
    protected float diameter;
    protected float speed;
    protected Callback callback;
    
    protected float angle;
    protected boolean hit = false;

    public Projectile(Vector2 targetPos, Vector2 pos, float diameter, float speed, Engine.Controls engineControls, Callback callback) {
        super(engineControls);
        this.targetPosition = targetPos;
        this.position = pos;
        this.diameter = diameter;
        this.speed = speed;
        this.callback = callback;
    }

    @Override
    public void tick() {
        tmp.set(targetPosition);
        tmp.sub(position);
        this.angle = tmp.angle();
        if(tmp.len() > speed) {
            tmp.nor();
            tmp.scl(speed);
            this.position.add(tmp);
        } else {
            this.position.set(targetPosition);
        }
        
        if(!engineControls.getCollisionMap().isWalkable(position) && !engineControls.getCollisionMap().isDebris(position)) {
            hit = true;
            callback.onProjectileCrashed(position);
        }
        
        if(!remove()) {
            if(targetPosition.dst(position) < diameter/2) {
                if(!hit) {
                    callback.onProjectileArrived(targetPosition);
                    onProjectileArrived();
                    hit = true;
                }
            }
        }
    }
    
    @Override
    public boolean remove() {
        return hit;
    }
    
    protected abstract void onProjectileArrived();
    
    public static Vector2 offsetRandomPos(Vector2 pos, Vector2 size) {
        Vector2 topLeft = pos.cpy().sub(size.cpy().scl(0.5f));
        
        int xCoord = MathUtils.random((int)size.x);
        int yCoord = MathUtils.random((int)size.y);
        
        return new Vector2(topLeft.x + xCoord, topLeft.y + yCoord).sub(pos);
    }
}
