package com.pancik.ciernypetrzlen.engine.component.entity.units;

import com.badlogic.gdx.graphics.Color;
import com.badlogic.gdx.graphics.g2d.TextureRegion;
import com.badlogic.gdx.math.MathUtils;
import com.badlogic.gdx.math.Vector2;
import com.badlogic.gdx.math.Vector3;
import com.pancik.ciernypetrzlen.engine.Animation;
import com.pancik.ciernypetrzlen.engine.Engine;
import com.pancik.ciernypetrzlen.engine.SoundData;
import com.pancik.ciernypetrzlen.engine.component.entity.Attackable;
import com.pancik.ciernypetrzlen.engine.component.entity.Fire;
import com.pancik.ciernypetrzlen.engine.component.entity.Monster;
import com.pancik.ciernypetrzlen.engine.component.entity.projectile.FinalBossFireball;
import com.pancik.ciernypetrzlen.engine.component.entity.projectile.FinalBossSkeletonball;
import com.pancik.ciernypetrzlen.engine.component.entity.projectile.Projectile;
import com.pancik.ciernypetrzlen.engine.component.particle.FinalBossDeathParticle;
import com.pancik.ciernypetrzlen.engine.component.spell.Buff;
import com.pancik.ciernypetrzlen.engine.pathfinding.BresenhamCollisionChecker;
import com.pancik.ciernypetrzlen.engine.player.YellText;

public class FinalBoss extends Monster {
    public static final Vector2 SIZE = new Vector2(1.25f, 1.25f);
    public static final int HEALTH = 5000;
    public static final float SPEED = 0.5f / 60.0f;
    public static final int ATTACK_DAMAGE = 20;
    public static final float ATTACK_RANGE = 10.0f;
    public static final int ATTACK_STEP = 5;
    public static final int ATTACK_COOLDOWN = 420;
    
    protected static final String textureName = "unit-boss";
    protected static TextureRegion[][] stand = Animation.cutAnimation3way(textureName, 0, 24, 24, 2);
    protected static TextureRegion[][] walk = Animation.cutAnimation3way(textureName, 48, 24, 24, 4);
    protected static TextureRegion[][] attack = Animation.cutAnimation3way(textureName, 144, 24, 24, 4);
    
    protected int failedShots = 0;
    protected int failedShotsRecoveryStep = 0;
    protected final int maxFailedShots = 2;
    protected final int failedShotsRecoveryLength = 180;

    protected int attacksCount = 0;
    
    public FinalBoss(Vector2 position, int team, int level, Engine.Controls engineControls) {
        super(position, SIZE, team, engineControls,
                new MonsterStats(level, HEALTH, 1.0f, SPEED, ATTACK_DAMAGE, ATTACK_RANGE, ATTACK_STEP, ATTACK_COOLDOWN));
    }

    @Override
    protected TextureRegion[][] getStandAnimation() {
        return stand;
    }

    @Override
    protected TextureRegion[][] getWalkAnimation() {
        return walk;
    }

    @Override
    protected TextureRegion[][] getAttackAnimation() {
        return attack;
    }

    @Override
    public void tick() {
        super.tick();

        if (failedShotsRecoveryStep > 0) {
            failedShotsRecoveryStep--;
        }
    }

    @Override
    public boolean isInRangeForAttack(Attackable a) {
        return super.isInRangeForAttack(a)
                && BresenhamCollisionChecker.isInLineOfSight(engineControls.getCollisionMap(), position.x, position.y, a.getPosition().x, a.getPosition().y)
                && failedShotsRecoveryStep == 0;
    }

    @Override
    public void attack(Attackable a) {
        super.attack(a);

        if (attackCooldown == 0) {
            setAttack(getAttackAnimation()[0].length * stats.attackStep, 3 * stats.attackStep, a,
                    (int)(((float)stats.attackDamage + getScalarModifier(Buff.Stat.ATTACK)) * getPercentualModifier(Buff.Stat.ATTACK)));
            attackCooldown = (int)((float)stats.attackCooldownLength * (1.0f/((1.0f + getScalarModifier(Buff.Stat.ATTACK_SPEED)) * getPercentualModifier(Buff.Stat.ATTACK_SPEED))));
        }
    }

    @Override
    protected void doAttack(int damage, Attackable a) {
        attacksCount++;
        if(attacksCount >= 5) {
            attacksCount -= 5;
            
            String[] yellTexts = {"YOU WILL DIE!", "YOU CAN'T DEFEAT ME!", "I WILL CRUSH YOU!", "AHAHAHA EAT SOME FIRE!"};
            
            Vector3 yellPosition = new Vector3();
            yellPosition.set(position.x, 0.0f, position.y - SIZE.y);
            engineControls.getPlayer().getTextEffectManager().addTextEffect(new YellText(engineControls,
                    yellPosition,
                    yellTexts[MathUtils.random(yellTexts.length - 1)],
                    Color.RED));
        }
        
        final Attackable target = a;
        final int targetDamage = damage;
        final int fireLifeLength = 1200;
        final int level = Math.max(1, stats.level - 5);
        if(attacksCount % 4 == 1) {
            float distance = 1.5f;
            final Vector2[] newDirections = {
                new Vector2(distance, distance),
                new Vector2(distance, -distance),
                new Vector2(-distance, distance),
                new Vector2(-distance, -distance),
            };
            engineControls.addEntity(new FinalBossSkeletonball(target.getPosition().cpy(), position.cpy(), engineControls, new Projectile.Callback() {
                @Override
                public void onProjectileArrived(Vector2 pos) {
                    for(int i = 0; i < newDirections.length; i++) {
                        engineControls.addEntity(new FinalBossSkeletonball(newDirections[i].add(pos), pos.cpy(), engineControls, new Projectile.Callback() {

                            @Override
                            public void onProjectileArrived(Vector2 pos) {
                                if(engineControls.getCollisionMap().isWalkable(pos)) {
                                    int random = MathUtils.random(99);
                                    Monster monster;
                                    if(random < 33) {
                                        monster = new Skeleton(pos, FinalBoss.this.team, level, engineControls);
                                    } else if (random < 66) {
                                        monster = new SkeletonArcher(pos, FinalBoss.this.team, level, engineControls);
                                    } else {
                                        monster = new SkeletonMage(pos, FinalBoss.this.team, level, engineControls);
                                    }
                                    engineControls.addEntity(monster);
                                }
                            }

                            @Override
                            public void onProjectileCrashed(Vector2 pos) {
                            }
                        }));
                    }

                    failedShots = 0;
                }

                @Override
                public void onProjectileCrashed(Vector2 pos) {
                    failedShots++;
                    if (failedShots > maxFailedShots) {
                        failedShots = 0;
                        failedShotsRecoveryStep = failedShotsRecoveryLength;
                    }
                }
            }));
        } else {
            float distance = MathUtils.random(1.0f) + 1.5f;
            final Vector2[] newDirections = {
                new Vector2(distance, distance),
                new Vector2(distance, -distance),
                new Vector2(-distance, distance),
                new Vector2(-distance, -distance),
            };
            engineControls.addEntity(new FinalBossFireball(target.getPosition().cpy(), position.cpy(), engineControls, new Projectile.Callback() {
                @Override
                public void onProjectileArrived(Vector2 pos) {
                    engineControls.addEntity(new Fire(pos.sub(0.0f, +0.25f), target, targetDamage, true, fireLifeLength, engineControls));

                    for(int i = 0; i < newDirections.length; i++) {
                        engineControls.addEntity(new FinalBossFireball(newDirections[i].add(pos), pos.cpy(), engineControls, new Projectile.Callback() {

                            @Override
                            public void onProjectileArrived(Vector2 pos) {
                                engineControls.addEntity(new Fire(pos.sub(0.0f, +0.25f), target, targetDamage, true, fireLifeLength, engineControls));
                            }

                            @Override
                            public void onProjectileCrashed(Vector2 pos) {
                                engineControls.addEntity(new Fire(pos.sub(0.0f, +0.25f), target, targetDamage, true, fireLifeLength, engineControls));
                            }
                        }));
                    }

                    failedShots = 0;
                }

                @Override
                public void onProjectileCrashed(Vector2 pos) {
                    engineControls.addEntity(new Fire(pos, target, targetDamage, true, fireLifeLength, engineControls));

                    failedShots++;
                    if (failedShots > maxFailedShots) {
                        failedShots = 0;
                        failedShotsRecoveryStep = failedShotsRecoveryLength;
                    }
                }
            }));
        }
    }
        
    @Override
    public String getName() {
        return "Big Bad Boss " + super.getName();
    }
    
    @Override
    public void onRemove() {
        super.onRemove();
        
        SoundData.playSound("bones", 0.5f);
        engineControls.addEntity(new FinalBossDeathParticle(position, 1.25f, engineControls));
    }
}
