package com.pancik.ciernypetrzlen.engine.component.entity.units;

import com.badlogic.gdx.graphics.g2d.TextureRegion;
import com.badlogic.gdx.math.Vector2;
import com.pancik.ciernypetrzlen.engine.Animation;
import com.pancik.ciernypetrzlen.engine.Engine;
import com.pancik.ciernypetrzlen.engine.SoundData;
import com.pancik.ciernypetrzlen.engine.component.entity.Attackable;
import com.pancik.ciernypetrzlen.engine.component.entity.Monster;
import com.pancik.ciernypetrzlen.engine.component.entity.projectile.Arrow;
import com.pancik.ciernypetrzlen.engine.component.entity.projectile.Projectile;
import com.pancik.ciernypetrzlen.engine.component.particle.SkeletonDeathParticle;

public class SkeletonArcher extends Monster.Ranged {
    public static final Vector2 SIZE = new Vector2(1.0f, 1.0f);
    public static final int HEALTH = 100;
    public static final float SPEED = 1.5f / 60.0f;
    public static final int ATTACK_DAMAGE = 2;
    public static final float ATTACK_RANGE = 5.0f;
    public static final int ATTACK_STEP = 5;
    public static final int ATTACK_COOLDOWN = 80;
    
    protected static final String textureName = "unit-skeleton-archer";
    protected static TextureRegion[][] stand = Animation.cutAnimation3way(textureName, 0, 16, 16, 2);
    protected static TextureRegion[][] walk = Animation.cutAnimation3way(textureName, 32, 16, 16, 4);
    protected static TextureRegion[][] attack = Animation.cutAnimation3way(textureName, 96, 16, 16, 4);

    public SkeletonArcher(Vector2 position, int team, int level, Engine.Controls engineControls) {
        super(position, SIZE, team, engineControls,
                new MonsterStats(level, HEALTH, 1.0f, SPEED, ATTACK_DAMAGE, ATTACK_RANGE, ATTACK_STEP, ATTACK_COOLDOWN));
    }

    @Override
    protected TextureRegion[][] getStandAnimation() {
        return stand;
    }

    @Override
    protected TextureRegion[][] getWalkAnimation() {
        return walk;
    }

    @Override
    protected TextureRegion[][] getAttackAnimation() {
        return attack;
    }

    @Override
    protected void createProjectile(Attackable target, Projectile.Callback callback) {
        SoundData.playSound("bow", 1.0f);
        engineControls.addEntity(new Arrow(target.getPosition(), position.cpy(), engineControls, callback));
    }

    @Override
    public String getName() {
        return "Skeleton Archer " + super.getName();
    }
    
    @Override
    public void onRemove() {
        super.onRemove();
        
        SoundData.playSound("bones", 0.25f);
        engineControls.addEntity(new SkeletonDeathParticle(position, 1.0f, engineControls));
    }
}
