package com.pancik.ciernypetrzlen.engine.component.level;

import com.badlogic.gdx.graphics.g2d.TextureRegion;
import com.badlogic.gdx.graphics.g3d.decals.Decal;
import com.badlogic.gdx.graphics.glutils.ShapeRenderer;
import com.badlogic.gdx.math.Vector2;
import com.badlogic.gdx.math.Vector3;
import com.badlogic.gdx.math.collision.Ray;
import com.pancik.ciernypetrzlen.DrawableData;
import com.pancik.ciernypetrzlen.engine.Engine;
import com.pancik.ciernypetrzlen.engine.SoundData;
import com.pancik.ciernypetrzlen.engine.component.entity.Attackable;
import com.pancik.ciernypetrzlen.engine.component.particle.ParticleDebrisDestroy;

public class Debris extends Attackable {
    protected DebrisType type;
    
    public Debris(Vector2 position, DebrisType type, Engine.Controls engineControls) {
        super(position, new Vector2(type.sizeX, type.sizeY), Attackable.DEBRIS_TEAM, type.health, type.getLoot(), engineControls);
        this.type = type;
    }
    
    @Override
    public void create() {
        TextureRegion tex = new TextureRegion(DrawableData.textureAtlas.findRegion(type.textureName),
               type.texBeginX, type.texBeginY, type.texSizeX, type.texSizeY);
        tex.flip(true, false);
        decal = engineControls.createDecal(position, size.x, size.y, tex);
        
        engineControls.getCollisionMap().setWalkable(position.x, position.y, false);
    }
    
    @Override
    public int getHit(int damage, Attackable from) {
        if(type.attackable) {
            return super.getHit(damage, from);
        }
        return 0;
    }

    @Override
    public void tick() {
    }

    @Override
    public Decal getDecal() {
        return decal;
    }

    @Override
    public void debugRender() {
        DrawableData.shapeRenderer.begin(ShapeRenderer.ShapeType.Line);
        DrawableData.shapeRenderer.setColor(1.0f, 1.0f, 0.0f, 1.0f);
        DrawableData.shapeRenderer.rect(position.x - size.x/2, position.y - size.y/2, size.x, size.y);
        DrawableData.shapeRenderer.setColor(1.0f, 1.0f, 1.0f, 1.0f);
        DrawableData.shapeRenderer.end();
    }

    @Override
    public boolean remove() {
        return isDead();
    }

    @Override
    public void onRemove() {
        super.onRemove();
        
        engineControls.getCollisionMap().setWalkable(position.x, position.y, true);
        
        engineControls.addEntity(new ParticleDebrisDestroy(position, 0.8f, engineControls));
        SoundData.playSound("debris-smash", 0.5f);
    }    

    @Override
    public boolean getClosestIntersection(Ray ray, Vector3 out) {
        if(type.attackable) {
            return super.getClosestIntersection(ray, out);
        } else {
            return false;
        }
    }

    @Override
    public void interact() {
    }

    @Override
    public String getName() {
        return type.debrisName;
    }
}
