package com.pancik.ciernypetrzlen.engine.component.spell;

import com.badlogic.gdx.graphics.g2d.TextureRegion;
import com.badlogic.gdx.math.MathUtils;
import com.badlogic.gdx.math.Vector2;
import com.pancik.ciernypetrzlen.DrawableData;
import com.pancik.ciernypetrzlen.engine.Engine;
import com.pancik.ciernypetrzlen.engine.SoundData;
import com.pancik.ciernypetrzlen.engine.component.entity.Attackable;
import com.pancik.ciernypetrzlen.engine.component.entity.Unit;
import com.pancik.ciernypetrzlen.engine.component.entity.projectile.Projectile;
import com.pancik.ciernypetrzlen.engine.component.entity.projectile.SpellFireball;
import com.pancik.ciernypetrzlen.engine.component.particle.FireballExplosionParticle;
import com.pancik.ciernypetrzlen.engine.component.particle.ParticleSpellFireball;
import java.util.List;

public class Fireball extends Spell {
    private static final float RANGE = 10.0f;
    private static final float RADIUS = 2.0f;
    private static int[] manaLevels = {20, 20, 36, 51};
    private static int[] damageLevels = {20, 20, 100, 200};
    
    public Fireball() {
        super(new int[] {5, 35, 66}, Type.POSITION);
    }

    @Override
    public void cast(Engine.Controls engineControls, Attackable caster, Vector2 position) {
        engineControls.addEntity(new ParticleSpellFireball(caster.getPosition(), 1.0f, engineControls));
        
        final Attackable finalCaster = caster;
        final Vector2 finalTarget = position.cpy();
        final Engine.Controls finalControls = engineControls;
        engineControls.addEntity(new SpellFireball(finalTarget, caster.getPosition().cpy(), engineControls, new Projectile.Callback() {

            @Override
            public void onProjectileArrived(Vector2 pos) {
                finalControls.addEntity(new FireballExplosionParticle(pos, 1.0f, finalControls));
                
                Unit u = null;
                if(finalCaster instanceof Unit) {
                    u = (Unit)finalCaster;
                    
                }
                
                List<Attackable> attackables = finalControls.getEntityManager().getAttackableEntities(pos, RADIUS);
                for(Attackable a : attackables) {
                    if(a.getTeam() != finalCaster.getTeam()) {
                        int damage = damageLevels[level] +  (1 + MathUtils.random(2)) * 5;
                        a.getHit(damage, finalCaster);
                        
                        if(u != null && u.getActionListener() != null) {
                            u.getActionListener().damageDealtSpell(damage, a);
                        }
                    }
                }
            }

            @Override
            public void onProjectileCrashed(Vector2 pos) {
                finalControls.addEntity(new FireballExplosionParticle(pos, 1.0f, finalControls));
                
                Unit u = null;
                if(finalCaster instanceof Unit) {
                    u = (Unit)finalCaster;
                    
                }
                
                List<Attackable> attackables = finalControls.getEntityManager().getAttackableEntities(pos, RADIUS);
                for(Attackable a : attackables) {
                    if(a.getTeam() != finalCaster.getTeam()) {
                        int damage = damageLevels[level] +  (1 + MathUtils.random(2)) * 5;
                        a.getHit(damage, finalCaster);
                        
                        if(u != null && u.getActionListener() != null) {
                            u.getActionListener().damageDealtSpell(damage, a);
                        }
                    }
                }
            }
        }));
        
        SoundData.playSound("spell", 1.0f);
    }

    @Override
    public boolean inRange(Vector2 position, Vector2 target) {
        return (position.dst(target) < RANGE);
    }

    @Override
    public int getManaCost() {
        return manaLevels[level];
    }

    @Override
    public TextureRegion getIcon() {
        return DrawableData.textureAtlas.findRegion("icon-spell-fireball");
    }

    @Override
    public String getName() {
        return "Fireball";
    }

    @Override
    public String getText() {
        return "Warrior casts large fireball headed towards target location causing around " + damageLevels[level] + " damage in range 1 meter of the explosion.";
    }
}
