package com.pancik.ciernypetrzlen.engine.component.spell;

import com.badlogic.gdx.graphics.g2d.TextureRegion;
import com.badlogic.gdx.math.Vector2;
import com.pancik.ciernypetrzlen.DrawableData;
import com.pancik.ciernypetrzlen.engine.Engine;
import com.pancik.ciernypetrzlen.engine.SoundData;
import com.pancik.ciernypetrzlen.engine.component.entity.Attackable;
import com.pancik.ciernypetrzlen.engine.component.particle.ParticleSpellHeal;

public class Heal extends Spell {
    private static int[] manaLevels = {24, 24, 36, 45};
    private static int[] healAmount = {30, 30, 120, 250};
    
    public Heal() {
        super(new int[] {5, 32, 48}, Type.SELF);
    }

    @Override
    public void cast(Engine.Controls engineControls, Attackable caster) {
        caster.changeHealth(healAmount[level]);
        
        engineControls.addEntity(new ParticleSpellHeal(caster.getPosition(), 1.0f, engineControls));
        
        SoundData.playSound("spell", 1.0f);
    }

    @Override
    public boolean inRange(Vector2 position, Vector2 target) {
        return true;
    }

    @Override
    public int getManaCost() {
        return manaLevels[level];
    }

    @Override
    public TextureRegion getIcon() {
        return DrawableData.textureAtlas.findRegion("icon-spell-heal");
    }

    @Override
    public String getName() {
        return "Heal";
    }

    @Override
    public String getText() {
        return "Warrior uses his holy powers to restore " + healAmount[level] + " of his health.";
    }
}
