package com.pancik.ciernypetrzlen.engine.component.spell;

import com.badlogic.gdx.graphics.g2d.TextureRegion;
import com.badlogic.gdx.math.Vector2;
import com.pancik.ciernypetrzlen.DrawableData;
import com.pancik.ciernypetrzlen.engine.Engine;
import com.pancik.ciernypetrzlen.engine.SoundData;
import com.pancik.ciernypetrzlen.engine.component.entity.Attackable;
import com.pancik.ciernypetrzlen.engine.component.particle.ParticleSpellRoar;

public class Roar extends Spell {
    private static int[] manaLevels = {16, 16, 32, 41};
    private static int[] modifierLevelsText = {20, 20, 25, 30};
    private static float[] modifierLevels = {1.2f, 1.2f, 1.25f, 1.30f};
    
    public Roar() {
        super(new int[] {4, 34, 47}, Type.SELF);
    }

    @Override
    public void cast(Engine.Controls engineControls, Attackable caster) {
        engineControls.addEntity(new ParticleSpellRoar(caster.getPosition(), 1.0f, engineControls));
        
        caster.addBuff(new RoarBuff());
        
        SoundData.playSound("roar", 1.0f);
    }

    @Override
    public boolean inRange(Vector2 position, Vector2 target) {
        return true;
    }

    @Override
    public int getManaCost() {
        return manaLevels[level];
    }

    @Override
    public TextureRegion getIcon() {
        return DrawableData.textureAtlas.findRegion("icon-spell-roar");
    }

    @Override
    public String getName() {
        return "Roar";
    }

    @Override
    public String getText() {
        return "Warrior roars in pure rage resulting in increased attack and movement speed by " + modifierLevelsText[level] + "% for 10 seconds.";
    }
    
    public class RoarBuff extends Buff {
        private static final int lifeLenght = 600;
        private int life = 0;
        
        @Override
        public void tick() {
            life++;
        }

        @Override
        public boolean remove() {
            return life >= lifeLenght;
        }

        @Override
        public TextureRegion getIcon() {
            return DrawableData.textureAtlas.findRegion("icon-spell-roar");
        }

        @Override
        public String getName() {
            return "Roar";
        }

        @Override
        public float getScalarModifier(Stat stat) {
            return 0.0f;
        }

        @Override
        public float getPercentualModifier(Stat stat) {
            if(stat == Stat.ATTACK_SPEED || stat == Stat.MOVEMENT_SPEED) {
                return modifierLevels[level];
            }
            return 1.0f;
        }

        @Override
        public void refresh() {
            life = 0;
        }

        @Override
        public void damageTaken(int damage) {
        }
    }
}
