package com.pancik.ciernypetrzlen.engine.component.spell;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.graphics.Color;
import com.badlogic.gdx.graphics.g2d.TextureRegion;
import com.badlogic.gdx.math.Vector2;
import com.pancik.ciernypetrzlen.DrawableData;
import com.pancik.ciernypetrzlen.engine.Engine;
import com.pancik.ciernypetrzlen.engine.SoundData;
import com.pancik.ciernypetrzlen.engine.component.entity.Attackable;
import com.pancik.ciernypetrzlen.util.RenderUtils;

public abstract class Spell {
    public static enum Type {
        SELF,
        POSITION,
        TARGET
    };
    
    protected TextureRegion textureTooltip = DrawableData.textureAtlas.findRegion("gui-tooltip");
    protected int level = 0;
    protected Type type;
    protected int[] upgradeCost;

    public Spell(int[] upgradeCost, Type type) {
        this.type = type;
        this.upgradeCost = upgradeCost;
    }

    public void render(int posX, int posY, int sizeX) {
        RenderUtils.blit(getIcon(), posX, posY, sizeX, sizeX, 0.0f, false);
    }
    
    public void renderTooltip(float x, float y, int sizeScale) {
        if(sizeScale > 3) {
            sizeScale = (int)Math.max(Math.round((float)sizeScale/1.5f), 1.0f);
        }
        
        float sizeX = 96 * sizeScale;
        float sizeY = 64 * sizeScale;
        float topY = y - sizeY;
        if(topY < 5) {
            topY = 5;
        }
        if(x + sizeX > Gdx.graphics.getWidth() - 5) {
            x = Gdx.graphics.getWidth() - sizeX - 5;
        }
        float currentY = topY + (3 * sizeScale);
        float wrapX = sizeX - (10 * sizeScale);
        
        RenderUtils.blit(textureTooltip, x, topY, sizeX, sizeY, 0.0f, false);
        float scale = 0.5f;
        if((int)(0.7f * sizeScale) > 0) {
            scale = (int)(0.7f * sizeScale);
        }
        DrawableData.font.setScale(scale);
        DrawableData.font.setColor(Color.YELLOW);
        currentY += DrawableData.font.drawWrapped(DrawableData.spriteBatch,
                getName() + " (" + getManaCost() + " mana)",
                x + (3 * sizeScale), currentY, wrapX).height;
        currentY += (5 * sizeScale);
        
        DrawableData.font.setColor(Color.WHITE);
        currentY += DrawableData.font.drawWrapped(DrawableData.spriteBatch,
            getText(),
            x + (3 * sizeScale), currentY, wrapX).height;
    }
    
    public void setLevel(int level) {
        this.level = level;
    }
    
    public int getLevel() {
        return level;
    }
    
    public Type getType() {
        return type;
    }
    
    public int getUpgradeCost() {
        if(level < 3) {
            return upgradeCost[level];
        } else {
            return 0;
        }
    }
    
    public void upgrade() {
        if(level < 3) {
            level++;
        }
    }
    
    public void cast(Engine.Controls engineControls, Attackable caster) {
    }
    public void cast(Engine.Controls engineControls, Attackable caster, Vector2 position) {
    }
    public void cast(Engine.Controls engineControls, Attackable caster, Attackable target) {
    }
    public abstract boolean inRange(Vector2 position, Vector2 target);
    public abstract int getManaCost();
    public abstract TextureRegion getIcon();
    public abstract String getName();
    public abstract String getText();
}
