package com.pancik.ciernypetrzlen.engine.component.spell;

import com.badlogic.gdx.graphics.g2d.TextureRegion;
import com.badlogic.gdx.math.MathUtils;
import com.badlogic.gdx.math.Vector2;
import com.pancik.ciernypetrzlen.DrawableData;
import com.pancik.ciernypetrzlen.engine.Engine;
import com.pancik.ciernypetrzlen.engine.SoundData;
import com.pancik.ciernypetrzlen.engine.component.entity.Attackable;
import com.pancik.ciernypetrzlen.engine.component.entity.Unit;
import com.pancik.ciernypetrzlen.engine.component.particle.ParticleSpellStomp;
import java.util.List;

public class Stomp extends Spell {
    private static int[] manaLevels = {20, 20, 35, 50};
    private static int[] damageAmount = {10, 10, 60, 120};
    private static float[] pushback = {0.10f, 0.10f, 0.15f, 0.20f};
    private static float pushbackRadius = 2.0f;
    private Vector2 tmp = new Vector2();
    
    public Stomp() {
        super(new int[] {6, 31, 63}, Type.SELF);
    }

    @Override
    public void cast(Engine.Controls engineControls, Attackable caster) {
        engineControls.addEntity(new ParticleSpellStomp(caster.getPosition(), 1.0f, engineControls));
        
        Vector2 casterPosition = caster.getPosition();
        Unit unitCaster = null;
        if(caster instanceof Unit) {
            unitCaster = (Unit)caster;
        }
        
        List<Attackable> attackables = engineControls.getEntityManager().getAttackableEntities(casterPosition, pushbackRadius);
        for(Attackable a : attackables) {
            if(a.getTeam() != caster.getTeam()) {
                int damage = damageAmount[level] +  (1 + MathUtils.random(2)) * 3;
                a.getHit(damage, a);
                
                if(unitCaster != null && unitCaster.getActionListener() != null) {
                    unitCaster.getActionListener().damageDealtSpell(damage, a);
                }
                
                if(a instanceof Unit) {
                    Unit u = (Unit)a;
                    tmp.set(a.getPosition()).sub(casterPosition);
                    u.addBuff(new StompBuff(u, tmp.nor().scl(pushback[level]).cpy(), 20, engineControls.getCollisionMap()));
                }
            }
        }
        
        SoundData.playSound("stomp", 0.5f);
    }

    @Override
    public boolean inRange(Vector2 position, Vector2 target) {
        return true;
    }

    @Override
    public int getManaCost() {
        return manaLevels[level];
    }

    @Override
    public TextureRegion getIcon() {
        return DrawableData.textureAtlas.findRegion("icon-spell-stomp");
    }

    @Override
    public String getName() {
        return "Stomp";
    }

    @Override
    public String getText() {
        return "Warrior brutally stomps ground which pushes all of the units away from him and causing around " + damageAmount[level] + " damage.";
    }
}
