package com.pancik.ciernypetrzlen.engine.component.spell;

import com.badlogic.gdx.graphics.g2d.TextureRegion;
import com.badlogic.gdx.math.MathUtils;
import com.badlogic.gdx.math.Vector2;
import com.pancik.ciernypetrzlen.DrawableData;
import com.pancik.ciernypetrzlen.engine.Engine;
import com.pancik.ciernypetrzlen.engine.SoundData;
import com.pancik.ciernypetrzlen.engine.component.entity.Attackable;
import com.pancik.ciernypetrzlen.engine.component.entity.Unit;
import com.pancik.ciernypetrzlen.engine.component.entity.projectile.AxeProjectile;
import com.pancik.ciernypetrzlen.engine.component.entity.projectile.Projectile;

public class ThrowAxe extends Spell {
    private static final float RANGE = 5.0f;
    private static int[] manaLevels = {15, 15, 26, 40};
    private static int[] damageLevels = {30, 30, 130, 230};
    
    public ThrowAxe() {
        super(new int[] {3, 29, 52}, Type.TARGET);
    }

    @Override
    public void cast(Engine.Controls engineControls, Attackable caster, Attackable target) {
        final Attackable finalCaster = caster;
        final Attackable finalTarget = target;
        engineControls.addEntity(new AxeProjectile(target.getPosition(), caster.getPosition().cpy(), engineControls, new Projectile.Callback() {

            @Override
            public void onProjectileArrived(Vector2 pos) {
                int damage = damageLevels[level] +  ((1 + MathUtils.random(5)) * 5);
                finalTarget.getHit(damage, finalCaster);
                
                if(finalCaster instanceof Unit) {
                    Unit u = (Unit)finalCaster;
                    if(u.getActionListener() != null) {
                        u.getActionListener().damageDealtSpell(damage, finalTarget);
                    }
                }
            }

            @Override
            public void onProjectileCrashed(Vector2 pos) {
            }
        }));
        
        SoundData.playSound("throw-axe", 1.0f);
    }

    @Override
    public boolean inRange(Vector2 position, Vector2 target) {
        return (position.dst(target) < RANGE);
    }

    @Override
    public int getManaCost() {
        return manaLevels[level];
    }

    @Override
    public TextureRegion getIcon() {
        return DrawableData.textureAtlas.findRegion("icon-spell-throw");
    }

    @Override
    public String getName() {
        return "Throw Axe";
    }

    @Override
    public String getText() {
        return "Warrior throws axe in range of 5 meters at the target causing around " + damageLevels[level] + " damage.";
    }
}
