package com.pancik.ciernypetrzlen.engine.gui;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.Input;
import com.badlogic.gdx.InputAdapter;
import com.badlogic.gdx.graphics.Color;
import com.badlogic.gdx.graphics.g2d.TextureRegion;
import com.badlogic.gdx.utils.Disposable;
import com.pancik.ciernypetrzlen.DrawableData;
import com.pancik.ciernypetrzlen.engine.Engine;
import com.pancik.ciernypetrzlen.engine.component.entity.Hero;
import com.pancik.ciernypetrzlen.engine.component.spell.Buff;
import com.pancik.ciernypetrzlen.engine.component.spell.Spell;
import static com.pancik.ciernypetrzlen.engine.gui.UIWindow.findScale;
import com.pancik.ciernypetrzlen.engine.player.Player;
import com.pancik.ciernypetrzlen.engine.player.StatsPack;
import com.pancik.ciernypetrzlen.engine.player.inventory.Item;
import com.pancik.ciernypetrzlen.engine.player.inventory.Usable;
import com.pancik.ciernypetrzlen.util.HorizontalAlignment;
import com.pancik.ciernypetrzlen.util.RenderUtils;
import com.pancik.ciernypetrzlen.util.VerticalAlignment;

public class ActionBar extends UIWindow implements Disposable {
    private TextureRegion textureSolidWhite = DrawableData.textureAtlas.findRegion("solid-white");
    private TextureRegion textureActionBar = new TextureRegion(DrawableData.textureAtlas.findRegion("gui-actionbar"), 0, 0, 124, 33);
    
    private InputHandler inputHandler;
    private ActionBarSlot[] slots;
    private StatsPack statsPack;

    private boolean highlightMana = false;
    private int highlightManaTimeLeft = 0;
    
    public static class PersistenceData {
        PersistentSlot[] slots = new PersistentSlot[5];

        public PersistenceData() {
            for(int i = 0; i < slots.length; i++) {
                slots[i] = new PersistentSlot();
            }
        }
    }
    
    public static class PersistentSlot {
        public String spell= null;
        public boolean item = false;
        public int x;
        public int y;
    }
    
    public ActionBar(Player player, Engine.Controls engineControls, PersistenceData data) {
        super(player, engineControls, 124, 33);
        sizeScale = findScale((int)(windowSizeX * 1.5f), (int)(windowSizeY * 1.5f), Gdx.graphics.getWidth(), Gdx.graphics.getHeight());
        this.inputHandler = new InputHandler();
        engineControls.getMainControls().getInputMultiplexer().addProcessor(0, inputHandler);
        this.statsPack = player.getStatsPack();
        
        this.slots = new ActionBarSlot[5];
        for(int i = 0; i < 5; i++) {
            this.slots[i] = new ActionBarSlot(18 * (i + 1), 16);
        }
        
        if(data != null) {
            for(int i = 0; i < slots.length; i++) {
                if(data.slots[i].spell != null) {
                    slots[i].spell = player.getCharacterWindow().getSpell(data.slots[i].spell);
                    /*
                    try {
                        Class clazz = Class.forName(data.slots[i].spell);
                        slots[i].spell = (Spell)clazz.newInstance();
                    } catch (Exception e) {
                        System.err.println("Loading a spell (" + data.slots[i].spell + ") has failed.");
                        e.printStackTrace(System.err);
                    }
                    */
                }
                if(data.slots[i].item) {
                    Item item = player.getInventory().getItem(data.slots[i].x, data.slots[i].y);;
                    if(item instanceof Usable) {
                        slots[i].item = (Usable)item;
                    }
                }
            }
        }
    }

    @Override
    public void resize(int sizeX, int sizeY) {
        sizeScale = findScale((int)(windowSizeX * 1.5f), (int)(windowSizeY * 1.5f), sizeX, sizeY);
    }

    @Override
    public void setVisibility(boolean visible) {
        super.setVisibility(visible);
        
        if(visible == true) {
            Usable[] inventoryItems = player.getInventory().getUsableItems();
            boolean[] exists = new boolean[5];

            for (int j = 0; j < inventoryItems.length; j++) {
                for(int i = 0; i < slots.length; i++) {
                    if(slots[i].item == inventoryItems[j]) {
                        exists[i] = true;
                    }
                    if(slots[i].spell != null) {
                        exists[i] = true;
                    }
                }
            }
            
            for(int i = 0; i < exists.length; i++) {
                if(!exists[i]) {
                    slots[i].item = null;
                }
            }
        }
    }
    
    @Override
    public void tick() {
        if(visible) {
        }
    }

    @Override
    public void renderUI() {
        if(highlightMana) {
            highlightManaTimeLeft--;
            if(highlightManaTimeLeft <= 0) {
                highlightMana = false;
            }
        }
        
        if(visible) {
            float mouseX = Gdx.input.getX();
            float mouseY = Gdx.input.getY();
            int topLeftX = getWindowTopLeftX();
            int topLeftY = getWindowTopLeftY();
            Hero hero = player.getHero();
            
            RenderUtils.blit(textureActionBar,
                    HorizontalAlignment.MIDDLE, 0,
                    VerticalAlignment.BOTTOM, 0,
                    124 * sizeScale, 33 * sizeScale,
                    0.0f,
                    false);
            
            float health = hero.getPercentualHealth();
            if(health < 0.0f) {
                health = 0.0f;
            }
            TextureRegion region = DrawableData.textureAtlas.findRegion("gui-actionbar");
            DrawableData.spriteBatch.draw(region.getTexture(),
                    topLeftX + 2 * sizeScale, topLeftY + 2 * sizeScale + (30.0f * (1.0f - health) * sizeScale), 0, 0,
                    14 * sizeScale, ((30.0f * health) * sizeScale), 1.0f, 1.0f, 0.0f, region.getRegionX() + 124, region.getRegionY(), 14, (int)(30.0f * health),
                    false, true);
            
            float mana = hero.getPercentualMana();
            if(mana < 0.0f) {
                mana = 0.0f;
            }
            region = DrawableData.textureAtlas.findRegion("gui-actionbar");
            DrawableData.spriteBatch.draw(region.getTexture(),
                    topLeftX + 108 * sizeScale, topLeftY + 2 * sizeScale + (30.0f * (1.0f - mana) * sizeScale), 0, 0,
                    14 * sizeScale, ((30.0f * mana) * sizeScale), 1.0f, 1.0f, 0.0f, region.getRegionX() + 138, region.getRegionY(), 14, (int)(30.0f * mana),
                    false, true);
            
            float experience = hero.getPercentualExperience();
            if(experience < 0.0f) {
                experience = 0.0f;
            }
            region = DrawableData.textureAtlas.findRegion("gui-actionbar");
            DrawableData.spriteBatch.draw(region.getTexture(),
                    topLeftX + 18 * sizeScale, topLeftY + 10 * sizeScale, 0, 0,
                    88.0f * experience * sizeScale, 4 * sizeScale, 1.0f, 1.0f, 0.0f, region.getRegionX(), region.getRegionY() + 33, (int)(88.0f * experience), 4,
                    false, true);
            
            DrawableData.font.setScale(Math.max(Math.round(getFontScale()/2.0f), 0.5f));
            RenderUtils.blitText(hero.getHealth() + "",
                        topLeftX + (9 * sizeScale),
                        topLeftY + (30 * sizeScale),
                        RenderUtils.TextAlligment.MIDDLE, RenderUtils.TextAlligment.MIDDLE);
            
            RenderUtils.blitText(hero.getMana() + "",
                        topLeftX + (115 * sizeScale),
                        topLeftY + (30 * sizeScale),
                        highlightMana ? Color.RED : Color.WHITE,
                        RenderUtils.TextAlligment.MIDDLE, RenderUtils.TextAlligment.MIDDLE);
            
            RenderUtils.blitText(statsPack.getCurrentExperience() + "/" + statsPack.getRequiredExperience(),
                        topLeftX + (62 * sizeScale),
                        topLeftY + (12 * sizeScale),
                        RenderUtils.TextAlligment.MIDDLE, RenderUtils.TextAlligment.MIDDLE);
            
            DrawableData.font.setScale(getFontScale());
            ActionBarSlot tooltip = null;
            for(int i = 0; i < slots.length; i++) {
                slots[i].render();
                RenderUtils.blitText((i + 1) + "",
                        topLeftX + ((slots[i].posx + 15) * sizeScale),
                        topLeftY + ((slots[i].posy + 15) * sizeScale),
                        RenderUtils.TextAlligment.RIGHT, RenderUtils.TextAlligment.TOP);
                
                if(player.getCastedSpell() == slots[i].spell && slots[i].spell != null) {
                    slots[i].casting();
                } else {
                    if (slots[i].hits(mouseX, mouseY)) {
                        slots[i].highlight();
                        tooltip = slots[i];
                    } else if(slots[i].spell != null && slots[i].spell.getManaCost() > player.getHero().getMana()) {
                        slots[i].noMana();
                    }
                }
            }
            
            //buffs
            int index = 0;
            for(Buff b : hero.getBuffs().values()) {
                if(b != null) {
                    RenderUtils.blit(b.getIcon(),
                            topLeftX + (19 + (9 * index)) * sizeScale, topLeftY + -1 * sizeScale, 8 * sizeScale, 8 * sizeScale, 0.0f, false);
                    if(mouseX > topLeftX + (19 + (9 * index)) * sizeScale && mouseX < topLeftX + (27 + (9 * index)) * sizeScale && mouseY > topLeftY + -1 * sizeScale && mouseY < topLeftY + 7 * sizeScale) {
                        RenderUtils.blitText(b.getName(),
                                topLeftX + (23 + (9 * index)) * sizeScale,
                                topLeftY + -3 * sizeScale,
                                Color.WHITE,
                                RenderUtils.TextAlligment.MIDDLE, RenderUtils.TextAlligment.TOP);
                    }

                    index++;
                }
            }
            
            if(tooltip != null) {
                if(tooltip.spell != null) {
                    tooltip.spell.renderTooltip(mouseX, mouseY, sizeScale);
                }
                if(tooltip.item != null) {
                    tooltip.item.renderTooltip(mouseX, mouseY, sizeScale);
                }
            }
            
            if(player.isDead()) {
                DrawableData.font.setScale(getFontScale());
                RenderUtils.blitText("You have died. Press the mouse to load the game at the start of the level.",
                        HorizontalAlignment.MIDDLE, 0,
                        VerticalAlignment.MIDDLE, 0,
                        RenderUtils.TextAlligment.MIDDLE, RenderUtils.TextAlligment.MIDDLE);
            }
        }
    }
    
    public ActionBarSlot[] getSlots() {
        return slots;
    }

    public PersistenceData getPersistenceData() {
        PersistenceData out = new PersistenceData();
        for (int i = 0; i < slots.length; i++) {
            if (slots[i].spell != null) {
                out.slots[i].spell = slots[i].spell.getClass().getName();
            }
            if (slots[i].item != null) {
                out.slots[i].item = true;
                out.slots[i].x = player.getInventory().getItemX(slots[i].item);
                out.slots[i].y = player.getInventory().getItemY(slots[i].item);
            }
        }
        return out;
    }
    
    @Override
    public void dispose() {
        engineControls.getMainControls().getInputMultiplexer().removeProcessor(inputHandler);
    }

    @Override
    protected int getWindowTopLeftX() {
        return Gdx.graphics.getWidth() / 2 - (windowSizeX/2 * sizeScale);
    }

    @Override
    protected int getWindowTopLeftY() {
        return Gdx.graphics.getHeight() - (windowSizeY * sizeScale);
    }
    
    public void useSlot(int index) {
        if (slots[index].spell != null) {
            if(player.getCastedSpell() == slots[index].spell) {
                player.castSpell(null);
            } else {
                if (player.getHero().getMana() >= slots[index].spell.getManaCost()) {
                    player.castSpell(slots[index].spell);
                } else {
                    highlightMana = true;
                    highlightManaTimeLeft = 60;
                }
            }
        }
        if (slots[index].item != null) {
            slots[index].item.use(player.getHero());
            player.getInventory().removeItem(slots[index].item);
            slots[index].item = null;
        }
    }

    private class InputHandler extends InputAdapter {
        @Override
        public boolean touchDown(int screenX, int screenY, int pointer, int button) {
            if(visible) {
                for(int i = 0; i < slots.length; i++) {
                    if(slots[i].hits(screenX, screenY)) {
                        useSlot(i);
                        return true;
                    }
                }
            }
            return false;
        }

        @Override
        public boolean keyDown(int keycode) {
            if(keycode == Input.Keys.NUM_1) {
                useSlot(0);
                return true;
            }
            if(keycode == Input.Keys.NUM_2) {
                useSlot(1);
                return true;
            }
            if(keycode == Input.Keys.NUM_3) {
                useSlot(2);
                return true;
            }
            if(keycode == Input.Keys.NUM_4) {
                useSlot(3);
                return true;
            }
            if(keycode == Input.Keys.NUM_5) {
                useSlot(4);
                return true;
            }
            return false;
        }
    }
    
    public class ActionBarSlot {
        public Usable item = null;
        public Spell spell = null;
        public int posx;
        public int posy;

        public ActionBarSlot(int posx, int posy) {
            this.posx = posx;
            this.posy = posy;
        }

        public void render() {
            int newX = getWindowTopLeftX() + (posx * sizeScale);
            int newY = getWindowTopLeftY() + (posy * sizeScale);
            int size = 16 * sizeScale;
            if(item != null) {
                item.render(newX, newY, size);
            }
            if(spell != null) {
                spell.render(newX, newY, size);
            }
        }
        
        public void highlight() {
            int newX = getWindowTopLeftX() + (posx * sizeScale);
            int newY = getWindowTopLeftY() + (posy * sizeScale);
            int size = 16 * sizeScale;
            DrawableData.spriteBatch.setColor(1.0f, 1.0f, 1.0f, 0.3f);
            DrawableData.spriteBatch.draw(textureSolidWhite, newX, newY, size, size);
            DrawableData.spriteBatch.setColor(1.0f, 1.0f, 1.0f, 1.0f);
        }
        
        public void casting() {
            int newX = getWindowTopLeftX() + (posx * sizeScale);
            int newY = getWindowTopLeftY() + (posy * sizeScale);
            int size = 16 * sizeScale;
            DrawableData.spriteBatch.setColor(1.0f, 0.0f, 0.0f, 0.3f);
            DrawableData.spriteBatch.draw(textureSolidWhite, newX, newY, size, size);
            DrawableData.spriteBatch.setColor(1.0f, 1.0f, 1.0f, 1.0f);
        }
        
        public void noMana() {
            int newX = getWindowTopLeftX() + (posx * sizeScale);
            int newY = getWindowTopLeftY() + (posy * sizeScale);
            int size = 16 * sizeScale;
            DrawableData.spriteBatch.setColor(0.0f, 0.0f, 1.0f, 0.3f);
            DrawableData.spriteBatch.draw(textureSolidWhite, newX, newY, size, size);
            DrawableData.spriteBatch.setColor(1.0f, 1.0f, 1.0f, 1.0f);
        }
        
        public boolean hits(float x, float y) {
            int newX = getWindowTopLeftX() + (posx * sizeScale);
            int newY = getWindowTopLeftY() + (posy * sizeScale);
            int size = 16 * sizeScale;
            return (x > newX && x < newX + size && y > newY && y < newY + size);
        }
    }
}
