package com.pancik.ciernypetrzlen.engine.gui;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.InputAdapter;
import com.badlogic.gdx.graphics.Color;
import com.badlogic.gdx.graphics.g2d.TextureRegion;
import com.badlogic.gdx.utils.Disposable;
import com.pancik.ciernypetrzlen.DrawableData;
import com.pancik.ciernypetrzlen.engine.Engine;
import com.pancik.ciernypetrzlen.engine.component.spell.Fireball;
import com.pancik.ciernypetrzlen.engine.component.spell.Heal;
import com.pancik.ciernypetrzlen.engine.component.spell.Roar;
import com.pancik.ciernypetrzlen.engine.component.spell.Spell;
import com.pancik.ciernypetrzlen.engine.component.spell.Stomp;
import com.pancik.ciernypetrzlen.engine.component.spell.ThrowAxe;
import com.pancik.ciernypetrzlen.engine.player.Player;
import com.pancik.ciernypetrzlen.engine.player.StatsPack;
import com.pancik.ciernypetrzlen.engine.player.inventory.Usable;
import com.pancik.ciernypetrzlen.util.HorizontalAlignment;
import com.pancik.ciernypetrzlen.util.RenderUtils;
import com.pancik.ciernypetrzlen.util.VerticalAlignment;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Locale;

public class CharacterWindow extends UIWindow implements Disposable {
    private TextureRegion textureSolidWhite = DrawableData.textureAtlas.findRegion("solid-white");
    private TextureRegion textureCharacter = new TextureRegion(DrawableData.textureAtlas.findRegion("gui-character"), 0, 0, 148, 128);
    private TextureRegion textureButtonUpMinus = new TextureRegion(DrawableData.textureAtlas.findRegion("gui-character"), 148, 0, 10, 10);
    private TextureRegion textureButtonDownMinus = new TextureRegion(DrawableData.textureAtlas.findRegion("gui-character"), 148, 10, 10, 10);
    private TextureRegion textureButtonUpPlus = new TextureRegion(DrawableData.textureAtlas.findRegion("gui-character"), 148, 20, 10, 10);
    private TextureRegion textureButtonDownPlus = new TextureRegion(DrawableData.textureAtlas.findRegion("gui-character"), 148, 30, 10, 10);
    private TextureRegion textureScrollButton = new TextureRegion(DrawableData.textureAtlas.findRegion("gui-character"), 148, 40, 8, 9);
    private TextureRegion textureButtonBuyUp = new TextureRegion(DrawableData.textureAtlas.findRegion("gui-character"), 148, 49, 17, 9);
    private TextureRegion textureButtonBuyDown = new TextureRegion(DrawableData.textureAtlas.findRegion("gui-character"), 148, 58, 17, 9);
    private TextureRegion textureLevelEmpty = new TextureRegion(DrawableData.textureAtlas.findRegion("gui-character"), 148, 67, 8, 8);
    private TextureRegion textureLevelFull = new TextureRegion(DrawableData.textureAtlas.findRegion("gui-character"), 156, 67, 8, 8);
    
    private InputHandler inputHandler;
    private ArrayList<Button> buttons = new ArrayList<Button>();
    private Scrollbar spellScrollBar;
    private Scrollbar itemScrollBar;
    private Slot[] slots;
    private ActionBar.ActionBarSlot[] actionBarSlots;
    private int spellsCurrentTop;
    private SpellPack[] spells = {
        new SpellPack(new Roar()),
        new SpellPack(new ThrowAxe()),
        new SpellPack(new Fireball()),
        new SpellPack(new Heal()),
        new SpellPack(new Stomp())
    };
    private int itemsCurrentTop;
    private ItemPack[] items;
    
    private boolean mousePressed = false;
    private Scrollbar lockedScrollBar = null;
    private Spell pickedSpell = null;
    private Usable pickedItem = null;

    private StatsPack statsPack;
    
    public CharacterWindow(Player player, Engine.Controls engineControls) {
        super(player, engineControls, 148, 128);
        this.inputHandler = new InputHandler();
        engineControls.getMainControls().getInputMultiplexer().addProcessor(0, inputHandler);
        this.statsPack = player.getStatsPack();
        
        spellScrollBar = new Scrollbar(83, 53, 51);
        itemScrollBar = new Scrollbar(134, 53, 51);
        
        int minusButtonX = 35;
        int plusButtonX = 55;
        buttons.add(new PlusMinusButton(plusButtonX, 20, false, new ButtonCallback() {
            @Override
            public void onClick() {
                statsPack.changePointAttack(1);
            }
        }));
        
        buttons.add(new PlusMinusButton(minusButtonX, 20, true, new ButtonCallback() {
            @Override
            public void onClick() {
                statsPack.changePointAttack(-1);
            }
        }));
        
        buttons.add(new PlusMinusButton(plusButtonX, 30, false, new ButtonCallback() {
            @Override
            public void onClick() {
                statsPack.changePointDefense(1);
            }
        }));
        
        buttons.add(new PlusMinusButton(minusButtonX, 30, true, new ButtonCallback() {
            @Override
            public void onClick() {
                statsPack.changePointDefense(-1);
            }
        }));
        
        buttons.add(new PlusMinusButton(plusButtonX, 40, false, new ButtonCallback() {
            @Override
            public void onClick() {
                statsPack.changePointVitality(1);
            }
        }));
        
        buttons.add(new PlusMinusButton(minusButtonX, 40, true, new ButtonCallback() {
            @Override
            public void onClick() {
                statsPack.changePointVitality(-1);
            }
        }));

        HashMap<String, Integer> spellLevels = statsPack.getSpellLevels();
        for(int i = 0; i < spells.length; i++) {
            if(spellLevels.containsKey(spells[i].spell.getClass().getName())) {
                spells[i].spell.setLevel(spellLevels.get(spells[i].spell.getClass().getName()));
            }
        }
        updateSpellBar();
        for(SpellPack sp : spells) {
            buttons.add(sp.button);
        }
        
        slots = new Slot[5];
        for(int i = 0; i < 5; i++) {
            slots[i] = new Slot(30 + (i * 18), 108);
        }
    }

    public void setActionBarSlots(ActionBar.ActionBarSlot[] slots) {
        this.actionBarSlots = slots;
    }
    
    @Override
    public void setVisibility(boolean visible) {
        super.setVisibility(visible);
        
        if(visible == true) {
            updateItemBar();
        } else {
            mousePressed = false;
            lockedScrollBar = null;
        }
    }

    @Override
    public void resize(int sizeX, int sizeY) {
        super.resize(sizeX, sizeY);
        
        updateSpellBar();
        updateItemBar();
    }

    @Override
    public void tick() {
        if(visible) {
        }
    }
    
    @Override
    public void renderUI() {
        if(visible) {
            float mouseX = Gdx.input.getX();
            float mouseY = Gdx.input.getY();
            int topLeftX = getWindowTopLeftX();
            int topLeftY = getWindowTopLeftY();
            
            RenderUtils.blit(textureCharacter,
                    HorizontalAlignment.MIDDLE, 0,
                    VerticalAlignment.MIDDLE, 0,
                    148 * sizeScale, 128 * sizeScale,
                    0.0f,
                    false);

            float fontSize = getFontScale();
            if(sizeScale > 2) {
                fontSize += 1.0f;
            }
            DrawableData.font.setScale(fontSize);
            RenderUtils.blitText("LEVEL " + statsPack.getLevel() + " WARRIOR",
                    topLeftX + (74 * sizeScale) + 1,
                    topLeftY + (8 * sizeScale) + 1,
                    Color.BLACK,
                    RenderUtils.TextAlligment.MIDDLE,
                    RenderUtils.TextAlligment.MIDDLE);
            
            RenderUtils.blitText("LEVEL " + statsPack.getLevel() + " WARRIOR",
                    topLeftX + (74 * sizeScale),
                    topLeftY + (8 * sizeScale),
                    RenderUtils.TextAlligment.MIDDLE,
                    RenderUtils.TextAlligment.MIDDLE);
            
            int statsX = 5;
            int statValsX = 50;
            int equipmentValsX = 65;
            int[] statVals = {statsPack.getPointsAttack(), statsPack.getPointsDefense(), statsPack.getPointsVitality()};
            int[] equipmentVals = {statsPack.getEquipmentPointsAttack(), statsPack.getEquipmentPointsDefense(), statsPack.getEquipmentPointsVitality()};
            int[] statsY = {25, 35, 45};
            String[] statsText = {"ATTACK", "DEFENSE", "VITALITY"};
            
            DrawableData.font.setScale(getFontScale());
            RenderUtils.blitText("POINTS LEFT: " + statsPack.getPointsFree(),
                    topLeftX + (statValsX * sizeScale),
                    topLeftY + (16 * sizeScale),
                    RenderUtils.TextAlligment.MIDDLE,
                    RenderUtils.TextAlligment.MIDDLE);
            
            for(int i = 0; i < statsY.length; i++) {
                RenderUtils.blitText(statsText[i],
                        topLeftX + (statsX * sizeScale),
                        topLeftY + (statsY[i] * sizeScale),
                        RenderUtils.TextAlligment.LEFT,
                        RenderUtils.TextAlligment.MIDDLE);
                
                RenderUtils.blitText(Integer.toString(statVals[i]),
                        topLeftX + (statValsX * sizeScale),
                        topLeftY + (statsY[i] * sizeScale),
                        RenderUtils.TextAlligment.MIDDLE,
                        RenderUtils.TextAlligment.MIDDLE);
                
                RenderUtils.blitText("  + " + equipmentVals[i],
                        topLeftX + (equipmentValsX * sizeScale),
                        topLeftY + (statsY[i] * sizeScale),
                        new Color(0.0f, 0.5f, 0.0f, 1.0f),
                        RenderUtils.TextAlligment.LEFT,
                        RenderUtils.TextAlligment.MIDDLE);
            }

            int beginY = 23;
            DrawableData.font.setScale(Math.max(Math.round(getFontScale()/2.0f), 0.5f));
            String text = sizeScale > 2 ? "Health/Mana: " : "H/M:";
            RenderUtils.blitText(text + statsPack.getHitpoints() + "(+" + statsPack.getHitpointRegeneration()
                    + ")/" + statsPack.getMana() + "(+" + statsPack.getManaRegeneration() + ")",
                        topLeftX + (87 * sizeScale),
                        topLeftY + (beginY * sizeScale),
                        RenderUtils.TextAlligment.LEFT,
                        RenderUtils.TextAlligment.BOTTOM);
            text = sizeScale > 2 ? "Damage reduction: " : "Dmg. red.:";
            RenderUtils.blitText(text + String.format(Locale.US, "%.2f", statsPack.getPercentualDefense()) + "%",
                        topLeftX + (87 * sizeScale),
                        topLeftY + ((beginY + 5) * sizeScale),
                        RenderUtils.TextAlligment.LEFT,
                        RenderUtils.TextAlligment.BOTTOM);
            RenderUtils.blitText("Attack: " + statsPack.getLowAttackDamage() + "-" + statsPack.getHighAttackDamage(),
                        topLeftX + (87 * sizeScale),
                        topLeftY + ((beginY + 10) * sizeScale),
                        RenderUtils.TextAlligment.LEFT,
                        RenderUtils.TextAlligment.BOTTOM);
            RenderUtils.blitText("Experience: " + statsPack.getCurrentExperience() + "/" + statsPack.getRequiredExperience(),
                        topLeftX + (87 * sizeScale),
                        topLeftY + ((beginY + 15) * sizeScale),
                        RenderUtils.TextAlligment.LEFT,
                        RenderUtils.TextAlligment.BOTTOM);
            RenderUtils.blitText("Gold: " + statsPack.getGold(),
                        topLeftX + (87 * sizeScale),
                        topLeftY + ((beginY + 20) * sizeScale),
                        RenderUtils.TextAlligment.LEFT,
                        RenderUtils.TextAlligment.BOTTOM);
            
            //spellbar
            Spell tooltipSpell = null;
            int[] circleX = {28 * sizeScale, 38 * sizeScale, 48 * sizeScale};
            int[] circleY = {58 * sizeScale, 75 * sizeScale, 92 * sizeScale};
            int[] buttonY = {61, 78, 95};
            for(int j = spellsCurrentTop; j < spellsCurrentTop + 3 && j < spells.length; j++) {
                int indexJ = j - spellsCurrentTop;
                
                spells[j].render();
                if(spells[j].hits(mouseX, mouseY)) {
                    spells[j].highlight();
                    tooltipSpell = spells[j].spell;
                }
                
                for(int i = 0; i < 3; i++) {
                    if(spells[j].spell.getLevel() > i) {
                        RenderUtils.blit(textureLevelFull, topLeftX + circleX[i], topLeftY + circleY[indexJ], 8 * sizeScale, 8 * sizeScale, 0.0f, false);
                    } else {
                        RenderUtils.blit(textureLevelEmpty, topLeftX + circleX[i], topLeftY + circleY[indexJ], 8 * sizeScale, 8 * sizeScale, 0.0f, false);
                    }
                }
                
                if(spells[j].spell.getLevel() < 3) {
                    int cost = spells[j].spell.getUpgradeCost();
                    RenderUtils.blitText("Cost: " + spells[j].spell.getUpgradeCost(),
                            topLeftX + (61 * sizeScale),
                            topLeftY + ((buttonY[indexJ] - 2) * sizeScale),
                            statsPack.getGold() >= cost ? Color.WHITE : Color.RED,
                            RenderUtils.TextAlligment.LEFT,
                            RenderUtils.TextAlligment.TOP);
                }
            }
            
            Usable tooltipItem = null;
            for(int i = itemsCurrentTop; i < itemsCurrentTop + 6 && i < items.length; i++) {
                items[i].render();
                if(items[i].hits(mouseX, mouseY)) {
                    items[i].highlight();
                    tooltipItem = items[i].item;
                }
            }
            
            for(Button b : buttons) {
                if(mousePressed && b.hits(mouseX, mouseY)) {
                    b.clicked = true;
                } else {
                    b.clicked = false;
                }

                b.render();

                if(!mousePressed && b.hits(mouseX, mouseY)) {
                    b.highlight();
                }
            }
            
            for(int i = 0; i < slots.length; i++) {
                slots[i].render(i);

                if(slots[i].hits(mouseX, mouseY)) {
                    slots[i].highlight();
                    if(actionBarSlots[i].spell != null) {
                        tooltipSpell = actionBarSlots[i].spell;
                    }
                }
            }
            
            spellScrollBar.render();
            itemScrollBar.render();
            
            if(pickedSpell != null) {
                int size = 16 * sizeScale;
                pickedSpell.render((int)mouseX - size/2, (int)mouseY - size/2, size);
            }
            if(pickedItem != null) {
                int size = 16 * sizeScale;
                pickedItem.render((int)mouseX - size/2, (int)mouseY - size/2, size);
            }
            
            if(tooltipSpell != null) {
                tooltipSpell.renderTooltip(mouseX, mouseY, sizeScale);
            }
            if(tooltipItem != null) {
                tooltipItem.renderTooltip(mouseX, mouseY, sizeScale);
            }
        }
    }
    
    @Override
    public void dispose() {
        engineControls.getMainControls().getInputMultiplexer().removeProcessor(inputHandler);
    }

    private void updateSpellBar() {
        float singleItem = 100.0f/(float)spells.length;
        spellsCurrentTop = ((int)(spellScrollBar.getPercentualPosition()/singleItem)/3)*3;
        
        int posX = 60;
        int[] posY = {61, 78, 95};
        int spellIconX = 8 * sizeScale;
        int[] spellIconY = {54 * sizeScale, 71 * sizeScale, 88 * sizeScale};
        for(int i = 0; i < spells.length; i++) {
            int index = i - spellsCurrentTop;
            if(index >= 0 && index < 3) {
                spells[i].button.posx = posX;
                spells[i].button.posy = posY[index];
                spells[i].spellPosX = getWindowTopLeftX() + spellIconX;
                spells[i].spellPosY = getWindowTopLeftY() + spellIconY[index];
                if(spells[i].spell.getLevel() < 3) {
                    spells[i].button.visible = true;
                } else {
                    spells[i].button.visible = false;
                }
            } else {
                spells[i].button.visible = false;
            }
        }
    }
    
    private void updateItemBar() {
        Usable[] inventoryItems = player.getInventory().getUsableItems();
        boolean[] exists = new boolean[5];
        
        items = new ItemPack[inventoryItems.length];
        for (int j = 0; j < items.length; j++) {
            items[j] = new ItemPack(inventoryItems[j]);
            
            for(int i = 0; i < actionBarSlots.length; i++) {
                if(actionBarSlots[i].item == inventoryItems[j]) {
                    exists[i] = true;
                }
                if(actionBarSlots[i].spell != null) {
                    exists[i] = true;
                }
            }
        }
        
        for(int i = 0; i < exists.length; i++) {
            if(!exists[i]) {
                actionBarSlots[i].item = null;
            }
        }
        
        
        float singleItem = 100.0f/(float)items.length;
        itemsCurrentTop = ((int)(itemScrollBar.getPercentualPosition()/singleItem)/6)*6;
        
        int[] posX = {96 * sizeScale, 113 * sizeScale};
        int[] posY = {54 * sizeScale, 54 * sizeScale, 71 * sizeScale, 71 * sizeScale, 88 * sizeScale, 88 * sizeScale};
        for(int i = 0; i < items.length; i++) {
            int index = i - itemsCurrentTop;
            if(index >= 0 && index < 6) {
                items[i].itemPosX = getWindowTopLeftX() + posX[index % 2];
                items[i].itemPosY = getWindowTopLeftY() + posY[index];
                
                items[i].used = false;
                for(ActionBar.ActionBarSlot slots : actionBarSlots) {
                    if(slots.item == items[i].item) {
                        items[i].used = true;
                        break;
                    }
                }
            }
        }
    }
    
    public Spell getSpell(String spellClassFullName) {
        for(SpellPack spellPack : spells) {
            if(spellPack.spell != null && spellPack.spell.getClass().getName().equals(spellClassFullName)) {
                return spellPack.spell;
            }
        }
        
        return null;
    }
    
    private class InputHandler extends InputAdapter {
        @Override
        public boolean touchDown(int screenX, int screenY, int pointer, int button) {
            if(visible) {
                if(pickedSpell == null && pickedItem == null) {
                    if(spellScrollBar.hits(screenX, screenY)) {
                        lockedScrollBar = spellScrollBar;
                    }
                    if(itemScrollBar.hits(screenX, screenY)) {
                        lockedScrollBar = itemScrollBar;
                    }

                    for(int i = spellsCurrentTop; i < spellsCurrentTop + 3 && i < spells.length; i++) {
                        if(spells[i].hits(screenX, screenY) && spells[i].spell.getLevel() > 0) {
                            pickedSpell = spells[i].spell;
                        }
                    }
                    
                    for(int i = itemsCurrentTop; i < itemsCurrentTop + 6 && i < items.length; i++) {
                        if(items[i].hits(screenX, screenY) && !items[i].used) {
                            pickedItem = items[i].item;
                            items[i].used = true;
                        }
                    }
                    
                    for(int i = 0; i < actionBarSlots.length; i++) {
                        if(slots[i].hits(screenX, screenY)) {
                            pickedItem = actionBarSlots[i].item;
                            pickedSpell = actionBarSlots[i].spell;
                            actionBarSlots[i].item = null;
                            actionBarSlots[i].spell = null;
                        }
                    }
                    
                    mousePressed = true;
                } else {
                    for(int i = 0; i < actionBarSlots.length; i++) {
                        if(slots[i].hits(screenX, screenY)) {
                            actionBarSlots[i].item = pickedItem;
                            actionBarSlots[i].spell = pickedSpell;
                        }
                    }
                    
                    updateItemBar();
                    pickedSpell = null;
                    pickedItem = null;
                }
            }
            return false;
        }

        @Override
        public boolean touchUp(int screenX, int screenY, int pointer, int button) {
            if(visible) {
                if(mousePressed && lockedScrollBar == null) {
                    for(Button b : buttons) {
                        if(b.hits(screenX, screenY)) {
                            b.callback.onClick();
                        }
                    }
                }

                lockedScrollBar = null;
                mousePressed = false;
            }
            return false;
        }

        @Override
        public boolean touchDragged(int screenX, int screenY, int pointer) {
            if(visible) {
                if(lockedScrollBar != null) {
                    lockedScrollBar.handleMouseY(screenY);

                    if(lockedScrollBar == spellScrollBar) {
                        updateSpellBar();
                    }
                    if(lockedScrollBar == itemScrollBar) {
                        updateItemBar();
                    }
                }
            }
            return false;
        }
    }
    
    private class ItemPack {
        public Usable item;
        public int itemPosX;
        public int itemPosY;
        public boolean used = false;

        public ItemPack(Usable item) {
            this.item = item;
        }
        
        public void render() {
            if(used) {
                DrawableData.spriteBatch.setColor(0.3f, 0.3f, 0.3f, 1.0f);
            }
            int size = 16 * sizeScale;
            item.render(itemPosX, itemPosY, size);
            DrawableData.spriteBatch.setColor(1.0f, 1.0f, 1.0f, 1.0f);
        }
        
        public void highlight() {
            if(!used) {
                int size = 16 * sizeScale;
                DrawableData.spriteBatch.setColor(1.0f, 1.0f, 1.0f, 0.3f);
                DrawableData.spriteBatch.draw(textureSolidWhite, itemPosX, itemPosY, size, size);
                DrawableData.spriteBatch.setColor(1.0f, 1.0f, 1.0f, 1.0f);
            }
        }
        
        public boolean hits(float mouseX, float mouseY) {
            return (mouseX > itemPosX && mouseX < itemPosX + (16 * sizeScale) && mouseY > itemPosY && mouseY < itemPosY + (16 * sizeScale)) && !used;
        }
    }
    
    private class SpellPack {
        public Spell spell;
        public BuyButton button;
        public int spellPosX;
        public int spellPosY;

        public SpellPack(Spell spell) {
            this.spell = spell;
            
            this.button = new BuyButton(0, 0, new ButtonCallback() {

                @Override
                public void onClick() {
                    if(SpellPack.this.spell.getUpgradeCost() <= statsPack.getGold()) {
                        statsPack.changeGold(-SpellPack.this.spell.getUpgradeCost());
                        SpellPack.this.spell.upgrade();
                        statsPack.getSpellLevels().put(SpellPack.this.spell.getClass().getName(), SpellPack.this.spell.getLevel());
                    }
                    updateSpellBar();
                }
            });
        }
        
        public void render() {
            if(spell.getLevel() == 0) {
                DrawableData.spriteBatch.setColor(0.3f, 0.3f, 0.3f, 1.0f);
            }
            int size = 16 * sizeScale;
            spell.render(spellPosX, spellPosY, size);
            DrawableData.spriteBatch.setColor(1.0f, 1.0f, 1.0f, 1.0f);
        }
        
        public void highlight() {
            int size = 16 * sizeScale;
            DrawableData.spriteBatch.setColor(1.0f, 1.0f, 1.0f, 0.3f);
            DrawableData.spriteBatch.draw(textureSolidWhite, spellPosX, spellPosY, size, size);
            DrawableData.spriteBatch.setColor(1.0f, 1.0f, 1.0f, 1.0f);
        }
        
        public boolean hits(float mouseX, float mouseY) {
            return (mouseX > spellPosX && mouseX < spellPosX + (16 * sizeScale) && mouseY > spellPosY && mouseY < spellPosY + (16 * sizeScale));
        }
    }
    
    protected class Scrollbar {
        public int posx;
        public int posy;
        public int height;
        
        public int currentY = 4;
        
        public Scrollbar(int posx, int posy, int height) {
            this.posx = posx;
            this.posy = posy;
            this.height = height - 5;
        }
        
        public void render() {
            int newX = getWindowTopLeftX() + (posx * sizeScale);
            int newY = getWindowTopLeftY() + (posy * sizeScale) + (currentY * sizeScale) - (4 * sizeScale);
            int sizeX = 8 * sizeScale;
            int sizeY = 9 * sizeScale;
            
            RenderUtils.blit(textureScrollButton, newX, newY, sizeX, sizeY, 0.0f, false);
        }

        public boolean hits(float x, float y) {
            int newX = getWindowTopLeftX() + (posx * sizeScale);
            int newY = getWindowTopLeftY() + ((posy + currentY - 4) * sizeScale) ;
            int sizeX = 8 * sizeScale;
            int sizeY = 9 * sizeScale;
            return (x > newX && x < newX + sizeX && y > newY && y < newY + sizeY);
        }
        
        public void handleMouseY(float y) {
            int minY = 4;
            int maxY = height;
            y -= (posy * sizeScale) + getWindowTopLeftY();
            y /= sizeScale;
            if(y < minY) {
                currentY = minY;
            } else if(y >= maxY) {
                currentY = maxY;
            } else {
                currentY = (int)y;
            }
        }
        
        public float getPercentualPosition() {
            return ((float)(currentY - 4)/(float)(height - 4)) * 100.0f;
        }
    }
    
    protected class Slot {
        public int posx;
        public int posy;

        public Slot(int posx, int posy) {
            this.posx = posx;
            this.posy = posy;
        }
        
        public void render(int index) {
            int newX = getWindowTopLeftX() + (posx * sizeScale);
            int newY = getWindowTopLeftY() + (posy * sizeScale);
            int size = 16 * sizeScale;
            if(actionBarSlots[index].item != null) {
                actionBarSlots[index].item.render(newX, newY, size);
            }
            if(actionBarSlots[index].spell != null) {
                actionBarSlots[index].spell.render(newX, newY, size);
            }
        }
        
        public void highlight() {
            int newX = getWindowTopLeftX() + (posx * sizeScale);
            int newY = getWindowTopLeftY() + (posy * sizeScale);
            int size = 16 * sizeScale;
            DrawableData.spriteBatch.setColor(1.0f, 1.0f, 1.0f, 0.3f);
            DrawableData.spriteBatch.draw(textureSolidWhite, newX, newY, size, size);
            DrawableData.spriteBatch.setColor(1.0f, 1.0f, 1.0f, 1.0f);
        }
        
        public boolean hits(float x, float y) {
            int newX = getWindowTopLeftX() + (posx * sizeScale);
            int newY = getWindowTopLeftY() + (posy * sizeScale);
            int size = 16 * sizeScale;
            return (x > newX && x < newX + size && y > newY && y < newY + size);
        }
    }
    
    protected interface ButtonCallback {
        public void onClick();
    }
    
    protected abstract class Button {
        public int posx;
        public int posy;
        public boolean clicked = false;
        public ButtonCallback callback;

        public Button(int posx, int posy, ButtonCallback callback) {
            this.posx = posx;
            this.posy = posy;
            this.callback = callback;
        }
        
        public abstract void render();
        public abstract void highlight();
        public abstract boolean hits(float x, float y);
    }
    
    protected class PlusMinusButton extends Button {
        public boolean minus;
        
        public PlusMinusButton(int posx, int posy, boolean minus, ButtonCallback callback) {
            super(posx, posy, callback);
            this.posx = posx;
            this.posy = posy;
            this.minus = minus;
            this.callback = callback;
        }
        
        
        @Override
        public void render() {
            int newX = getWindowTopLeftX() + (posx * sizeScale);
            int newY = getWindowTopLeftY() + (posy * sizeScale);
            int size = 10 * sizeScale;
            
            if (minus) {
                if (!clicked) {
                    RenderUtils.blit(textureButtonUpMinus, newX, newY, size, size, 0.0f, false);
                } else {
                    RenderUtils.blit(textureButtonDownMinus, newX, newY, size, size, 0.0f, false);
                }
            } else {
                if (!clicked) {
                    RenderUtils.blit(textureButtonUpPlus, newX, newY, size, size, 0.0f, false);
                } else {
                    RenderUtils.blit(textureButtonDownPlus, newX, newY, size, size, 0.0f, false);
                }
            }
        }
        
        @Override
        public void highlight() {
            int newX = getWindowTopLeftX() + ((posx + 1) * sizeScale);
            int newY = getWindowTopLeftY() + ((posy + 1) * sizeScale);
            int size = 8 * sizeScale;
            DrawableData.spriteBatch.setColor(1.0f, 1.0f, 1.0f, 0.1f);
            DrawableData.spriteBatch.draw(textureSolidWhite, newX, newY, size, size);
            DrawableData.spriteBatch.setColor(1.0f, 1.0f, 1.0f, 1.0f);
        }
        
        @Override
        public boolean hits(float x, float y) {
            int newX = getWindowTopLeftX() + (posx * sizeScale);
            int newY = getWindowTopLeftY() + (posy * sizeScale);
            int size = 10 * sizeScale;
            return (x > newX && x < newX + size && y > newY && y < newY + size);
        }
    }
    
    protected class BuyButton extends Button {
        public boolean visible = false;
        
        public BuyButton(int posx, int posy, ButtonCallback callback) {
            super(posx, posy, callback);
            this.posx = posx;
            this.posy = posy;
            this.callback = callback;
        }
        
        
        @Override
        public void render() {
            if(visible) {
                int newX = getWindowTopLeftX() + (posx * sizeScale);
                int newY = getWindowTopLeftY() + (posy * sizeScale);
                int sizeX = 17 * sizeScale;
                int sizeY = 9 * sizeScale;

                if (!clicked) {
                    RenderUtils.blit(textureButtonBuyUp, newX, newY, sizeX, sizeY, 0.0f, false);
                } else {
                    RenderUtils.blit(textureButtonBuyDown, newX, newY, sizeX, sizeY, 0.0f, false);
                }
                
                DrawableData.font.setScale(Math.max(Math.round(getFontScale()/2.0f), 0.5f));
                RenderUtils.blitText((sizeScale > 2) ? "Upgrade" : "Buy",
                        newX + sizeX/2,
                        newY + sizeY/2,
                        RenderUtils.TextAlligment.MIDDLE,
                        RenderUtils.TextAlligment.MIDDLE);
            }
        }
        
        @Override
        public void highlight() {
            if(visible) {
                int newX = getWindowTopLeftX() + ((posx + 1) * sizeScale);
                int newY = getWindowTopLeftY() + ((posy + 1) * sizeScale);
                int sizeX = 15 * sizeScale;
                int sizeY = 7 * sizeScale;
                DrawableData.spriteBatch.setColor(1.0f, 1.0f, 1.0f, 0.1f);
                DrawableData.spriteBatch.draw(textureSolidWhite, newX, newY, sizeX, sizeY);
                DrawableData.spriteBatch.setColor(1.0f, 1.0f, 1.0f, 1.0f);
            }
        }
        
        @Override
        public boolean hits(float x, float y) {
            if(visible) {
                int newX = getWindowTopLeftX() + (posx * sizeScale);
                int newY = getWindowTopLeftY() + (posy * sizeScale);
                int sizeX = 15 * sizeScale;
                int sizeY = 7 * sizeScale;
                return (x > newX && x < newX + sizeX && y > newY && y < newY + sizeY);
            } else {
                return false;
            }
        }
    }
}
