package com.pancik.ciernypetrzlen.engine.gui;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.graphics.g2d.TextureRegion;
import com.pancik.ciernypetrzlen.DrawableData;
import com.pancik.ciernypetrzlen.engine.Engine;
import com.pancik.ciernypetrzlen.engine.component.entity.Interactable;
import static com.pancik.ciernypetrzlen.engine.gui.UIWindow.findScale;
import com.pancik.ciernypetrzlen.engine.player.Player;
import com.pancik.ciernypetrzlen.util.HorizontalAlignment;
import com.pancik.ciernypetrzlen.util.RenderUtils;
import com.pancik.ciernypetrzlen.util.VerticalAlignment;

public class TargetInfo extends UIWindow {
    private static final float SCALE_COEF = 4.0f;
    private TextureRegion textureBar = new TextureRegion(DrawableData.textureAtlas.findRegion("gui-target-bar"), 0, 0, 64, 16);
    
    private Interactable interactable;
    
    public TargetInfo(Player player, Engine.Controls engineControls) {
        super(player, engineControls, 64, 16);
        sizeScale = findScale((int)(windowSizeX * SCALE_COEF), (int)(windowSizeY * SCALE_COEF), Gdx.graphics.getWidth(), Gdx.graphics.getHeight());
    }
    
    @Override
    public void resize(int sizeX, int sizeY) {
        sizeScale = findScale((int)(windowSizeX * SCALE_COEF), (int)(windowSizeY * SCALE_COEF), sizeX, sizeY);
    }
    
    @Override
    protected int getWindowTopLeftX() {
        return Gdx.graphics.getWidth() / 2 - (windowSizeX/2 * sizeScale);
    }

    @Override
    protected int getWindowTopLeftY() {
        return 10 * sizeScale;
    }
    
    public void setInteractable(Interactable interactable) {
        this.interactable = interactable;
        
        if(interactable == null) {
            setVisibility(false);
        } else {
            setVisibility(true);
        }
    }

    @Override
    public void tick() {
        if(visible) {
        }
    }

    @Override
    public void renderUI() {
        if(visible && interactable != null && interactable.getName() != null) {
            int topLeftX = getWindowTopLeftX();
            int topLeftY = getWindowTopLeftY();
            
            RenderUtils.blit(textureBar,
                    HorizontalAlignment.MIDDLE, 0,
                    VerticalAlignment.TOP, 10 * sizeScale,
                    64 * sizeScale, 16 * sizeScale,
                    0.0f,
                    false);
            
            float health = interactable.getPercentualHealth();
            if(health < 0.0f) {
                health = 0.0f;
            }
            TextureRegion region = DrawableData.textureAtlas.findRegion("gui-target-bar");
            DrawableData.spriteBatch.draw(region.getTexture(),
                    topLeftX + 2 * sizeScale, topLeftY + 2 * sizeScale, 0, 0,
                    60.0f * health * (float)sizeScale, 12 * sizeScale, 1.0f, 1.0f, 0.0f, region.getRegionX(), region.getRegionY() + 16, (int)(60.0f * health), 12,
                    false, true);
            
            DrawableData.font.setScale(getFontScale());
            RenderUtils.blitText(interactable.getName(),
                    topLeftX + 32 * sizeScale,
                    topLeftY + 8 * sizeScale,
                    RenderUtils.TextAlligment.MIDDLE, RenderUtils.TextAlligment.MIDDLE);
        }
    }
}
