package com.pancik.ciernypetrzlen.engine.gui;

import com.badlogic.gdx.Gdx;
import com.pancik.ciernypetrzlen.engine.Engine;
import static com.pancik.ciernypetrzlen.engine.gui.InventoryWindow.findScale;
import com.pancik.ciernypetrzlen.engine.player.Player;

public abstract class UIWindow {
    protected Engine.Controls engineControls;
    protected Player player;
    protected boolean visible = false;
    protected int sizeScale;
    protected int windowSizeX;
    protected int windowSizeY;

    public UIWindow(Player player, Engine.Controls engineControls, int windowSizeX, int windowSizeY) {
        this.engineControls = engineControls;
        this.player = player;
        this.windowSizeX = windowSizeX;
        this.windowSizeY = windowSizeY;
        this.sizeScale = findScale(windowSizeX, windowSizeY, Gdx.graphics.getWidth(), Gdx.graphics.getHeight());
    }

    public void resize(int sizeX, int sizeY) {
        sizeScale = findScale(windowSizeX, windowSizeY, sizeX, sizeY);
    }

    public boolean isVisible() {
        return visible;
    }

    public void setVisibility(boolean visible) {
        this.visible = visible;
    }

    public static int findScale(int windowSizeX, int windowSizeY, int width, int height) {
        int scaleX = Math.max((width - 100) / windowSizeX, 1);
        int scaleY = Math.max((height - 100) / windowSizeY, 1);
        return Math.min(scaleX, scaleY);
    }

    protected int getWindowTopLeftX() {
        return Gdx.graphics.getWidth() / 2 - (windowSizeX/2 * sizeScale);
    }

    protected int getWindowTopLeftY() {
        return Gdx.graphics.getHeight() / 2 - (windowSizeY/2 * sizeScale);
    }
    
    protected float getFontScale() {
        float fontSize = 0.5f;
        if (sizeScale > 1 && sizeScale < 4) {
            fontSize = 1.0f;
        } else if (sizeScale >= 4) {
            fontSize = (int) (0.5f * sizeScale);
        }
        return fontSize;
    }
    
    public abstract void tick();
    public abstract void renderUI();
}
