package com.pancik.ciernypetrzlen.engine.gui;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.Graphics;
import com.badlogic.gdx.InputAdapter;
import com.badlogic.gdx.graphics.g2d.TextureRegion;
import com.badlogic.gdx.math.MathUtils;
import com.badlogic.gdx.utils.Disposable;
import com.pancik.ciernypetrzlen.DrawableData;
import com.pancik.ciernypetrzlen.MainApplicationListener;
import com.pancik.ciernypetrzlen.screens.MainMenuScreen;
import com.pancik.ciernypetrzlen.util.HorizontalAlignment;
import com.pancik.ciernypetrzlen.util.RenderUtils;
import com.pancik.ciernypetrzlen.util.VerticalAlignment;
import java.util.ArrayList;

public class WonGameWindow extends UIWindow implements Disposable {
    private TextureRegion textureButtonUp = new TextureRegion(DrawableData.textureAtlas.findRegion("gui-menu-button"), 0, 0, 128, 32);
    private TextureRegion textureButtonDown = new TextureRegion(DrawableData.textureAtlas.findRegion("gui-menu-button"), 0, 32, 128, 32);
    private TextureRegion textureSolidWhite = DrawableData.textureAtlas.findRegion("solid-white");
    private InputHandler inputHandler;
    
    private MainApplicationListener.Controls mainControls;
    private ArrayList<Button> buttons = new ArrayList<Button>();
    private boolean mousePressed = false;
    
    public WonGameWindow(MainApplicationListener.Controls mainControls) {
        super(null, null, 128, 192);
        this.mainControls = mainControls;
        this.inputHandler = new InputHandler();
        mainControls.getInputMultiplexer().addProcessor(0, inputHandler);
        
        buttons.add(new MenuButton(0, 64 + 48, "Back to the menu", new ButtonCallback() {

            @Override
            public void onClick() {
                WonGameWindow.this.mainControls.setScreen(new MainMenuScreen(WonGameWindow.this.mainControls), true);
            }
        }));
    }
    
    @Override
    public void tick() {
        if(visible) {
        }
    }
    
    @Override
    public void renderUI() {
        if(visible) {
            float mouseX = Gdx.input.getX();
            float mouseY = Gdx.input.getY();
            
            DrawableData.font.setScale(getFontScale() * 2.0f);
            RenderUtils.blitText("CONGRATULATIONS, YOU HAVE WON THE GAME!",
                    getWindowTopLeftX() + ( 64 * sizeScale), getWindowTopLeftY() + ((0 + 48) * sizeScale),
                    RenderUtils.TextAlligment.MIDDLE, RenderUtils.TextAlligment.BOTTOM);
            RenderUtils.blitText("Programing + UI Design(lol): Juraj Pancik",
                    getWindowTopLeftX() + ( 64 * sizeScale), getWindowTopLeftY() + ((16 + 48) * sizeScale),
                    RenderUtils.TextAlligment.MIDDLE, RenderUtils.TextAlligment.BOTTOM);
            RenderUtils.blitText("Graphical Artist: Zuzana Schwarzova",
                    getWindowTopLeftX() + ( 64 * sizeScale), getWindowTopLeftY() + ((32 + 48) * sizeScale),
                    RenderUtils.TextAlligment.MIDDLE, RenderUtils.TextAlligment.BOTTOM);
            RenderUtils.blitText("Level Design + Sound + Web Site: Martin Zilak",
                    getWindowTopLeftX() + ( 64 * sizeScale), getWindowTopLeftY() + ((48 + 48) * sizeScale),
                    RenderUtils.TextAlligment.MIDDLE, RenderUtils.TextAlligment.BOTTOM);
            
            for(Button b : buttons) {
                if(mousePressed && b.hits(mouseX, mouseY)) {
                    b.clicked = true;
                } else {
                    b.clicked = false;
                }

                b.render();

                if(!mousePressed && b.hits(mouseX, mouseY)) {
                    b.highlight();
                }
            }
        }
    }
    
    @Override
    public void dispose() {
        mainControls.getInputMultiplexer().removeProcessor(inputHandler);
    }
    
    private class InputHandler extends InputAdapter {

        @Override
        public boolean touchDown(int screenX, int screenY, int pointer, int button) {
            if(visible) {
                mousePressed = true;
            }

            return false;
        }
        
        @Override
        public boolean touchUp(int screenX, int screenY, int pointer, int button) {
            if(visible) {
                if(mousePressed) {
                    for(Button b : buttons) {
                        if(b.hits(screenX, screenY) && !b.disabled) {
                            b.callback.onClick();
                        }
                    }
                }

                mousePressed = false;
            }
            return false;
        }
    }
    
    protected interface ButtonCallback {
        public void onClick();
    }
    
    protected abstract class Button {
        public int posx;
        public int posy;
        public boolean clicked = false;
        public boolean disabled = false;
        public ButtonCallback callback;

        public Button(int posx, int posy, ButtonCallback callback) {
            this.posx = posx;
            this.posy = posy;
            this.callback = callback;
        }
        
        public abstract void render();
        public abstract void highlight();
        public abstract boolean hits(float x, float y);
    }
    
    protected class MenuButton extends Button {
        private static final int SIZE_X = 128;
        private static final int SIZE_Y = 32;
        public String text;
        
        public MenuButton(int posx, int posy, String text, ButtonCallback callback) {
            super(posx, posy, callback);
            this.posx = posx;
            this.posy = posy;
            this.text = text;
            this.callback = callback;
        }
        
        
        @Override
        public void render() {
            int newX = getWindowTopLeftX() + (posx * sizeScale);
            int newY = getWindowTopLeftY() + (posy * sizeScale);
            int sizeX = SIZE_X * sizeScale;
            int sizeY = SIZE_Y * sizeScale;

            if (!clicked && !disabled) {
                RenderUtils.blit(textureButtonUp, newX, newY, sizeX, sizeY, 0.0f, false);
            } else {
                RenderUtils.blit(textureButtonDown, newX, newY, sizeX, sizeY, 0.0f, false);
            }

            DrawableData.font.setScale(getFontScale() * 2.0f);
            RenderUtils.blitText(text,
                    newX + sizeX / 2,
                    newY + sizeY / 2,
                    RenderUtils.TextAlligment.MIDDLE,
                    RenderUtils.TextAlligment.MIDDLE);
        }

        @Override
        public void highlight() {
            if(!disabled) {
                int newX = getWindowTopLeftX() + ((posx + 2) * sizeScale);
                int newY = getWindowTopLeftY() + ((posy + 2) * sizeScale);
                int sizeX = (SIZE_X - 4) * sizeScale;
                int sizeY = (SIZE_Y - 4) * sizeScale;
                DrawableData.spriteBatch.setColor(1.0f, 1.0f, 1.0f, 0.1f);
                DrawableData.spriteBatch.draw(textureSolidWhite, newX, newY, sizeX, sizeY);
                DrawableData.spriteBatch.setColor(1.0f, 1.0f, 1.0f, 1.0f);
            }
        }

        @Override
        public boolean hits(float x, float y) {
            int newX = getWindowTopLeftX() + (posx * sizeScale);
            int newY = getWindowTopLeftY() + (posy * sizeScale);
            int sizeX = SIZE_X * sizeScale;
            int sizeY = SIZE_Y * sizeScale;
            return ((x > newX && x < newX + sizeX && y > newY && y < newY + sizeY) && !disabled);
        }
    }
}
