package com.pancik.ciernypetrzlen.engine.pathfinding;

import com.badlogic.gdx.math.Vector2;
import com.badlogic.gdx.utils.Array;

public class CollisionMap {
    private static final Array<Cell> neighbours = new Array();
    private final boolean[][] tiles;
    private final boolean[][] debris;

    public CollisionMap(boolean[][] tiles) {
        this.tiles = tiles;
        
        debris = new boolean[tiles.length][tiles[0].length];
    }

    public Cell getCell(float x, float z) {
        Cell cell = new TileCell();
        cell.getPosition().set((int) x + 0.5f, (int) z + 0.5f);
        return cell;
    }

    public boolean isWalkable(float fx, float fy) {
        int x = (int) fx;
        int y = (int) fy;
        if (x < 0) {
            return false;
        }
        if (x >= this.tiles.length) {
            return false;
        }
        if (y < 0) {
            return false;
        }
        if (y >= this.tiles[0].length) {
            return false;
        }
        return this.tiles[x][y];
    }

    public boolean isWalkable(Vector2 newPos) {
        return isWalkable(newPos.x, newPos.y);
    }
    
    public boolean isDebris(float fx, float fy) {
        int x = (int) fx;
        int y = (int) fy;
        if (x < 0) {
            return false;
        }
        if (x >= this.debris.length) {
            return false;
        }
        if (y < 0) {
            return false;
        }
        if (y >= this.debris[0].length) {
            return false;
        }
        return this.debris[x][y];
    }

    public boolean isDebris(Vector2 newPos) {
        return isDebris(newPos.x, newPos.y);
    }
    
    public void setWalkable(float x, float y, boolean val) {
        this.tiles[(int)x][(int)y] = val;
        this.debris[(int)x][(int)y] = !val;
    }

    private class TileCell
            implements Cell {

        private final Vector2 position = new Vector2();

        private TileCell() {
        }

        @Override
        public Vector2 getPosition() {
            return this.position;
        }

        @Override
        public int hashCode() {
            int prime = 31;
            int result = 1;
            result = 31 * result + getOuterType().hashCode();
            result = 31 * result + (this.position == null ? 0 : this.position.hashCode());
            return result;
        }

        @Override
        public boolean equals(Object obj) {
            if (this == obj) {
                return true;
            }
            if (obj == null) {
                return false;
            }
            if (getClass() != obj.getClass()) {
                return false;
            }
            TileCell other = (TileCell) obj;
            if (!getOuterType().equals(other.getOuterType())) {
                return false;
            }
            if (this.position == null) {
                if (other.position != null) {
                    return false;
                }
            } else if (!this.position.equals(other.position)) {
                return false;
            }
            return true;
        }

        @Override
        public Iterable<Cell> getNeighbours() {
            CollisionMap.neighbours.clear();
            int centerX = (int) getPosition().x;
            int centerZ = (int) getPosition().y;

            if (isWalkable(centerX, centerZ - 1)) {
                CollisionMap.neighbours.add(CollisionMap.this.getCell(centerX + 0.5f, centerZ - 1 + 0.5f));
            }

            if (isWalkable(centerX + 1, centerZ)) {
                CollisionMap.neighbours.add(CollisionMap.this.getCell(centerX + 1 + 0.5f, centerZ + 0.5f));
            }

            if (isWalkable(centerX, centerZ + 1)) {
                CollisionMap.neighbours.add(CollisionMap.this.getCell(centerX + 0.5f, centerZ + 1 + 0.5f));
            }

            if (isWalkable(centerX - 1, centerZ)) {
                CollisionMap.neighbours.add(CollisionMap.this.getCell(centerX - 1 + 0.5f, centerZ + 0.5f));
            }

            return CollisionMap.neighbours;
        }

        private boolean isWalkable(int x, int y) {
            if (x < 0) {
                return false;
            }
            if (x > CollisionMap.this.tiles.length - 1) {
                return false;
            }
            if (y < 0) {
                return false;
            }
            if (y > CollisionMap.this.tiles[0].length - 1) {
                return false;
            }
            return CollisionMap.this.tiles[x][y];
        }

        @Override
        public String toString() {
            return "TileCell [position=" + this.position + "]";
        }

        private CollisionMap getOuterType() {
            return CollisionMap.this;
        }
    }
}