package com.pancik.ciernypetrzlen.engine.player;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.graphics.Color;
import com.badlogic.gdx.math.Vector3;
import com.pancik.ciernypetrzlen.DrawableData;
import com.pancik.ciernypetrzlen.engine.Engine;

public class DamageText extends TextEffect {
    private static final int lifeLength = 180;
    
    protected Color color;
    protected Vector3 cameraPos = new Vector3();
    protected int life = 0;
    
    public DamageText(Engine.Controls engineControls, Vector3 gamePosition, String text, Color color) {
        super(engineControls, gamePosition, text);
        this.color = color;
    }

    @Override
    public void tick() {
        life++;
        
        gamePosition.z -= 0.01f;
        cameraPos.set(gamePosition);
        engineControls.getCamera().project(cameraPos);
        cameraPos.set(cameraPos.x, Gdx.graphics.getHeight() - cameraPos.y, 0.0f);
        cameraPos.sub(bounds.width/2, bounds.height/2, 0.0f);
    }

    @Override
    public void renderUI() {
        DrawableData.font.setScale(2.0f);
        DrawableData.font.setColor(Color.BLACK);
        DrawableData.font.draw(DrawableData.spriteBatch, text,
                cameraPos.x,
                cameraPos.y + 2.0f);
        DrawableData.font.setColor(color);
        DrawableData.font.draw(DrawableData.spriteBatch, text,
                cameraPos.x,
                cameraPos.y);
    }

    @Override
    public boolean remove() {
        return (life >= lifeLength);
    }

    @Override
    public void onRemove() {
    }

    @Override
    public boolean shouldCheckCollision() {
        return true;
    }

    @Override
    public Vector3 getCameraPosition() {
        return cameraPos;
    }

    @Override
    public void setPos(float x, float y) {
        cameraPos.set(x, y, cameraPos.z);
    }
}
