package com.pancik.ciernypetrzlen.engine.player;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.Input;
import com.badlogic.gdx.InputAdapter;
import com.badlogic.gdx.graphics.Color;
import com.badlogic.gdx.graphics.PerspectiveCamera;
import com.badlogic.gdx.input.GestureDetector;
import com.badlogic.gdx.math.Intersector;
import com.badlogic.gdx.math.Plane;
import com.badlogic.gdx.math.Vector2;
import com.badlogic.gdx.math.Vector3;
import com.badlogic.gdx.math.collision.Ray;
import com.badlogic.gdx.utils.Disposable;
import com.pancik.ciernypetrzlen.DrawableData;
import com.pancik.ciernypetrzlen.engine.Engine;
import com.pancik.ciernypetrzlen.engine.IDObject;
import com.pancik.ciernypetrzlen.engine.PersistentData;
import com.pancik.ciernypetrzlen.engine.SoundData;
import com.pancik.ciernypetrzlen.engine.component.entity.Attackable;
import com.pancik.ciernypetrzlen.engine.component.entity.Entity;
import com.pancik.ciernypetrzlen.engine.component.entity.Hero;
import com.pancik.ciernypetrzlen.engine.component.entity.Interactable;
import com.pancik.ciernypetrzlen.engine.component.entity.Unit;
import com.pancik.ciernypetrzlen.engine.component.spell.Spell;
import com.pancik.ciernypetrzlen.engine.gui.ActionBar;
import com.pancik.ciernypetrzlen.engine.gui.CharacterWindow;
import com.pancik.ciernypetrzlen.engine.gui.GameMenuWindow;
import com.pancik.ciernypetrzlen.engine.gui.InventoryWindow;
import com.pancik.ciernypetrzlen.engine.gui.TargetInfo;
import com.pancik.ciernypetrzlen.engine.player.inventory.Item;

public class Player extends IDObject implements Disposable {
    protected static final Vector2 tmp = new Vector2();
    
    protected Engine.Controls engineControls;
    protected PersistentData persistentData;
    protected TextEffectManager textEffectManager;
    
    protected float minZoom;
    protected float maxZoom;
    protected Vector2 mousePos;
    protected int mousePointer;
    protected final Vector3 intersection = new Vector3();
    protected final Plane plane = new Plane(new Vector3(0.0f, 1.0f, 0.0f), 0.0f);

    protected InputHandler inputHandler;
    protected GestureDetector gestureDetector;

    protected StatsPack statsPack;
    protected Hero hero;
    protected Interactable highlighted;
    protected Spell spell = null;
    
    protected ActionBar actionBar;
    protected InventoryWindow inventory;
    protected YellText currentYell = null;
    protected CharacterWindow character;
    protected TargetInfo targetInfo;
    protected GameMenuWindow gameMenuWindow;
    
    public Player(Engine.Controls engineControls, PersistentData data) {
        this.engineControls = engineControls;
        this.persistentData = data;
        textEffectManager = new TextEffectManager();
        
        this.minZoom = 6.666667f;
        this.maxZoom = 10.0f;
        
        if(data.statsPack != null) {
            this.statsPack = data.statsPack;
        } else {
            this.statsPack = new StatsPack();
        }
        this.hero = new Hero(engineControls.getLevel().getSpawnPoint().cpy(), 1, statsPack, engineControls);
        if(this.statsPack.getCurrentHealth() > 0) {
            this.hero.setHealth(this.statsPack.getCurrentHealth());
        }
        if(this.statsPack.getCurrentMana() > 0) {
            this.hero.setMana(this.statsPack.getCurrentMana());
        }
        this.hero.setActionListener(new Unit.ActionListener() {

            @Override
            public void damageDealtAttack(int damage, Attackable attackable) {
                Vector3 position = new Vector3();
                position.set(attackable.getPosition().x, 0.0f, attackable.getPosition().y - 1.0f);
                textEffectManager.addTextEffect(new DamageText(Player.this.engineControls,
                        position.cpy(),
                        Integer.toString(damage),
                        Color.WHITE));
            }

            @Override
            public void damageRecieved(int damage, Attackable attackable) {
                //System.out.println("Recieved: " + damage + " damage.");
            }

            @Override
            public void damageDealtSpell(int damage, Attackable attackable) {
                Vector3 position = new Vector3();
                position.set(attackable.getPosition().x, 0.0f, attackable.getPosition().y - 1.0f);
                textEffectManager.addTextEffect(new DamageText(Player.this.engineControls,
                        position.cpy(),
                        Integer.toString(damage),
                        Color.YELLOW));
            }
        });
        engineControls.addEntity(this.hero);
        
        inputHandler = new InputHandler();
        engineControls.getMainControls().getInputMultiplexer().addProcessor(0, inputHandler);
        gestureDetector = new GestureDetector(new GestureHandler());
        engineControls.getMainControls().getInputMultiplexer().addProcessor(0, gestureDetector);
        
        this.inventory = new InventoryWindow(this, engineControls, data.invetoryData);
        this.character = new CharacterWindow(this, engineControls);
        this.actionBar = new ActionBar(this, engineControls, data.actionBarData);
        this.character.setActionBarSlots(actionBar.getSlots());
        this.targetInfo = new TargetInfo(this, engineControls);
        this.actionBar.setVisibility(true);
        this.gameMenuWindow = new GameMenuWindow(this, engineControls);
    }
    
    public void tick() {
        if(!hero.isDead()) {
            if(spell != null && spell.getType() == Spell.Type.SELF) {
                if(hero.getMana() >= spell.getManaCost()) {
                    spell.cast(engineControls, hero);
                    hero.changeMana(-spell.getManaCost());
                }
                hero.onInteract();
                spell = null;
            }
        }
        
        float closestResult = 1234567890;
        Interactable closestEntity = null;
        Vector3 cameraPos = engineControls.getCamera().position;
        Ray cameraPickRay = engineControls.getCamera().getPickRay(Gdx.input.getX(), Gdx.input.getY());
        for(Entity e : engineControls.getEntityManager().getEntities()) {
            if(e instanceof Interactable && e != hero) {
                Interactable h = (Interactable)e;
                
                if(h.getClosestIntersection(cameraPickRay, intersection)) {
                    float res = cameraPos.dst(intersection);
                    if(cameraPos.dst(intersection) < closestResult) {
                        closestResult = res;
                        closestEntity = h;
                    }
                }
            }
        }
        if(highlighted != null) {
            highlighted.dehighlight();
        }
        if(closestEntity != null) {
            highlighted = closestEntity;
            highlighted.highlight();
        } else {
            highlighted = null;
        }
        targetInfo.setInteractable(highlighted);
        
        if(mousePos != null) {
            hero.setPath(null);
            hero.setTargetPosition(null);
            boolean move = true;
            
            if(highlighted != null) {
                if(highlighted instanceof Attackable) {
                    Attackable a = (Attackable) highlighted;
                    if(spell != null && spell.getType() == Spell.Type.TARGET && spell.inRange(hero.getPosition(), a.getPosition()) && hero.getMana() >= spell.getManaCost()) {
                        spell.cast(engineControls, hero, a);
                        hero.changeMana(-spell.getManaCost());
                        hero.onInteract();
                        spell = null;
                    } else {
                        if(hero.isInRangeForAttack(a) && a.getTeam() != hero.getTeam()) {
                            hero.attack(a);
                            move = false;
                        }
                    }
                }
            }
            
            if(move) {
                Intersector.intersectRayPlane(engineControls.getCamera().getPickRay(mousePos.x, mousePos.y), plane, intersection);

                hero.moveTo(intersection.x, intersection.z);
            }
        }
        
        engineControls.focusCamera(hero.getPosition());
        
        textEffectManager.tick();
    }
    
    public void tickUI() {
        targetInfo.tick();

        actionBar.tick();
        
        inventory.tick();
        
        character.tick();
        
        gameMenuWindow.tick();
    }
    
    public void renderUI() {
        textEffectManager.renderUI();
        
        targetInfo.renderUI();
        
        actionBar.renderUI();
        
        inventory.renderUI();
        
        character.renderUI();
        
        gameMenuWindow.renderUI();
    }
    
    public Hero getHero() {
        return hero;
    }
    
    protected class InputHandler extends InputAdapter {

        @Override
        public boolean keyDown(int keycode) {
            if(keycode == Input.Keys.E) {
                if(!character.isVisible() && !gameMenuWindow.isVisible()) {
                    if(inventory.isVisible()) {
                        hideUI();
                        mousePos = null;
                    } else {
                        showInventory();
                        mousePos = null;
                    }
                }
            }
            if(keycode == Input.Keys.W) {
                if(!inventory.isVisible() && !gameMenuWindow.isVisible()) {
                    if(character.isVisible()) {
                        hideUI();
                        mousePos = null;
                    } else {
                        showCharacter();
                        mousePos = null;
                    }
                }
            }
            if(keycode == Input.Keys.ESCAPE) {
                if(character.isVisible() || inventory.isVisible() || gameMenuWindow.isVisible()) {
                    hideUI();
                } else {
                    showGameMenu();
                }
                mousePos = null;
            }
            
            return false;
        }

        @Override
        public boolean keyUp(int keycode) {
            return false;
        }

        @Override
        public boolean touchDown(int screenX, int screenY, int pointer, int button) {
            if(!inventory.isVisible() && !character.isVisible()) {
                if(button == Input.Buttons.LEFT && !hero.isDead()) {
                    if (highlighted != null) {
                        if (highlighted.getPosition().dst(hero.getPosition()) < hero.getInteractRange()
                                && (!(highlighted instanceof Attackable) || ((Attackable)highlighted).getTeam() == hero.getTeam())) {
                            highlighted.interact();
                            hero.onInteract();
                            return true;
                        }
                    }

                    mousePos = new Vector2(screenX, screenY);
                    mousePointer = pointer;
                    if(spell != null) {
                        Intersector.intersectRayPlane(engineControls.getCamera().getPickRay(mousePos.x, mousePos.y), plane, intersection);
                        tmp.set(intersection.x, intersection.z);
                        if ( spell.getType() == Spell.Type.POSITION && spell.inRange(hero.getPosition(), tmp) && hero.getMana() >= spell.getManaCost()) {
                            spell.cast(engineControls, hero, tmp);
                            hero.changeMana(-spell.getManaCost());
                            hero.onInteract();
                            spell = null;
                        }
                    }
                    return true;
                }
            }
            return false;
        }

        @Override
        public boolean touchUp(int screenX, int screenY, int pointer, int button) {
            if(!inventory.isVisible() && !character.isVisible()) {
                if(!hero.isDead()) {
                    if(button == Input.Buttons.LEFT) {
                        if(mousePos != null && mousePointer == pointer) {
                            mousePos = null;
                            return true;
                        }
                    }
                } else {
                    engineControls.reload();
                }
            }
            return false;
        }

        @Override
        public boolean touchDragged(int screenX, int screenY, int pointer) {
            if(!inventory.isVisible() && !character.isVisible()) {
                if(mousePos != null && mousePointer == pointer) {
                    mousePos.set(screenX, screenY);
                    return true;
                }
            }
            return false;
        }

        @Override
        public boolean scrolled(int amount) {
            if(!inventory.isVisible() && !character.isVisible()) {
                PerspectiveCamera camera = engineControls.getCamera();

                float delta = amount;
                if (camera.position.y + delta > maxZoom) {
                    delta = maxZoom - camera.position.y;
                }
                if (camera.position.y + delta < minZoom) {
                    delta = minZoom - camera.position.y;

                }
                camera.position.y += delta;
            }
            return false;
        }
    }
    
    protected class GestureHandler extends GestureDetector.GestureAdapter {

        @Override
        public boolean tap(float x, float y, int count, int button) {
            if(!inventory.isVisible() && !character.isVisible()) {
                if(button == Input.Buttons.RIGHT) {
                    Intersector.intersectRayPlane(engineControls.getCamera().getPickRay(x, y), plane, intersection);
                    hero.setTargetPosition(new Vector2(intersection.x, intersection.z));

                    return true;
                }
            }
            return false;
        }
    }
    
    public void setLevelAndSaveExit(String level) {
        persistentData.currentLevel = level;
        statsPack.setCurrentHealth(hero.getHealth());
        statsPack.setCurrentMana(hero.getMana());
        persistentData.statsPack = statsPack;
        persistentData.invetoryData = inventory.getPersistenceData();
        persistentData.actionBarData = actionBar.getPersistenceData();
        persistentData.save();
        engineControls.reload();
    }
    
    @Override
    public void dispose() {
        engineControls.getMainControls().getInputMultiplexer().removeProcessor(inputHandler);
        engineControls.getMainControls().getInputMultiplexer().removeProcessor(gestureDetector);
        
        inventory.dispose();
        character.dispose();
        actionBar.dispose();
        gameMenuWindow.dispose();
    }
    
    public void resize(int sizeX, int sizeY) {
        inventory.resize(sizeX, sizeY);
        
        character.resize(sizeX, sizeY);
        
        actionBar.resize(sizeX, sizeY);
        
        targetInfo.resize(sizeX, sizeY);
        
        gameMenuWindow.resize(sizeX, sizeY);
    }
    
    public boolean addInventoryItem(Item item) {
        boolean added = inventory.addItemToInventory(item);
        
        if(!added) {
            if(currentYell != null) {
                currentYell.destroy();
            }
            
            Vector3 yellPosition = new Vector3();
            Vector2 heroPos = hero.getPosition();
            if(heroPos != null) {
                yellPosition.set(heroPos.x, 0.0f, heroPos.y - 1.0f);
                currentYell = new YellText(engineControls,
                        yellPosition,
                        "My inventory is full.",
                        Color.WHITE);
                engineControls.getPlayer().getTextEffectManager().addTextEffect(currentYell);
            }
        } else {
            SoundData.playSound("item-inventory", 1.0f);
        }
        
        return added;
    }
    
    public void yellLevelUp() {
        if (currentYell != null) {
            currentYell.destroy();
        }

        Vector3 yellPosition = new Vector3();
        Vector2 heroPos = hero.getPosition();
        if (heroPos != null) {
            yellPosition.set(heroPos.x, 0.0f, heroPos.y - 1.0f);
            currentYell = new YellText(engineControls,
                    yellPosition,
                    "I feel stronger...",
                    Color.WHITE);
            engineControls.getPlayer().getTextEffectManager().addTextEffect(currentYell);
        }
    }

    public StatsPack getStatsPack() {
        return statsPack;
    }
    
    public InventoryWindow getInventory() {
        return inventory;
    }
    
    public CharacterWindow getCharacterWindow() {
        return character;
    }
    
    public ActionBar getActionBar() {
        return actionBar;
    }
    
    public void castSpell(Spell spell) {
        this.spell = spell;
    }
    
    public Spell getCastedSpell() {
        return spell;
    }
    
    public TextEffectManager getTextEffectManager() {
        return textEffectManager;
    }
    
    public boolean isDead() {
        return hero.isDead();
    }
    
    public void showInventory() {
        inventory.setVisibility(true);
        character.setVisibility(false);
        gameMenuWindow.setVisibility(false);
        actionBar.setVisibility(false);
        targetInfo.setVisibility(false);
        engineControls.pauseGame(true);
    
    }
    
    public void showCharacter() {
        inventory.setVisibility(false);
        character.setVisibility(true);
        gameMenuWindow.setVisibility(false);
        actionBar.setVisibility(false);
        targetInfo.setVisibility(false);
        engineControls.pauseGame(true);
    }
    
    public void showGameMenu() {
        inventory.setVisibility(false);
        character.setVisibility(false);
        gameMenuWindow.setVisibility(true);
        actionBar.setVisibility(false);
        targetInfo.setVisibility(false);
        engineControls.pauseGame(true);
    }
    
    public void hideUI() {
        inventory.setVisibility(false);
        character.setVisibility(false);
        gameMenuWindow.setVisibility(false);
        actionBar.setVisibility(true);
        targetInfo.setVisibility(true);
        engineControls.pauseGame(false);
    }
}
