package com.pancik.ciernypetrzlen.engine.player;

import com.badlogic.gdx.math.MathUtils;
import com.pancik.ciernypetrzlen.engine.component.spell.Spell;
import java.util.HashMap;

public class StatsPack {
    private int level = 1;
    private int pointsFree = 3;
    private int pointsAttack = 0;
    private int pointsDefense = 0;
    private int pointsVitality = 0;
    private int equipmentPointsAttack = 0;
    private int equipmentPointsDefense = 0;
    private int equipmentPointsVitality = 0;
    private int currentExperience = 0;
    private int requiredExperience = 10;
    private int gold = 0;
    private HashMap<String, Integer> spellLevels = new HashMap<String, Integer>();
    private int currentHealth = -1;
    private int currentMana = -1;

    public int getLowAttackDamage() {
        return 1 + Math.round((float)Math.pow((float)(pointsAttack + equipmentPointsAttack), 1.3f));
    }
    
    public int getHighAttackDamage() {
        return 3 + Math.round((float)Math.pow((float)(pointsAttack + equipmentPointsAttack), 1.4f));
    }
    
    public int getAttackDamage() {
        int low = getLowAttackDamage();
        int delta = getHighAttackDamage() - low;
        return low + MathUtils.random(delta);
    }
    
    public float getPercentualDefense() {
        return ((float)(pointsDefense + equipmentPointsDefense) + 1.0f)/((float)(pointsDefense + equipmentPointsDefense) + 15.0f);
    }
    
    public int getHitpoints() {
        return 200 + ((pointsVitality + equipmentPointsVitality) * 10);
    }
    
    public int getHitpointRegeneration() {
        return (int)((2.0f * (float)getHitpoints())/200.0f);
    }
    
    public int getMana() {
        return 50 + ((pointsVitality + equipmentPointsVitality) * 6);
    }
    
    public int getManaRegeneration() {
        return (int)((1.0f * (float)getHitpoints())/200.0f);
    }
    
    public float getMovementSpeedCoefficient() {
        return 1.0f;
    }
    
    public float getAttackSpeedCoefficient() {
        return 1.0f;
    }
    
    public int getCurrentExperience() {
        return currentExperience;
    }
    
    public int getRequiredExperience() {
        return requiredExperience;
    }
    
    public boolean rewardExperience(int amount) {
        currentExperience += amount;
        if(currentExperience >= requiredExperience) {
            requiredExperience *= 1.5f;
            currentExperience = 0;
            level += 1;
            pointsFree += 3;
            return true;
        }
        return false;
    }

    public int getLevel() {
        return level;
    }

    public int getGold() {
        return gold;
    }

    public void changeGold(int delta) {
        gold += delta;
    }
    
    public int getPointsFree() {
        return pointsFree;
    }

    public int getPointsAttack() {
        return pointsAttack;
    }

    public int getPointsDefense() {
        return pointsDefense;
    }

    public int getPointsVitality() {
        return pointsVitality;
    }
    
    public void changePointAttack(int delta) {
        if(delta > 0) {
            if(delta > pointsFree) {
                delta = pointsFree;
            }
            pointsFree -= delta;
            pointsAttack += delta;
        } else {
            delta = Math.abs(delta);
            if(delta > pointsAttack) {
                delta = pointsAttack;
            }
            pointsFree += delta;
            pointsAttack -= delta;
        }
    }
    
    public void changePointDefense(int delta) {
        if(delta > 0) {
            if(delta > pointsFree) {
                delta = pointsFree;
            }
            pointsFree -= delta;
            pointsDefense += delta;
        } else {
            delta = Math.abs(delta);
            if(delta > pointsDefense) {
                delta = pointsDefense;
            }
            pointsFree += delta;
            pointsDefense -= delta;
        }
    }
    
    public void changePointVitality(int delta) {
        if(delta > 0) {
            if(delta > pointsFree) {
                delta = pointsFree;
            }
            pointsFree -= delta;
            pointsVitality += delta;
        } else {
            delta = Math.abs(delta);
            if(delta > pointsVitality) {
                delta = pointsVitality;
            }
            pointsFree += delta;
            pointsVitality -= delta;
        }
    }

    public void setEquipmentPointsAttack(int equipmentPointsAttack) {
        this.equipmentPointsAttack = equipmentPointsAttack;
    }

    public void setEquipmentPointsDefense(int equipmentPointsDefense) {
        this.equipmentPointsDefense = equipmentPointsDefense;
    }

    public void setEquipmentPointsVitality(int equipmentPointsVitality) {
        this.equipmentPointsVitality = equipmentPointsVitality;
    }

    public int getEquipmentPointsAttack() {
        return equipmentPointsAttack;
    }

    public int getEquipmentPointsDefense() {
        return equipmentPointsDefense;
    }

    public int getEquipmentPointsVitality() {
        return equipmentPointsVitality;
    }
    
    public HashMap<String, Integer> getSpellLevels() {
        return spellLevels;
    }
    
    public int getCurrentHealth() {
        return currentHealth;
    }

    public int getCurrentMana() {
        return currentMana;
    }

    public void setCurrentHealth(int currentHealth) {
        this.currentHealth = currentHealth;
    }

    public void setCurrentMana(int currentMana) {
        this.currentMana = currentMana;
    }
}