package com.pancik.ciernypetrzlen.engine.player;

import com.badlogic.gdx.math.Rectangle;
import com.badlogic.gdx.math.Vector3;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;

public class TextEffectManager {
    private List<TextEffect> texts;
    
    public TextEffectManager() {
        this.texts = new ArrayList<TextEffect>();
    }
    
    public void tick() {
        for(int i = 0; i < texts.size(); i++) {
            if(!texts.get(i).remove()) {
                texts.get(i).tick();
            }
        }
        
        ArrayList<TextEffect> shouldBeRemoved = new ArrayList<TextEffect>();

        for(TextEffect t: texts) {
            if(t.remove()) {
                shouldBeRemoved.add(t);
            }
        }

        for(TextEffect t : shouldBeRemoved) {
            texts.remove(t);
        }
        
        ArrayList<TextEffect> collidable = new ArrayList<TextEffect>();
        for(int i = 0; i < texts.size(); i++) {
            if(texts.get(i).shouldCheckCollision()) {
                collidable.add(texts.get(i));
            }
        }
        
        Rectangle r1 = new Rectangle();
        Rectangle r2 = new Rectangle();
        for(int j = 0; j < collidable.size(); j++) {
            r1.set(collidable.get(j).getCameraPosition().x,
                   collidable.get(j).getCameraPosition().y + collidable.get(j).getBounds().height/2,
                   collidable.get(j).getBounds().width,
                   collidable.get(j).getBounds().height);
                    
            for(int i = 0; i < collidable.size(); i++) {
                if(i == j) {
                    continue;
                }
                
                r2.set(collidable.get(i).getCameraPosition().x,
                   collidable.get(i).getCameraPosition().y + collidable.get(i).getBounds().height/2,
                   collidable.get(i).getBounds().width,
                   collidable.get(i).getBounds().height);
                
                if (r1.contains(r2.x, r2.y) || r1.contains(r2.x, r2.y + r2.height)) {
                    collidable.get(i).getCameraPosition().set(r1.x + r1.width, r2.y - r2.height/2, 0.0f);
                    continue;
                }
                if (r1.contains(r2.x + r2.width, r2.y) || r1.contains(r2.x + r2.width, r2.y + r2.height)) {
                    collidable.get(i).getCameraPosition().set(r1.x - r2.width, r2.y - r2.height/2, 0.0f);
                    continue;
                }
            }
        }
        
        Collections.sort(texts, new Comparator<TextEffect>() {

            @Override
            public int compare(TextEffect t, TextEffect t1) {
                return (int)Math.signum((t.shouldCheckCollision() ? 1 : 0) - (t1.shouldCheckCollision() ? 1 : 0));
            }
        });
    }
    
    public void renderUI() {
        for(TextEffect t : texts) {
            if(!t.remove()) {
                t.renderUI();
            }
        }
    }
    
    public void addTextEffect(TextEffect t) {
        texts.add(t);
    }
}
