package com.pancik.ciernypetrzlen.engine.player;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.graphics.Color;
import com.badlogic.gdx.graphics.g2d.BitmapFont;
import com.badlogic.gdx.math.Vector3;
import com.pancik.ciernypetrzlen.DrawableData;
import com.pancik.ciernypetrzlen.engine.Engine;
import com.pancik.ciernypetrzlen.util.RenderUtils;

public class YellText extends TextEffect {
    protected Color color;
    protected Vector3 cameraPos = new Vector3();
    protected int life = 0;
    protected int lifeLength = 120;
    protected int sizeCoef;
    
    public YellText(Engine.Controls engineControls, Vector3 gamePosition, String text, Color color) {
        super(engineControls, gamePosition, text);
        this.color = color;
        
        if (bounds.width < 96.0f) {
            sizeCoef = 1;
        } else if (bounds.width < 300.0f) {
            sizeCoef = 2;
        } else if (bounds.width < 750.0f) {
            sizeCoef = 3;
        } else if (bounds.width < 1200.0f) {
            sizeCoef = 4;
        } else {
            sizeCoef = 5;
        }
        lifeLength = 120 * sizeCoef;
        if(sizeCoef == 5) {
            lifeLength += 120;
        }
    }

    @Override
    public void tick() {
        life++;
        
        cameraPos.set(gamePosition);
        engineControls.getCamera().project(cameraPos);
        cameraPos.set(cameraPos.x, Gdx.graphics.getHeight() - cameraPos.y, 0.0f);
        cameraPos.sub((48.0f) * sizeCoef, (32.0f) * sizeCoef, 0.0f);
    }

    @Override
    public void renderUI() {
        RenderUtils.blit(DrawableData.textureAtlas.findRegion("gui-yell"),
                cameraPos.x, cameraPos.y, 96 * sizeCoef, 32 * sizeCoef,
                0.0f, true);
        
        DrawableData.font.setScale(2.0f);
        DrawableData.font.setColor(color);
        DrawableData.font.drawWrapped(DrawableData.spriteBatch, text,
                cameraPos.x + 6.0f,
                cameraPos.y + 6.0f,
                (96.0f * sizeCoef) - 12.0f,
                BitmapFont.HAlignment.CENTER);
    }

    @Override
    public boolean remove() {
        return (life >= lifeLength);
    }

    @Override
    public void onRemove() {
    }

    @Override
    public boolean shouldCheckCollision() {
        return false;
    }

    @Override
    public Vector3 getCameraPosition() {
        return cameraPos;
    }

    @Override
    public void setPos(float x, float y) {
        cameraPos.set(x, y, cameraPos.z);
    }
    
    public void destroy() {
        this.life = lifeLength;
    }
}
