package com.pancik.ciernypetrzlen.engine.player.inventory;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.graphics.Color;
import com.badlogic.gdx.graphics.g2d.TextureRegion;
import com.pancik.ciernypetrzlen.DrawableData;
import com.pancik.ciernypetrzlen.util.RenderUtils;

public abstract class Item {
    protected static Color GREY = new Color(0.7f, 0.7f, 0.7f, 1.0f);
    protected static Color WHITE = new Color(1.0f, 1.0f, 1.0f, 1.0f);
    protected static Color GREEN = new Color(0.0f, 0.7f, 0.0f, 1.0f);
    protected static Color PURPLE = new Color(1.0f, 0.0f, 1.0f, 1.0f);
    protected static Color BLUE = new Color(0.1f, 0.1f, 1.0f, 1.0f);
    protected static class Attribute {
        public String text;
        public Color color;

        public Attribute(String text, Color color) {
            this.text = text;
            this.color = color;
        }
    }
    
    public void render(float x, float y, float size) {
        RenderUtils.blit(getTexture(), x, y, size, size, 0.0f, false);
    }
    
    public void renderTooltip(float x, float y, int sizeScale) {
        if(sizeScale > 3) {
            sizeScale = (int)Math.max(Math.round((float)sizeScale/1.5f), 1.0f);
        }
        
        float sizeX = 96 * sizeScale;
        float sizeY = 64 * sizeScale;
        float topY = y - sizeY;
        if(topY < 5) {
            topY = 5;
        }
        if(x + sizeX > Gdx.graphics.getWidth() - 5) {
            x = Gdx.graphics.getWidth() - sizeX - 5;
        }
        float currentY = topY + (3 * sizeScale);
        float wrapX = sizeX - (10 * sizeScale);
        
        RenderUtils.blit(DrawableData.textureAtlas.findRegion("gui-tooltip"), x, topY, sizeX, sizeY, 0.0f, false);
        float scale = 0.5f;
        if((int)(0.7f * sizeScale) > 0) {
            scale = (int)(0.7f * sizeScale);
        }
        DrawableData.font.setScale(scale);
        DrawableData.font.setColor(getNameColor());
        currentY += DrawableData.font.drawWrapped(DrawableData.spriteBatch,
                getName(),
                x + (3 * sizeScale), currentY, wrapX).height;
        currentY += (3 * sizeScale);
        
        if (this instanceof Equipment) {
            DrawableData.font.setColor(new Color(222.0f/255.0f, 185.0f/255.0f, 118.0f/255.0f, 1.0f));
            currentY += DrawableData.font.drawWrapped(DrawableData.spriteBatch,
                    "<" + ((Equipment)this).getSlot().toString() + ">",
                    x + (3 * sizeScale), currentY, wrapX).height;
            currentY += (3 * sizeScale);
        }
        currentY += (2 * sizeScale);
        
        Attribute[] attributes = getAttributes();
        if(attributes != null) {
            for(Attribute a : getAttributes()) {
                DrawableData.font.setColor(a.color);
                currentY += DrawableData.font.drawWrapped(DrawableData.spriteBatch,
                    a.text,
                    x + (3 * sizeScale), currentY, wrapX).height;
                currentY += (3 * sizeScale);
            }
            currentY += (2 * sizeScale);
        }
        
        DrawableData.font.setColor(GREY);
        currentY += DrawableData.font.drawWrapped(DrawableData.spriteBatch,
            getText(),
            x + (3 * sizeScale), currentY, wrapX).height;
    }
    
    public abstract TextureRegion getTexture();
    
    protected abstract Color getNameColor();
    public abstract String getName();
    protected abstract Attribute[] getAttributes();
    protected abstract String getText();
    public abstract int getTransmuteValue();
}
