package com.pancik.ciernypetrzlen.util;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.graphics.Color;
import com.badlogic.gdx.graphics.Texture;
import com.badlogic.gdx.graphics.g2d.BitmapFont;
import com.badlogic.gdx.graphics.g2d.TextureRegion;
import com.pancik.ciernypetrzlen.DrawableData;
import com.pancik.ciernypetrzlen.MainApplicationListener;

public final class RenderUtils {
    public static enum TextAlligment {
        LEFT, MIDDLE, RIGHT, TOP, BOTTOM;
    }

    public static class ScreenInfo {
        float posx, posy, sizex, sizey;

        public ScreenInfo(int posx, int posy, int sizex, int sizey) {
            this.posx = posx;
            this.posy = posy;
            this.sizex = sizex;
            this.sizey = sizey;
        }

        public ScreenInfo(float posx, float posy, float sizex, float sizey) {
            this.posx = posx;
            this.posy = posy;
            this.sizex = sizex;
            this.sizey = sizey;
        }
    }

    public static int calculateX(HorizontalAlignment horizontal, int deltax, int sizex) {
        int width = Gdx.graphics.getWidth();

        switch(horizontal) {
            case LEFT:
                return deltax;
            case MIDDLE:
                return (width/2) + deltax - sizex/2;
            case RIGHT:
                return width + deltax - sizex;
        }
        return -1;
    }

    public static int calculateY(VerticalAlignment vertical, int deltay, int sizey) {
        int height = Gdx.graphics.getHeight();

        switch(vertical) {
            case TOP:
                return deltay;
            case MIDDLE:
                return (height/2) + deltay - sizey/2;
            case BOTTOM:
                return height + deltay - sizey;
        }
        return -1;
    }
    
    public static void blit(TextureRegion what,
            HorizontalAlignment horizontal,
            int deltax,
            VerticalAlignment vertical,
            int deltay,
            int sizex, int sizey,
            float rotation, boolean flip) {
        int calculatedX = calculateX(horizontal, deltax, sizex);
        int calculatedY = calculateY(vertical, deltay, sizey);
        
        DrawableData.spriteBatch.draw(what.getTexture(), calculatedX, calculatedY, sizex/2, sizey/2, sizex, sizey, 1, 1, rotation,
                what.getRegionX(), what.getRegionY(), what.getRegionWidth(), what.getRegionHeight(), flip, true);
    }
    
    public static void blit(TextureRegion what, float posx, float posy, float sizex, float sizey, float rotation, boolean flip) {
        DrawableData.spriteBatch.draw(what.getTexture(),
                posx, posy, sizex/2, sizey/2, sizex, sizey,
                1, 1, rotation,
                what.getRegionX(), what.getRegionY(), what.getRegionWidth(), what.getRegionHeight(), flip, true);
    }
    
    public static void blit(TextureRegion what, float posx, float posy, float sizex, float sizey,
            float textureX, float textureY, float textureWidth, float textureHeight,
            float rotation, boolean flip) {
        DrawableData.spriteBatch.draw(what.getTexture(),
                posx, posy, sizex/2, sizey/2, sizex, sizey,
                1.0f, 1.0f, rotation,
                what.getRegionX() + (int) textureX, what.getRegionY() + (int) textureY, (int) textureWidth, (int) textureHeight, flip, true);
    }
    
    public static void blitText(CharSequence str, int x, int y, TextAlligment horizontal, TextAlligment vertical) {
        blitText(str, x, y, Color.WHITE, horizontal, vertical);
    }

    public static void blitText(CharSequence str, int x, int y, Color color,TextAlligment horizontal, TextAlligment vertical) {
        BitmapFont.TextBounds tb = DrawableData.font.getBounds(str);
        float deltax = 0;
        float deltay = 0;

        switch(horizontal) {
            case LEFT:
                deltax = 0;
                break;
            case MIDDLE:
                deltax = tb.width/2;
                break;
            case RIGHT:
                deltax = tb.width;
                break;
        }

        switch(vertical) {
            case TOP:
                deltay = tb.height;
                break;
            case MIDDLE:
                deltay = tb.height/2;
                break;
            case BOTTOM:
                deltay = 0;
                break;
        }

        DrawableData.font.setColor(color);
        DrawableData.font.draw(DrawableData.spriteBatch, str, (int)(x - deltax), (int)(y - deltay));
        DrawableData.font.setColor(Color.WHITE);
    }

    public static void blitText(CharSequence str,
            HorizontalAlignment horizontal,
            int deltax,
            VerticalAlignment vertical,
            int deltay,
            TextAlligment textHorizontal,
            TextAlligment textVertical,
            Color mainColor, float colorAlpha,
            Color underColor) {

        BitmapFont.TextBounds tb = DrawableData.font.getBounds(str);
        float textDeltax = 0;
        float textDeltay = 0;

        switch(textHorizontal) {
            case LEFT:
                textDeltax = 0;
                break;
            case MIDDLE:
                textDeltax = tb.width/2;
                break;
            case RIGHT:
                textDeltax = tb.width;
                break;
        }

        switch(textVertical) {
            case TOP:
                textDeltay = tb.height;
                break;
            case MIDDLE:
                textDeltay = tb.height/2;
                break;
            case BOTTOM:
                textDeltay = 0;
                break;
        }

        int calculatedX = calculateX(horizontal, deltax, 0);
        int calculatedY = calculateY(vertical, deltay, 0);
        
        DrawableData.font.setColor(mainColor.r, mainColor.g, mainColor.b, colorAlpha);
        DrawableData.font.draw(DrawableData.spriteBatch, str, (int)(calculatedX - textDeltax), (int)(calculatedY - textDeltay));
        DrawableData.font.setColor(Color.WHITE);
    }

    public static void blitText(CharSequence str,
            HorizontalAlignment horizontal,
            int deltax,
            VerticalAlignment vertical,
            int deltay,
            TextAlligment textHorizontal,
            TextAlligment textVertical) {
        blitText(str, horizontal, deltax, vertical, deltay, textHorizontal, textVertical, Color.WHITE);
    }
    
    public static void blitText(CharSequence str,
            HorizontalAlignment horizontal,
            int deltax,
            VerticalAlignment vertical,
            int deltay,
            TextAlligment textHorizontal,
            TextAlligment textVertical,
            Color color) {
        blitText(str, horizontal, deltax, vertical, deltay, textHorizontal, textVertical, color, color.a, Color.BLACK);
    }
}